
package org.netbeans.modules.asm.x86.syntax.att.generated;

import java.util.List;

import org.netbeans.modules.asm.x86.syntax.*;
import org.netbeans.modules.asm.x86.syntax.att.*;

%%

%public
%class ATTScanner
%implements X86Scanner

%function next
%type X86Token

%char
%unicode

%{
        
   private X86Token tokenHelper(X86TokenId tok) {
        return new X86Token(tok, yytext(), yychar, 
                            yychar + yylength());
   }

%}


ALPHA = [A-Za-z]
DIGIT = [0-9]

/* 'Char' macroses */

COMMA = ","
COLON = ":"
LPAREN = "("
RPAREN = ")"
DOT = "."
MINUS = "-"

/*
DIV = "/"
STAR = "*"
LSPAREN = "["
RSPAREN = "]" */

PARAM_DELIM = {WhiteSpace}* {COMMA} {WhiteSpace}*

/* Number Literals */

PosDecIntegerLiteral = 0 | [1-9][0-9]*
DecIntegerLiteral = {MINUS}? {PosDecIntegerLiteral}

HexIntegerLiteral = 0 [xX] 0* {HexDigit} {1,8}
HexDigit          = [0-9a-fA-F]

OctIntegerLiteral = 0+ [1-3]? {OctDigit} {1,15}
OctDigit          = [0-7]

BinIntegerLiteral = 0 [bB] {BinDigit} {1,64}
BinDigit = [0-1]

IntegerLiteral = {DecIntegerLiteral} | {BinIntegerLiteral} | {OctIntegerLiteral} |
                 {HexIntegerLiteral} 
                 
    
	
/* Floating point literals */       
 
FloatLiteral  = ({FLit1}|{FLit2}|{FLit3}) {Exponent}? [fF]
DoubleLiteral = ({FLit1}|{FLit2}|{FLit3}) {Exponent}?

FLit1    = [0-9]+ \. [0-9]* 
FLit2    = \. [0-9]+ 
FLit3    = [0-9]+ 
Exponent = [eE] [+-]? [0-9]+

NumberLiteral = {IntegerLiteral} | {FloatLiteral} | {DoubleLiteral}

/* Whitespaces and comments */

LineTerminator = \r|\n|\r\n
InputCharacter = [^\r\n]
WhiteSpace = {LineTerminator} | [ \t\f]

CComment = "/*" [^*] ~"*/" | "/*" "*"+ "/"
SimpleComment = "/" {InputCharacter}* {LineTerminator}
Comment = {CComment} | {SimpleComment}

/* ID */

Ident = ({ALPHA}|_) ({ALPHA}|{DIGIT}|_)*

/* String literal ToDo: FIX */

StringLiteral = "\"" ({InputCharacter} | "\\\"")* ("\"" | LineTerminator)
CharacterLiteral = "'" ({InputCharacter} | "\\\'")* ("'" | LineTerminator) 

/* ASM ops */

SimpleReg = "%" {Ident}
RegMemOp = (({SimpleReg} {COLON})? {IntegerLiteral}?  {LPAREN} 
            ({SimpleReg} | ({SimpleReg} {PARAM_DELIM} {SimpleReg}) | 
            ({SimpleReg}? {PARAM_DELIM} {SimpleReg} {PARAM_DELIM} {IntegerLiteral})) 
             {RPAREN}
           )  | ({SimpleReg} {COLON} { IntegerLiteral })

NumberOperand = "$" {DecIntegerLiteral}		   

/* Labels */

InstLabel = ({Ident} | {PosDecIntegerLiteral}) {COLON}
Label = {Ident} | ({PosDecIntegerLiteral} [fb])					   
	
/* ASM section data */	

Section = ({DOT} {Ident})					   
					   
%% 

<YYINITIAL> {
	{Comment} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{Ident} { return tokenHelper(X86TokenId.ASM_LABEL); }
	{StringLiteral} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{SimpleReg} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{RegMemOp} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{Section} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{Label} { return tokenHelper(X86TokenId.ASM_LABEL); }
	{InstLabel} { return tokenHelper(X86TokenId.ASM_LABEL_INST); }
	{RegMemOp} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{CharacterLiteral} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{NumberLiteral} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{NumberOperand} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{COMMA} { return tokenHelper(X86TokenId.ASM_P_MARK); }
	{WhiteSpace} {  }	
        
}

. {  }
