
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.actionperformers;

import org.netbeans.api.project.Project;
import org.netbeans.modules.coverage.utils.ProjectStructureUtils;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;

/**
 * Base class for the action performer classes. The class is used when there is a need to perform some 
 * action on a node
 * @author Andrey Korostelev
 */
public abstract class ClassAbstractActionPerformer  implements Runnable{
    
    /** Creates a new instance of ClassAbstractActionPerformer */
    private String classFullName = null;
    private Node xnode = null;
    private String commandName = null;
    String outputTabName = null;
    private Project project = null;
    
    /**
     * Performs the action
     */
    public abstract void run();
    
    /**
     * Constructs the action performer class based on the node and action
     * @param xnode The node that the actions will be invoked for
     * @param commandName The name of the command
     */
    public ClassAbstractActionPerformer(Node xnode, String commandName){
        this.xnode = xnode;
        this.commandName = commandName;        
        this.outputTabName = ((DataObject)getNode().getCookie(DataObject.class)).getPrimaryFile().getName();
        classFullName = ProjectStructureUtils.getClassFullNameByNode(xnode);
        project = ProjectStructureUtils.getProjectForNode(xnode);
    }
           
        
    /**
     * Returns the full name of the class represented by this node
     * @return Class full name
     */
    protected String getClassFullName() { return classFullName ;}
    /**
     * Returns the associated node
     * @return The associated node
     */
    protected Node getNode() { return xnode ;}
    
    /**
     * Returns the command name
     * @return Returns the command name
     */
    protected String getCommandName() { return commandName ;}
    
    /**
     * Returns the output tab name where the output will be written for this node
     * @return Returns output tab name
     */
    protected String getOutputTabName() { return outputTabName ;}
           
   
     
     /**
     * Returns the project associated with the node which is used to perform the action
     * @return Returns the project this action node belongs to
     */
    public Project getProject() {
        return project;
    }  
    
}
