/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.config;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;
import org.netbeans.api.project.Project;
import org.netbeans.modules.coverage.ui.CommonDialogsCollection;
import org.netbeans.modules.coverage.utils.*;
import org.openide.filesystems.FileAlreadyLockedException;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.nodes.Node;
/**
 * For given node finds it's project node, finds org.netbeans.api.project.Project object,
 * then returns project.properties file as java.util.Properties pbject.<br>
 *
 * <b>TODO: </b>
 * This object should cache project project properties.
 * But it should implement FileChangeListener to listen for project file
 * changes and remove it from cache after external modification...<br>
 *
 * <b>TODO: </b>
 * Add properties saving to registry... <br>After properties modification would be moved from
 * project.peroperties to module wizard, all module properties for project should be saved to
 * registry.
 *
 * @author Andrey Korostelev
 */
class ProjectPropertiesReader {
    
    /** Creates a new instance of ProjectPropertiesReader */
    private ProjectPropertiesReader() {
    }
    
    /**
     *Returns project properties by Node
     **/
    public static Properties getProjectPropertiesByNode(Node node){
        Project project = ProjectStructureUtils.getProjectForNode(node);
        return loadProjectProperties(project);
    }
    
    /**
     *Loads project configuration properties from file
     **/
    public static Properties loadProjectProperties(Project project){
        Properties props = new Properties();
        
        // find prooperties file for project
        FileObject projectDir = project.getProjectDirectory();
        FileObject projectPropertiesFile = projectDir.getFileObject("nbproject/coverage.properties");
        
        // if projectPropertiesFile exists, load properties
        if (projectPropertiesFile != null){
            InputStream is = null;
            try {
                is = projectPropertiesFile.getInputStream();
                props.load(is);
            } catch (FileNotFoundException ex) {
                ex.printStackTrace();
                CommonDialogsCollection.userError("can't load properties from coverage properties file... ");
            } catch (IOException ex) {
                ex.printStackTrace();
                CommonDialogsCollection.userError("can't load properties from coverage properties file... ");
            } finally {
                if (is != null)
                    try {is.close();} catch (IOException ex) {
                        ex.printStackTrace();
                    }
            }
        }
        return props;
    }
    /**
     *Stores project configuration properties to file
     *
     **/
    public static void storeProjectPropertiesToFile(Project project, Properties props){
        // find prooperties file for project
        FileObject projectDir = project.getProjectDirectory();
        FileObject projectPropertiesFile = projectDir.getFileObject("nbproject/coverage.properties");
        
        if (projectPropertiesFile == null){
            try{
                projectPropertiesFile = FileUtil.createData(projectDir, "nbproject/coverage.properties");
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        
        if (projectPropertiesFile != null){
            OutputStream os = null;
            FileLock fileLock = null;
            
            try {
                fileLock = projectPropertiesFile.lock();
                os = projectPropertiesFile.getOutputStream(fileLock);
                props.store(os, "coverage viewer module properties for project");
                os.flush();
            } catch (ClassCastException ex) {
                ex.printStackTrace();
                CommonDialogsCollection.userError("Can't Store properties to coverage properties file... There are non-string objects.");
            } catch (FileAlreadyLockedException ex){
                ex.printStackTrace();
                CommonDialogsCollection.userError("Coverage properties file is already locked. Please retry later.");
            } catch (IOException ex) {
                ex.printStackTrace();
                CommonDialogsCollection.userError("can't store properties to coverage properties file... Please retry later.");
            } finally {
                try{
                    if (os != null) os.close();
                } catch (IOException ex){
                    ex.printStackTrace();
                }
                if (fileLock != null) fileLock.releaseLock();
            }
        } else {
            CommonDialogsCollection.userError("Can't store properties. coverage properties file wasn't created. ");
        }
    }
    
    
    
}
