/*
* ProjectUtils.java
  */

package org.netbeans.modules.coverage.utils;
import org.netbeans.api.java.classpath.ClassPath;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectInformation;
import org.netbeans.api.project.ProjectUtils;
import org.netbeans.api.project.ui.OpenProjects;
import org.netbeans.modules.coverage.ui.CommonDialogsCollection;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.netbeans.api.java.source.JavaSource;
import org.netbeans.api.java.source.ClasspathInfo;
/**
* Temporary class to store helpfull methods
* @author  akorostelev
*/
public class ProjectStructureUtils {


    /**
     * Returns full name of the class thais associated witha node
     * @param node Node to find class name for
     * @return Full class name or null if could not determine
     */
    public static String getClassFullNameByNode(Node node){
        String result = null;
        if(node != null) {
            DataObject dob = node.getCookie(DataObject.class);
            FileObject fo = dob.getPrimaryFile();
            JavaSource js = JavaSource.forFileObject(fo);
            if(js != null) {
                ClasspathInfo cpi = js.getClasspathInfo();
                ClassPath cp = cpi.getClassPath (ClasspathInfo.PathKind.SOURCE);
                if(cp != null) {
                    return cp.getResourceName(fo,'.',false);
                }
            }
        }
        return(result);
    }




    /**
     * Returns opened project associated with the given name
     * @param pName Project name
     * @return Project object
     */
    public static Project selectProjectByName(String pName){
        Project project = null;
        Project[] projects = OpenProjects.getDefault ().getOpenProjects();
        ProjectInformation pInfo = null;
        for (int i = 0; i < projects.length; i++){
            pInfo = ProjectUtils.getInformation(projects[i]);
            // TEMP action.
            // getDisplayName used for modules in nb5.0
            // getName will work for java project in nb5.0
            // and java project/nb module in nb5.5
            if ( pInfo.getDisplayName().equalsIgnoreCase(pName)
            || pInfo.getName().equalsIgnoreCase(pName)){
                project = projects[i];
                break;
            }
        }
        return project;
    }

    /**
     * returns Project object tha contains specified node
     * @param node Node to get project for
     * @return Enclosing project
     */
    public static Project getProjectForNode(Node node){
        DataObject cc = (DataObject)node.getCookie(DataObject.class);
        Project project = FileOwnerQuery.getOwner(cc.getPrimaryFile());
        if (project == null){
            CommonDialogsCollection.userError("Unsapported type of  project:"+ ProjectUtils.getInformation(project).getDisplayName()+".  This module works only with java projects.");
        }
        return project;
    }



}
