/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.api;

import java.io.File;
import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.api.AntTargetExecutor;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.api.project.ui.OpenProjects;
import org.netbeans.jellytools.JellyTestCase;
import org.netbeans.jellytools.actions.SaveAllAction;
import org.netbeans.jemmy.EventTool;
import org.netbeans.junit.NbTestSuite;
import org.netbeans.modules.bpel.samples.AsynchronousSampleWizardIterator;
import org.openide.execution.ExecutorTask;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.filesystems.Repository;
import org.openide.loaders.DataObject;
import org.openide.loaders.TemplateWizard;

/**
 *
 * @author mikk@netbeans.org
 */
public class NewAsyncProjectTests extends JellyTestCase{
    
    private EventTool eventTool = new EventTool();
    
    
    public NewAsyncProjectTests(String name) {
        super(name);
    }
    
    private final String WORK_DIR = System.getProperty("xtest.workdir");
    
    public static NbTestSuite suite() {
        NbTestSuite suite = new NbTestSuite(NewAsyncProjectTests.class);
        return suite;
    }
    
    
    public void testCreateSimpleProject(){
        createProject(WORK_DIR+File.separator+"asyncsimple", "asyncsimpleproject");
    }
    
    public void testCreateCaseSensitiveProject(){
        createProject(WORK_DIR+File.separator+"AsYnCCaSeProJECt", "ASYnc");
    }
    
    
    public void testCreateNameWithSpacesProject(){
        createProject(WORK_DIR+File.separator+"The async project folder", "my a s ync name");
    }
    
    public void testCreateNameWithDashProject(){
        createProject(WORK_DIR+File.separator+"The-dashed-async-project-folder", "my-async-name");
    }
    
    
    public void testCreateNumberNameProject(){
        createProject(WORK_DIR+File.separator+"789", "012");
    }
    
    
    private void createProject(String projectFolder, String projectName){
        try{
            AsynchronousSampleWizardIterator iterator = AsynchronousSampleWizardIterator.createIterator();
            FileObject trsCompositeApp = Repository.getDefault().getDefaultFileSystem().findResource("/Templates/Project/Samples/SOA/AsynchronousSample");
            
            assertNotNull("Can't find template for SynchronousSampleProject. Check 'CVS/enterprise/bpel/samples/src/org/netbeans/modules/bpel/samples/resources/layer.xml' for correct path.", trsCompositeApp);            
            
            TemplateWizard descriptor = new TemplateWizard();
            descriptor.setTemplate(DataObject.find(trsCompositeApp));
            descriptor.putProperty(AsynchronousSampleWizardIterator.PROJDIR, new File(projectFolder));
            descriptor.putProperty(AsynchronousSampleWizardIterator.NAME, projectName);
            iterator.initialize(descriptor);
            iterator.instantiate();
            
            assertEquals(isASyncProjectCorrectlyCreated(projectName, projectFolder), true);
            
            //opening project
            Project project = ProjectManager.getDefault().findProject(FileUtil.toFileObject(new File(projectFolder+File.separator+projectName)));
            OpenProjects.getDefault().open(new Project[]{project}, false);
            
            
            //building project
            FileObject prjBuild = project.getProjectDirectory().getFileObject("build.xml");
            DataObject prjDataObj = DataObject.find(prjBuild);
            AntProjectCookie prjAntCookie = (AntProjectCookie)prjDataObj.getCookie(AntProjectCookie.class);
            AntTargetExecutor.Env prjEnv = new AntTargetExecutor.Env();
            AntTargetExecutor prjExecutor = AntTargetExecutor.createTargetExecutor(prjEnv);
            ExecutorTask task = prjExecutor.execute(prjAntCookie, new String[]{"dist"});
            int res = task.result();
            assertEquals((res==0), true);
            
            
            /*
             //this is an alternative way to invoke project context menu and it works.
            ActionProvider ap = (ActionProvider)project.getLookup().lookup( ActionProvider.class);
            ap.invokeAction(ActionProvider.COMMAND_BUILD, project.getLookup());
             */
            eventTool.waitNoEvent(1000);
        }catch(Exception e){
            e.printStackTrace(getLog());
            fail("Exception found");
        }
    }
    
    
    
    private boolean isASyncProjectCorrectlyCreated(String projectName, String projectLocation){
        //TODO: we now check the main classes only. Should we check all?
        File rootFolder = new File(projectLocation);
        if (!rootFolder.exists()){
            return false;
        }
        File moduleFolder = new File(projectLocation+File.separator+projectName);
        if (!moduleFolder.exists()){
            return false;
        }
        String srcPath = moduleFolder.getPath()+File.separator+"src"+File.separator;
        File file = new File(srcPath+"AsynchronousSample.bpel");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"AsynchronousSample.xsd");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"AsynchronousSample.wsdl");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"AsynchronousSampleClient.wsdl");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"AsynchronousSampleClient.bpel");
        if (!file.exists()){
            return false;
        }
        
        File appFolder = new File(projectLocation+File.separator+projectName+"Application");
        if (!appFolder.exists() || !appFolder.isDirectory()){
            return false;
        }
        
        return true;
    }
    
    
    protected void tearDown(){
        try{
            eventTool.waitNoEvent(1000);
            new SaveAllAction().performAPI();
        }catch(Exception e){}
    }
    
    
    
}
