/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.api;

import java.io.File;
import org.netbeans.jellytools.JellyTestCase;

import org.netbeans.jellytools.actions.SaveAllAction;
import org.netbeans.jemmy.EventTool;
import org.netbeans.junit.NbTestSuite;
import org.netbeans.modules.bpel.project.wizards.NewBpelproProjectWizardIterator;
import org.netbeans.modules.bpel.project.wizards.WizardProperties;
import org.netbeans.modules.compapp.projects.base.ui.customizer.IcanproProjectProperties;
import org.openide.WizardDescriptor;

/**
 *
 * @author mikk@netbeans.org
 */
public class NewBPELProjectTests extends JellyTestCase{
    
    private EventTool eventTool = new EventTool();
    
    public NewBPELProjectTests(String name) {
        super(name);
    }
    
    private final String WORK_DIR = System.getProperty("xtest.workdir");
    
    public static NbTestSuite suite() {
        NbTestSuite suite = new NbTestSuite(NewBPELProjectTests.class);
        return suite;
    }
    
    
    public void testCreateSimpleMainProject(){
        createProject(WORK_DIR+File.separator+"bpelsimple1", "bpelsimple1", true);
    }
    
    public void testCreateSimpleProject(){
        createProject(WORK_DIR+File.separator+"bpelsimple2", "bpelsimple2", false);
    }   
        
   
    public void testCreateProjectWithSpaces(){
        createProject(WORK_DIR+File.separator+"bpel simple", "bpel simple",  true);
    }
    
    public void testCreateCaseSensitiveProject(){
        createProject(WORK_DIR+File.separator+"bPelCaSEsensSITive", "bPelCaSEsensSITive", true);
    }
        
    
    
    private void createProject(String projectFolder, String projectName, boolean mainProject){
        try{
            NewBpelproProjectWizardIterator iterator = new NewBpelproProjectWizardIterator();
            
            WizardDescriptor.Panel panels[] = new WizardDescriptor.Panel[1];
            WizardDescriptor d = new WizardDescriptor(panels);
            
            d.putProperty(WizardProperties.SET_AS_MAIN, mainProject );
            //d.putProperty(WizardProperties.J2EE_LEVEL, IcanproProjectProperties.J2EE_1_4);
            d.putProperty(WizardProperties.PROJECT_DIR, new File(projectFolder));
            d.putProperty(WizardProperties.NAME, projectName);
            
            iterator.initialize(d);
            iterator.instantiate();
                        
            assertEquals(isBPELProjectCorrectlyCreated(projectName, projectFolder), true);           
            
        }catch(Exception e){
            e.printStackTrace(getLog());
            fail("Exception found");
        }
    }
    
    
    
    private boolean isBPELProjectCorrectlyCreated(String projectName, String projectLocation){
        File rootFolder = new File(projectLocation);
        if (!rootFolder.exists()){
            return false;
        }                
        
        return true;
    }
    
    
    protected void tearDown(){
        try{
            eventTool.waitNoEvent(1000);
            new SaveAllAction().performAPI();
        }catch(Exception e){}
    }
    
    
    
}
