/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.api;

import java.io.File;
import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.api.AntTargetExecutor;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.api.project.ui.OpenProjects;
import org.netbeans.jellytools.JellyTestCase;
import org.netbeans.jellytools.actions.SaveAllAction;
import org.netbeans.jemmy.EventTool;
import org.netbeans.junit.NbTestSuite;
import org.netbeans.modules.bpel.samples.TravelReservationServiceWizardIterator;
import org.openide.execution.ExecutorTask;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.filesystems.Repository;
import org.openide.loaders.DataObject;
import org.openide.loaders.TemplateWizard;


/**
 * @author mikk@netbeans.org
 */
public class NewTRSProjectTests extends JellyTestCase{
    
    private EventTool eventTool = new EventTool();
    
   
    public NewTRSProjectTests(String name) {
        super(name);
    }
    
    private final String WORK_DIR = System.getProperty("xtest.workdir");
    
    public static NbTestSuite suite() {
        NbTestSuite suite = new NbTestSuite(NewTRSProjectTests.class);
        return suite;
    }
    
    
    public void testCreateSimpleProject(){
        createProject(WORK_DIR+File.separator+"TRSsimple", "TRSsimpleproject");
    }
    
    public void testCreateCaseSensitiveProject(){
        createProject(WORK_DIR+File.separator+"TRSCaSeProJECt", "TRS");
    }
    
    
    public void testCreateNameWithSpacesProject(){
        createProject(WORK_DIR+File.separator+"The TRS project folder", "my t rs name");
    }
    
    public void testCreateNameWithDashProject(){
        createProject(WORK_DIR+File.separator+"The-dashed-TRS-project-folder", "my-TRS-name");
    }
    
    
    public void testCreateNumberNameProject(){
        createProject(WORK_DIR+File.separator+"545", "321");
    }
    
    
    private void createProject(String projectFolder, String projectName){
        try{
            TravelReservationServiceWizardIterator iterator = TravelReservationServiceWizardIterator.createIterator();
            FileObject trsCompositeApp = Repository.getDefault().getDefaultFileSystem().findResource("/Templates/Project/Samples/SOA/TravelReservationService");
            
            TemplateWizard descriptor = new TemplateWizard();
            descriptor.setTemplate(DataObject.find(trsCompositeApp));
            descriptor.putProperty(TravelReservationServiceWizardIterator.PROJDIR, new File(projectFolder));
            descriptor.putProperty(TravelReservationServiceWizardIterator.NAME, projectName);
            iterator.initialize(descriptor);
            iterator.instantiate();
            
            assertEquals(isTRSProjectCorrectlyCreated(projectName, projectFolder), true);
            
            //opening project
            Project project = ProjectManager.getDefault().findProject(FileUtil.toFileObject(new File(projectFolder+File.separator+projectName)));
            OpenProjects.getDefault().open(new Project[]{project}, false);
            
            
            //building project
            FileObject prjBuild = project.getProjectDirectory().getFileObject("build.xml");
            DataObject prjDataObj = DataObject.find(prjBuild);
            AntProjectCookie prjAntCookie = (AntProjectCookie)prjDataObj.getCookie(AntProjectCookie.class);
            AntTargetExecutor.Env prjEnv = new AntTargetExecutor.Env();
            AntTargetExecutor prjExecutor = AntTargetExecutor.createTargetExecutor(prjEnv);
            ExecutorTask task = prjExecutor.execute(prjAntCookie, new String[]{"dist"});
            int res = task.result();
            assertEquals((res==0), true);
            
            eventTool.waitNoEvent(1000);
        }catch(Exception e){
            e.printStackTrace(getLog());
            fail("Exception found");
        }
    }
    
    
    
    private boolean isTRSProjectCorrectlyCreated(String projectName, String projectLocation){
        //TODO: we now check the main classes only. Should we check all?
        File rootFolder = new File(projectLocation);
        if (!rootFolder.exists()){
            return false;
        }
        File moduleFolder = new File(projectLocation+File.separator+projectName);
        if (!moduleFolder.exists()){
            return false;
        }
        String srcPath = moduleFolder.getPath()+File.separator+"src"+File.separator;
        File file = new File(srcPath+"AirlineReservationService.wsdl");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"HotelReservationService.wsdl");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"OTA_TravelItinerary.xsd");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"TravelReservationService.bpel");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"TravelReservationService.wsdl");
        if (!file.exists()){
            return false;
        }
        
        file = new File(srcPath+"VehicleReservationService.wsdl");
        if (!file.exists()){
            return false;
        }
        
        File appFolder = new File(projectLocation+File.separator+projectName+"Application");
        if (!appFolder.exists() || !appFolder.isDirectory()){
            return false;
        }
        
        return true;
    }
    
    
    protected void tearDown(){
        try{
            eventTool.waitNoEvent(1000);
            new SaveAllAction().performAPI();
        }catch(Exception e){}
    }
}
