/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.diagramelements;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;
import org.netbeans.jellytools.JellyTestCase;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.TimeoutExpiredException;
import org.netbeans.jemmy.operators.JDialogOperator;
import org.netbeans.jemmy.operators.JTextAreaOperator;
import org.netbeans.junit.NbTestSuite;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.verifiers.SubPatternVerifier;
import org.netbeans.orchlib.ext.Util;
import static org.netbeans.test.bpel.diagramelements.DE_Util.*;

public class DE_Pick_Alarm extends JellyTestCase {
    private static final String workDir = System.getProperty("xtest.workdir");
    private static PrintStream myOut = null;
    
    private static String prName = "OrchDEProject17";
    private static String bpelName = prName;
    private PatternTypes patternRootType = PatternTypes.PICK;
    private PatternTypes patternType = PatternTypes.ON_ALARM;

    private MultiviewEditorOperator editor = null;
    private SubPatternVerifier patternVerifier = null;
    
    private static boolean isFirst = true;
    

    /** Need to be defined because of JUnit */
    public DE_Pick_Alarm(String name) {
        super(name);
    }
    
    public static NbTestSuite suite() {
        NbTestSuite suite = new NbTestSuite(org.netbeans.test.bpel.diagramelements.DE_Pick_Alarm.class);
        return suite;
    }

    public void testCheckContextMenu(){
	String[] items = {"Go to Source", "Delete"};
	if (!patternVerifier.checkContextMenu(items)){
	    fail("Test failed. See log for details.");
	}
    }
    
    public void testRemoveByPopup(){
	if (!patternVerifier.checkRemoveByPopup()){
	    fail("Test failed. See log for details.");
	}
    }

    public void testRemoveByShortcut(){
	if (!patternVerifier.checkRemoveByShortcut()){
	    fail("Test failed. See log for details.");
	}
    }
    
    public void testRemoveByCrossButton(){
	if (!patternVerifier.checkRemoveByCrossButton()){
	    fail("Test failed. See log for details.");
	}
    }
    
//------------------------------------------------------------------------------
    
    public void setUp() throws FileNotFoundException{
        System.out.println("########  "+getName()+"  #######");
        waitPopupTime = JemmyProperties.getCurrentTimeouts().getTimeout("JMenuOperator.WaitPopupTimeout");
        waitWindowTime = JemmyProperties.getCurrentTimeouts().getTimeout("WindowWaiter.WaitWindowTimeout");
        waitComponentTime = JemmyProperties.getCurrentTimeouts().getTimeout("ComponentOperator.WaitComponentTimeout");
        waitDialogTime = JemmyProperties.getCurrentTimeouts().getTimeout("DialogWaiter.WaitDialogTimeout");
        JemmyProperties.setCurrentTimeout("JMenuOperator.WaitPopupTimeout", 5000);
        JemmyProperties.setCurrentTimeout("WindowWaiter.WaitWindowTimeout", 5000);
        JemmyProperties.setCurrentTimeout("ComponentOperator.WaitComponentTimeout", 5000);
        JemmyProperties.setCurrentTimeout("DialogWaiter.WaitDialogTimeout", 10000);
	
        if (isFirst){
            isFirst = false;
            try{
		Util.createOrchestrationProject(prName, workDir, bpelName, bpelName);
//		Util.createBPEL(prName, prName);
            }catch(Exception e){
                fail("Exception while creating project: " + prName);
            }
        }
	
        editor = openProjectBPEL(prName, bpelName);
	patternVerifier = new SubPatternVerifier(editor, patternRootType, patternType, getLog());
    }
    
    public void tearDown() throws FileNotFoundException, IOException, InterruptedException{
        try{
            JemmyProperties.setCurrentTimeout("DialogWaiter.WaitDialogTimeout", 2000);
            JDialogOperator dlgError = new JDialogOperator("Unexpected Exception");
            JTextAreaOperator textarea = new JTextAreaOperator(dlgError);
            String str = textarea.getDisplayedText();
            int pos = str.indexOf("\n");
            if(pos != -1){str = str.substring(1, pos-1);}
            dlgError.close();
	    fail("Exception dialog with message: '" + str + "'");
        }catch(TimeoutExpiredException e1){
	}finally{
	    JemmyProperties.setCurrentTimeout("JMenuOperator.WaitPopupTimeout", waitPopupTime);
	    JemmyProperties.setCurrentTimeout("WindowWaiter.WaitWindowTimeout", waitWindowTime);
	    JemmyProperties.setCurrentTimeout("ComponentOperator.WaitComponentTimeout", waitComponentTime);
            JemmyProperties.setCurrentTimeout("DialogWaiter.WaitDialogTimeout", waitDialogTime);
	}
        closeSaveDlg();
    }
    
  
    private long waitPopupTime;
    private long waitWindowTime;
    private long waitComponentTime;
    private long waitDialogTime;
}
       
