/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * Util.java
 *
 */

package org.netbeans.test.bpel.dnd;


import java.util.Stack;
import org.netbeans.jellytools.ProjectsTabOperator;
import org.netbeans.jellytools.nodes.Node;
import org.netbeans.jellytools.nodes.ProjectRootNode;
import org.netbeans.jemmy.EventTool;
import org.netbeans.orchlib.core.operators.ConnectionOperator;
import org.netbeans.orchlib.core.operators.DesignViewOperator;
import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.VisualElementOperator;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;

/**
 * Utility class that holds some methods used in the tests in this package
 * @author mikk@netbeans.org
 */
public class Util {
    
    /** Creates a new instance of Util */
    public Util() {
    }
    
    private static EventTool eventTool = new EventTool();
    
    private static Node getNode(String nodePath, String projectName){
        ProjectsTabOperator pto = ProjectsTabOperator.invoke();
        ProjectRootNode root = new ProjectRootNode(pto.tree(),projectName);
        
        Node node = new Node(root, nodePath);
        return node;
    }
    
    /**
     * Drags pattern to the middle of the predefined Sequence activity.
     * The method highly depends on the BPEL project used together with
     * these tests
     * @param pattern
     * @param view
     * @throws org.netbeans.orchlib.exceptions.ElementNotFoundException
     * @return
     */
    public static PatternOperator dragPatternToSeqMiddle(PatternOperator pattern, DesignViewOperator view) throws ElementNotFoundException{
        PatternOperator container = new PatternOperator(view, "Seq1", PatternTypes.SEQUENCE, 0);
        PatternOperator process = new PatternOperator(view, PatternTypes.PROCESS, 0);
        ConnectionOperator conn = new ConnectionOperator("Empty1", "Empty2", process);
        view.dragPattern(pattern, conn, container);

        return new PatternOperator(container, pattern.getName(), 0);
    }
    
    
    /**
     * Drags pattern to the first position of the predefined Sequence activity.
     * The method highly depends on the BPEL project used together with
     * these tests
     */
    public static PatternOperator dragPatternToSeqFirst(PatternOperator pattern, DesignViewOperator view) throws ElementNotFoundException{
        PatternOperator container = new PatternOperator(view, "Seq2", PatternTypes.SEQUENCE, 0);
        PatternOperator process = new PatternOperator(view, PatternTypes.PROCESS, 0);
        ConnectionOperator conn = new ConnectionOperator("Empty2", "Empty3", process);
        view.dragPattern(pattern, conn, container);

        return new PatternOperator(container, pattern.getName(), 0);
    }
    
    
    /**
     * Drags pattern to the last position of the predefined Sequence activity.
     * The method highly depends on the BPEL project used together with
     * these tests
     */
    public static PatternOperator dragPatternToSeqLast(PatternOperator pattern, DesignViewOperator view) throws ElementNotFoundException{
        PatternOperator container = new PatternOperator(view, "Seq3", PatternTypes.SEQUENCE, 0);
        PatternOperator process = new PatternOperator(view, PatternTypes.PROCESS, 0);
        PatternOperator swContainer = new PatternOperator(process, "If1", 0);
        VisualElementOperator sw = new VisualElementOperator(swContainer, ElementTypes.CONTENT_ELEMENT, 0);
        VisualElementOperator emp = new VisualElementOperator(process, "Empty6", 0);
        eventTool.waitNoEvent(1000);
        ConnectionOperator conn = new ConnectionOperator(emp, sw);
        view.dragPattern(pattern, conn, container);

        return new PatternOperator(container, pattern.getName(), 0);
    }
    
    
    /**
     * Drags pattern to the If Else activity.
     * The method highly depends on the BPEL project used together with
     * these tests
     */
    public static PatternOperator dragPatternToIfThen(PatternOperator pattern, DesignViewOperator view) throws ElementNotFoundException{
        PatternOperator container = new PatternOperator(view, "If1", PatternTypes.IF, 0);
        VisualElementOperator switch1 = new VisualElementOperator(container, ElementTypes.STUB_ELEMENT, 0);
        view.dragPattern(pattern, switch1);

        return new PatternOperator(container, pattern.getName(), 0);
    }
    
    
    /**
     * Drags pattern to predefined If activity.
     * The method highly depends on the BPEL project used together with
     * these tests
     */
    public static PatternOperator dragPatternToIf(PatternOperator pattern, DesignViewOperator view) throws ElementNotFoundException{
        PatternOperator container = new PatternOperator(view, "If1", PatternTypes.IF, 0);
        VisualElementOperator caseHolder = new VisualElementOperator(container, ElementTypes.PLACEHOLDER_ELEMENT, 0);
        view.dragPattern(pattern, caseHolder);

        return new PatternOperator(container, pattern.getName(), 0);
    }
    
    
    /**
     * Dragging pattern to Pick OnMessage activity
     * The method highly depends on the BPEL project used together with
     * these tests
     */
    public static PatternOperator dragPatternToPickOnMessage(PatternOperator pattern, DesignViewOperator view) throws ElementNotFoundException{
        PatternOperator container = new PatternOperator(view, "Pick1", PatternTypes.PICK, 0);
        container = new PatternOperator(container, PatternTypes.ON_MESSAGE, 0);
        VisualElementOperator onMessageHolder = new VisualElementOperator(container, ElementTypes.PLACEHOLDER_ELEMENT, 0);
        view.dragPattern(pattern, onMessageHolder);

        return new PatternOperator(container, pattern.getName(), 0);
    }
    
    /**
     * Drags pattern to the predefined Pick OnAlarm activity.
     * The method highly depends on the BPEL project used together with
     * these tests
     */
    public static PatternOperator dragPatternToPickOnAlarm(PatternOperator pattern, DesignViewOperator view) throws ElementNotFoundException{
        PatternOperator container = new PatternOperator(view, "Pick1", PatternTypes.PICK, 0);
        container.select();
        eventTool.waitNoEvent(1500);
        
        container = new PatternOperator(container, PatternTypes.ON_ALARM, 0);
        container.select();
        eventTool.waitNoEvent(1500);
        
        VisualElementOperator onMessageHolder = new VisualElementOperator(container, ElementTypes.PLACEHOLDER_ELEMENT, 0);
        onMessageHolder.select();
        eventTool.waitNoEvent(1500);
        
        view.dragPattern(pattern, onMessageHolder);

        System.err.println("dfdfgdfgdfg"+pattern.getName());
        return new PatternOperator(container, pattern.getName(), 0);
    }
    
    /**
     * Creates the pattern that will be dragged later at the default position and names it
     * @param name Activity name
     * @param type Pattern type
     * @param view Design view
     * @return created pattern
     * @throws org.netbeans.orchlib.exceptions.ElementNotFoundException
     */
    public static PatternOperator createAndNamePattern(String name, PatternTypes type, DesignViewOperator view) throws ElementNotFoundException{
        //placing element in first container
        PatternOperator container = new PatternOperator(view, "Empty", PatternTypes.SEQUENCE, 0);
        
        VisualElementOperator target = new VisualElementOperator(container, ElementTypes.PLACEHOLDER_ELEMENT, 0);
        //System.err.println("[TEST]before creation");
        view.createPatternFromPalette(type, target);
        //System.err.println("[TEST]after creation");
        //eventTool.waitNoEvent(1000);
        
        //naming element
        PatternOperator element = new PatternOperator(container, type, 0);
        //namePattern(element, type, name);
        //eventTool.waitNoEvent(1000);
        return new PatternOperator(container, type, 0);
    }
    
    /**
     * Opens BPEL project and the .bpel file from the project
     */
    public static  MultiviewEditorOperator openProjectBPEL(String projectName, String bpelName){
        Node node = getNode("Process Files|"+projectName, "DNDTestApplication");
        node.tree().clickOnPath(node.getTreePath(),2);
        eventTool.waitNoEvent(2000);
        MultiviewEditorOperator editor = new MultiviewEditorOperator(bpelName);
        return editor;
    }
    
    
    
}
