/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.roundtrip;

import java.util.ArrayList;
import javax.swing.tree.TreePath;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.SourceViewOperator;
import org.netbeans.orchlib.ext.Util;
import org.netbeans.orchlib.ext.operators.ExtJTreeOperator;
import org.netbeans.orchlib.utils.ITitlesAndLabels;
import org.netbeans.orchlib.utils.TypeSpecificProperties;
import org.netbeans.jellytools.EditorOperator;
import org.netbeans.jellytools.ProjectsTabOperator;
import org.netbeans.jellytools.properties.Property;
import org.netbeans.jellytools.nodes.ProjectRootNode;
import org.netbeans.jellytools.properties.PropertySheetOperator;
import org.netbeans.jellytools.actions.Action;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.TimeoutExpiredException;
import org.netbeans.jemmy.operators.JButtonOperator;
import org.netbeans.jemmy.operators.JDialogOperator;
import org.netbeans.jemmy.operators.Operator.StringComparator;
import org.netbeans.junit.NbTestSuite;
import org.w3c.dom.Node;
import org.w3c.dom.DOMException;
import org.w3c.dom.NodeList;
import org.netbeans.junit.ParametricTestCase;
import org.netbeans.junit.ParametricTestSuite;

/**
 *
 * @author apepin
 */
public class Forward extends ParametricTestSuite{
    
    /** Creates a new instance of ForwardParam */
    private EventTool eventTool = new EventTool();
    private final String workDir = System.getProperty("xtest.workdir");
    private final static String projName = "OrchRoundTripProject1";
    private final static String bpelProjName = projName+".bpel";

    PatternTypes type = null;
    PatternHandler handler = null;
    
    protected Object[] getParameters(){
        PatternTypes[] types = TypeSpecificProperties.getPalettePatternsTypes();
        ArrayList params = new ArrayList(types.length);
        //exclude some types from testing
        for(PatternTypes tp: types){
            switch (tp){
                case PARTNER_LINK:  break;
                default:            params.add(tp);
            }
        }
        return params.toArray();
//        return new Object[]{PatternTypes.EMPTY};
    }
    
    protected ParametricTestCase[] cases(Object parameter){
        type = (PatternTypes)parameter;
//        String typeName = TypeSpecificProperties.getPatternPaletteName(type);
        handler = PatternHandler.createPatternHandler(type, editor);
        ParametricTestCase[] testCases = new ParametricTestCase[]{new CheckCreated(), new CheckRenamed(), new CheckDeleted()};
        return testCases;
    }
    
    public static NbTestSuite suite() {
        return new Forward();
    }
    
    ProjectsTabOperator pto = ProjectsTabOperator.invoke();
    MultiviewEditorOperator editor = null;
    PropertySheetOperator pso = null;
    long waitPropertyTime;

    public void prepare(){
        //set timeouts
        System.out.println("########  "+getName()+"  #######");
        waitPropertyTime = JemmyProperties.getCurrentTimeout("Waiter.WaitingTime");
        //check if the project already exists
        ExtJTreeOperator projTree = new ExtJTreeOperator(pto);
        org.netbeans.jellytools.nodes.Node projnode = null;

//            String waitNodeTimeout = "JTreeOperator.WaitNextNodeTimeout";
        long waitNodeTime = JemmyProperties.getCurrentTimeout("JTreeOperator.WaitNextNodeTimeout");
        JemmyProperties.setCurrentTimeout("JTreeOperator.WaitNextNodeTimeout", 2000);
        try{
            projnode = new ProjectRootNode(projTree,projName);
        }
        catch(TimeoutExpiredException e){}
        finally{
            JemmyProperties.setCurrentTimeout("JTreeOperator.WaitNextNodeTimeout", waitNodeTime);
        }
        if(projnode == null){
            //create a new project
            Util.createOrchestrationProject(projName, workDir);
            eventTool.waitNoEvent(1000);
            Util.createBPEL(projName, projName);
        }
        //open the project for editing
        projTree = new ExtJTreeOperator(pto);
        TreePath projPath = new ProjectRootNode(projTree,projName).getTreePath();
        projTree.expandPath(projPath, "Process Files", bpelProjName);
        TreePath bpel = projTree.findPathEx(projPath, "Process Files", bpelProjName);
        projTree.clickOnPath(bpel ,2);
        eventTool.waitNoEvent(5000);

        editor = new MultiviewEditorOperator(bpelProjName);

        new Action(ITitlesAndLabels.MAIN_MENU_WINDOW_PROPERTIES, null).performMenu();
        pso = PropertySheetOperator.invoke();
        pso.setComparator(new StringComparator(){
            public boolean equals(String caption, String match){
                if(caption == null || match == null)
                    return false;
                return caption.equals(match);
            }
        });

    }
    public void cleanup(){
        startQuestionCloser();
        EditorOperator.closeDiscardAll();
        stopQuestionCloser();
    }
////////////////////////////////////////////////////////////////test cases////////////////////////////////////////
    public class CheckCreated extends ParametricTestCase{

        public void parametrize(Object parameter){
            setName(getName()+handler.getPatternName());
        }
        public void execute(){
            NodeList result = null;
            try{

                handler.createElement();
                eventTool.waitNoEvent(400);
                //find the element 
                PatternOperator elementPattern = handler.findElement();
                //test the element
                elementPattern.select();
                eventTool.waitNoEvent(400);
                //check for exception
                if(Util.closeException()){
                    fail("An Exception occured while processing the element of type "+handler.getPatternName()+".");
                }
                //check XML
                SourceViewOperator source = editor.getSourceView();
                result = source.findNodesByXpath(handler.getXPath());
                if(result == null || !existsNodeWithName(result, elementPattern.getName())){
                    fail("Element of type "+handler.getPatternName()+" and name "+elementPattern.getName()+" was not found in the BPEL source");
                }                
                log("FirstTest: Element "+elementPattern.getName()+" passed");
            }catch(Exception e){
                e.printStackTrace(getLog());
                fail(e.getMessage());
            }
        }
    }
    
    public class CheckRenamed extends ParametricTestCase{
        String xPath = null;
        String goldName = "Sasha";
        
        public void parametrize(Object parameter){
            setName(getName()+handler.getPatternName());
        }
        public void execute(){
            NodeList result = null;
            try{
                PatternOperator elementPattern = handler.findElement();
                elementPattern.select();
                eventTool.waitNoEvent(400);
                //check for exception
                if(Util.closeException()){
                    fail("An Exception occured while processing the element of type "+handler.getPatternName()+".");
                }
                //rename element via properties
                setProperty(pso, "Name", goldName);
                eventTool.waitNoEvent(400);
                handler.setName(goldName);
                
                //check XML
                SourceViewOperator source = editor.getSourceView();
                eventTool.waitNoEvent(400);
                result = source.findNodesByXpath(handler.getXPath());
            }catch(TimeoutExpiredException e){
                fail("Impossible to change name of "+handler.getType()+" element.");
            }catch(Exception e){
                e.printStackTrace(getLog());
                fail(e.getMessage());
            }
            if(result == null || !existsNodeWithName(result, goldName)){
                fail("Element of type "+handler.getPatternName()+" with a name "+goldName+" was not found in the BPEL source");
            }
            
        }
    }
    
    public class CheckDeleted extends ParametricTestCase{
        String xPath = null;

        public void parametrize(Object parameter){
            setName(getName()+handler.getPatternName());
        }
        public void execute(){
            String elName = null;
            NodeList result = null;
            try{
                PatternOperator elementPattern = handler.findElement();
                elementPattern.select();
                eventTool.waitNoEvent(400);
                //check for exception
                if(Util.closeException()){
                    fail("An Exception occured while processing the element of type "+handler.getPatternName()+".");
                }
                elName = elementPattern.getName();
                
                handler.deleteElement();
                eventTool.waitNoEvent(400);

                SourceViewOperator source = editor.getSourceView();
                eventTool.waitNoEvent(400);
                result = source.findNodesByXpath(handler.getXPath());
                log("Result is "+result);
            }catch(Throwable e){
                e.printStackTrace(getLog());
                fail(e.getMessage());
            }
            if(result != null && existsNodeWithName(result, elName)){
                fail("Element "+handler.getPatternName()+" was not removed from the BPEL source");
            }
        }
    }
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////    
    
    private boolean existsNodeWithName(NodeList list, String name){
        for(int i=0; i<list.getLength(); i++){
            Node node = list.item(i);
            String nodeName = null;
            try{
                Node attrNode = node.getAttributes().getNamedItem("name");
                if(attrNode != null)
                    nodeName = attrNode.getNodeValue();
            }
            catch(DOMException e){
            }
            if(nodeName != null && nodeName.equals(name)){
                return true;
            }
        }
        return false;
    }

    private void setProperty(PropertySheetOperator pso, String propName, String textValue){
        JemmyProperties.setCurrentTimeout("Waiter.WaitingTime", 2000);
        try{
            new Property(pso, propName).setValue(textValue);
        } finally{
            JemmyProperties.setCurrentTimeout("Waiter.WaitingTime", waitPropertyTime);
        }
        
    }
    
    
    private Thread closer = null;
    public void startQuestionCloser(){
        //close warning windows
        final String title = "Question";
        final String button = "Discard";
        closer = new Thread(new Runnable(){
            public void run(){
                for(;;){
                    if(JDialogOperator.findJDialog(title, false, true) != null){
                        new JButtonOperator(new JDialogOperator(title), button ).pushNoBlock();
                    }
                    try{
                        Thread.sleep(100);
                    }catch(InterruptedException e){
                        break;
                    }
                }
            }
        });
        closer.start();
    }
    public void stopQuestionCloser(){
        if(closer != null){
            closer.interrupt();
            closer = null;
        }
    }
}
