/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.roundtrip;

import java.awt.Point;
import java.awt.event.KeyEvent;
import java.util.HashMap;
import java.util.Map;
import org.netbeans.orchlib.core.operators.ConnectionOperator;
import org.netbeans.orchlib.core.operators.DesignViewOperator;
import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.VisualElementOperator;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.ext.Util;
import org.netbeans.orchlib.utils.TypeSpecificProperties;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.TimeoutExpiredException;
import org.netbeans.jemmy.operators.JPopupMenuOperator;

/**
 *
 * @author apepin
 */
public class PatternHandler {
    protected EventTool eventTool = new EventTool();
    protected MultiviewEditorOperator editor = null;
    PatternTypes type = null;
    String xPath = null;
    String name = null;
    Map<String, Prop> propmap = new HashMap<String, Prop>();
    
    public PatternHandler(PatternTypes pt, MultiviewEditorOperator edo){
        editor = edo;
        type = pt;
        xPath = "/process/sequence/"+getXPathName();
        
    }
    String getPatternName(){
        return TypeSpecificProperties.getPatternPaletteName(type).replaceAll(" ", "");
    }
    
    String getXPathName(){
        return getPatternName().substring(0, 1).toLowerCase() + getPatternName().substring(1);
    }
    
    String getXPath(){
        return xPath;
    }
    
    void addXPath(String extra){
        if(extra != null)
            xPath += extra;
    }
    
    void setName(String elName){
        name = elName;
    }
    
    PatternTypes getType(){
        return type;
    }
    
//    public Map<String, String> getPropMap(){
//        return propmap;
//    }
    
    public void setProperty(String key, String value){
        propmap.put(key, new Prop(value));
    }
    
    public String getProperty(String key){
        return propmap.get(key).value;
    }
    
    public String getPropertyType(String key){
        return propmap.get(key).isAttribute? "attribute": "";
    }
    
    public void setPropertyType(String key, boolean isAttribute){
        propmap.get(key).isAttribute = isAttribute;
    }
    
    public String[] getPropNames(){
        return propmap.keySet().toArray(new String[0]);
    }
    
    void createElement() throws ElementNotFoundException{
        
        DesignViewOperator view = editor.getDesignView();
        //create an element
        PatternOperator patRoot = view.getRootPatternOperator();
        PatternOperator patTarget = new PatternOperator(patRoot, PatternTypes.SEQUENCE, 0);
        VisualElementOperator el1 = new VisualElementOperator(patRoot, ElementTypes.CONTENT_ELEMENT, 0);
        VisualElementOperator el2 = new VisualElementOperator(patTarget, ElementTypes.ANY, 0);
        if(el2.getTypeString().equals(ElementTypes.PLACEHOLDER_ELEMENT.toString())){
            view.createPatternFromPalette(type, el2);
        }else{
            ConnectionOperator conn = new ConnectionOperator(el1, el2);            
            //workaround for a bug
            patTarget.select();
            eventTool.waitNoEvent(500);
            //workaround end
            view.createPatternFromPalette(type, conn, patTarget);            
        }        
    }
    
    PatternOperator findElement() throws ElementNotFoundException{
        PatternOperator elementPattern = null;
        DesignViewOperator view = editor.getDesignView();
        PatternOperator patRoot = view.getRootPatternOperator();
        PatternOperator patTarget = new PatternOperator(patRoot, PatternTypes.SEQUENCE, 0);
        long waitPatternTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
        JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 2000);
        try{
            elementPattern = new PatternOperator(patTarget, type, 0);
            if(name == null)
                setName(elementPattern.getName());
            else{//check element's name
                if(!name.equals(elementPattern.getName())){
                    elementPattern = null;
                    throw new ElementNotFoundException("Element of type "+getType().toString()+" and name "+name+" was not found on the diagram");
                }
            }
        } catch(TimeoutExpiredException e){
            throw new ElementNotFoundException("Element of type "+getType().toString()+" was not found on the diagram");
        } finally{
            JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatternTime);
        }
        return elementPattern;
    }
    
    void deleteElement() throws ElementNotFoundException{
        PatternOperator elementPattern = findElement();
        DesignViewOperator view = editor.getDesignView();
        //delete the element
        elementPattern.select();
        //check for exception
        eventTool.waitNoEvent(500);
        if(Util.closeException()){
            junit.framework.Assert.fail("An Exception occured while processing the element of type "+getPatternName()+".");
        }
        eventTool.waitNoEvent(500);
        view.pushKey(KeyEvent.VK_DELETE);
    }
    
    public static PatternHandler createPatternHandler(PatternTypes type, MultiviewEditorOperator editor){
        PatternHandler handler = null;
        switch(type){
            case PARTNER_LINK:
                handler = new PartnerLinkHandler(type, editor);
                break;
            default:
                handler = new PatternHandler(type, editor);
        }
        return handler;
    }
    
    class Prop{
        String value = "";
        boolean isAttribute = true;
        Prop(String val){
            value = val;
        }
    }
    
    
}

class PartnerLinkHandler extends PatternHandler{
    public PartnerLinkHandler(PatternTypes pt, MultiviewEditorOperator edo){
        super(pt, edo);
        xPath = "/process/partnerLinks/partnerLink";
        name = "PartnerLink1";
    }
    
    void createElement() throws ElementNotFoundException{
        DesignViewOperator view = editor.getDesignView();
        view.createPatternFromPalette(type, new Point(50, 50));
        //close OK editor
        String filename = editor.getName();
        filename = filename.substring(0, filename.indexOf('.'));
        Util.closePropertyEditor(filename);
        eventTool.waitNoEvent(500);
        //workaround for a bug
        PatternOperator patRoot = view.getRootPatternOperator();
        patRoot.select();
        eventTool.waitNoEvent(500);
        //workaround end
        
    }
    PatternOperator findElement() throws ElementNotFoundException{
        DesignViewOperator view = editor.getDesignView();
        PatternOperator elementPattern = null;
        long waitPatternTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
        JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 2000);
        try{
            elementPattern = new PatternOperator(view, name, type, 0);
        } catch(TimeoutExpiredException e){
            throw new ElementNotFoundException("Element of type "+getType().toString()+" was not found on the diagram");
        } finally{
            JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatternTime);
        }
        return elementPattern;
    }
    
    void deleteElement() throws ElementNotFoundException{
        super.deleteElement();
        //close dialog
        Util.closePopup("No");
    }
    
}
class FaultHandler extends PatternHandler{
    public FaultHandler(PatternTypes pt, MultiviewEditorOperator edo){
        super(pt, edo);
        xPath = "/process/compensationHandler/compensate";
        name = "Catcher1";
        
    }
    
    void createElement() throws ElementNotFoundException{
        DesignViewOperator view = editor.getDesignView();
        PatternOperator patRoot = view.getRootPatternOperator();
        //create Compensate Handler
        JPopupMenuOperator menu = patRoot.getPopup();
        menu.pushMenu("Add Fault Handlers");
        //create Compensate element
        PatternOperator compHandler = findHandler();
        view.createPatternFromPalette(type, new VisualElementOperator(compHandler));
    }
    
    PatternOperator findElement() throws ElementNotFoundException{
        PatternOperator compHandler = findHandler();
        PatternOperator elementPattern = null;
        long waitPatternTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
        JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 2000);
        try{
            elementPattern = new PatternOperator(compHandler, type, 0);
        } catch(TimeoutExpiredException e){
        } finally{
            JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatternTime);
        }
        return elementPattern;
    }
    
    PatternOperator findHandler() throws ElementNotFoundException{
        DesignViewOperator view = editor.getDesignView();
        PatternOperator compHandler = null;
        long waitPatternTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
        JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 2000);
        try{
            compHandler = new PatternOperator(view, "Fault Handlers", 0);
        } catch(TimeoutExpiredException e){
        } finally{
            JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatternTime);
        }
        return compHandler;
    }
    
//    class CompensateHandler extends PatternHandler{
//        public CompensateHandler(PatternTypes pt){
//            super(pt);
//            xPath = "/process/compensationHandler/compensate";
//            name = "Compensate1";
//
//        }
//
//        void createElement() throws ElementNotFoundException{
//            DesignViewOperator view = editor.getDesignView();
//            PatternOperator patRoot = view.getRootPatternOperator();
//            //create Compensate Handler
//            JPopupMenuOperator menu = patRoot.getPopup();
//            menu.pushMenu("Add Fault Handlers");
//            //create Compensate element
//            PatternOperator compHandler = findHandler();
//            view.createPatternFromPalette(type, new VisualElementOperator(compHandler));
//        }
//
//        PatternOperator findElement() throws ElementNotFoundException{
//            PatternOperator compHandler = findHandler();
//            PatternOperator elementPattern = null;
//            long waitPatternTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
//            JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 2000);
//            try{
//                elementPattern = new PatternOperator(compHandler, type, 0);
//            } catch(TimeoutExpiredException e){
//            } finally{
//                JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatternTime);
//            }
//            return elementPattern;
//        }
//
//        PatternOperator findHandler() throws ElementNotFoundException{
//            DesignViewOperator view = editor.getDesignView();
//            PatternOperator compHandler = null;
//            long waitPatternTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
//            JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 2000);
//            try{
//                compHandler = new PatternOperator(view, "Fault Handlers", 0);
//            } catch(TimeoutExpiredException e){
//            } finally{
//                JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatternTime);
//            }
//            return compHandler;
//        }
//
//    }
    
}
