/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.verification.tests;

import java.io.File;
import org.netbeans.jellytools.JellyTestCase;
import org.netbeans.jellytools.actions.SaveAction;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.operators.JButtonOperator;
import org.netbeans.jemmy.operators.JDialogOperator;
import org.netbeans.orchlib.core.operators.DesignViewOperator;
import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.VisualElementOperator;
import org.netbeans.test.bpel.verification.lib.DiagramManager;
import org.netbeans.test.bpel.verification.lib.ProjectManager;
import org.netbeans.test.bpel.verification.lib.Util;

/**
 * @author mikk@netbeans.org
 */
public class VerificationTest extends JellyTestCase{
    
    private EventTool eventTool = new EventTool();
    
    private final String PROJECT_NAME = "SampleBPELModule";
    private final String PROJECT_LOCATION = System.getProperty("xtest.workdir");
    private final String WSDL_LOCATION = System.getProperty("xtest.data")+
            File.separator+"data"+File.separator+"verification";
    private final String WSDL_NAME = "SimpleWSDL";
    private final String SCHEMA_NAME = "SimpleXMLSchema";
    private final String BPEL_NAME = "SimpleBPEL";
    private final String EDIT_MENU= "Edit";
    private final String CUSTOM_EDITOR_TTL= "Property Editor";
    private final String CREATE_PARTNER_LINK_TTL = "Create New Partner Link";
    
    public VerificationTest(String name) {
        super(name);
    }
    
    public void testCreateProject() throws Exception {
        ProjectManager.createProject(PROJECT_NAME, PROJECT_LOCATION);
        ProjectManager.verifyNewBPELModule(PROJECT_NAME, PROJECT_LOCATION);
    }
    
    public void testAddBPELToProject() throws Exception {
        ProjectManager.addBPELToProject(PROJECT_NAME, BPEL_NAME);
        Util.verifyProjectPath(PROJECT_NAME, Util.PROCESS_SOURCES_PATH+"|"+BPEL_NAME);
    }
    
    public void testImportWSDL() throws Exception {
        ProjectManager.addExternalWSDL(PROJECT_NAME, WSDL_LOCATION);
        Util.verifyProjectPath(PROJECT_NAME, Util.PROCESS_SOURCES_PATH+"|"+WSDL_NAME);
        Util.verifyProjectPath(PROJECT_NAME, Util.PROCESS_SOURCES_PATH+"|"+SCHEMA_NAME);
    }
    
    public void testAddPartnerLink() throws Exception {
        try{
            MultiviewEditorOperator multiView = ProjectManager.openBPEL(PROJECT_NAME, BPEL_NAME);
            DesignViewOperator designView = multiView.getDesignView();
            designView.createPatternFromPalette(PatternTypes.PARTNER_LINK, designView.getFreePoint());
            JDialogOperator opDlg = new JDialogOperator(CREATE_PARTNER_LINK_TTL);
            new JButtonOperator(opDlg, "OK").push();
            opDlg.waitClosed();
            
            //verification
            new PatternOperator(designView, PatternTypes.PARTNER_LINK, 0);
        }finally{
            try{
                new SaveAction().performAPI();
            }catch(Exception e){}
        }
    }
    
    public void testCreateReceive() throws Exception {
        try{
            MultiviewEditorOperator multiView = ProjectManager.openBPEL(PROJECT_NAME, BPEL_NAME);
            DesignViewOperator designView = multiView.getDesignView();           
            
            //TODO this was added to assure test stability. Consider removing
            eventTool.waitNoEvent(2000);
            
            VisualElementOperator placeholderEl = new VisualElementOperator(designView.getRootPatternOperator(), ElementTypes.PLACEHOLDER_ELEMENT, 0);
            designView.createPatternFromPalette(PatternTypes.RECEIVE, placeholderEl);            
            PatternOperator receivePattern = new PatternOperator(designView.getRootPatternOperator(), PatternTypes.RECEIVE, 0);
            
            DiagramManager.callPopupOnPattern(receivePattern, EDIT_MENU);
            JDialogOperator propertiesEditor = new JDialogOperator(CUSTOM_EDITOR_TTL);
            
            String receiveName = "ReceiveName";
            DiagramManager.setBasicReceiveProperties(propertiesEditor, receiveName);
            
            //verification
            receivePattern = new PatternOperator(designView.getRootPatternOperator(), PatternTypes.RECEIVE, 0);
            DiagramManager.verifyPatternName(receivePattern, receiveName);
        }finally{
            try{
                new SaveAction().performAPI();
            }catch(Exception e){}
        }
    }
    
    public void testCleanAndBuild() throws Exception {
        ProjectManager.cleanAndBuild(PROJECT_NAME);
        ProjectManager.verifyBuildSuccessfull(PROJECT_NAME);
    }
    
}
