/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


/*
 * PaletteOperator.java
 */

package org.netbeans.orchlib.core.operators;

import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.KeyEvent;
import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.SwingUtilities;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.utils.TypeSpecificProperties;
import org.netbeans.jellytools.TopComponentOperator;
import org.netbeans.jemmy.ComponentChooser;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.Timeout;
import org.netbeans.jemmy.drivers.input.MouseRobotDriver;
import org.netbeans.jemmy.operators.ContainerOperator;
import org.netbeans.jemmy.operators.JListOperator;
import org.netbeans.jemmy.operators.JToggleButtonOperator;

/**
 * This operator represents BPEL Palette and allows to perform basic actions with the palette.
 * @author Mikhail.Kondratyev@sun.com
 */
public class PaletteOperator extends TopComponentOperator {
    
    public final static String PALETTE_NAME = "Palette";//"Orchestration Palette";
    public final static String[] GROUPS = {"Web Service", "Basic Activities", "Structured Activities"};
    
    private EventTool eventTool = new EventTool();
    
    
    public PaletteOperator() {
        super(PALETTE_NAME);
    }
    
    
    public void selectPattern(String patternName) throws ElementNotFoundException{
        JListOperator listOperator = findGroupList(patternName);
        eventTool.waitNoEvent(500);
        int itemIndex = listOperator.findItemIndex(new PaletteListItemChooser(patternName));
        //some bug workaround
        listOperator.scrollToItem(itemIndex);
        listOperator.getTimeouts().setTimeout("JScrollBarOperator.WholeScrollTimeout",  500);
        listOperator.clickOnItem(itemIndex, 1);
        listOperator.getTimeouts().setTimeout("JScrollBarOperator.WholeScrollTimeout",  30000);
    }
    
    
    private JListOperator findGroupList(String patternName) throws ElementNotFoundException{
        expandAllGroups();
        eventTool.waitNoEvent(500);
        ComponentChooser listChooser = new JListByItemChooser(patternName);
        Component comp = waitSubComponent(listChooser);
        if (comp == null){
            throw new ElementNotFoundException();
        }
        return new JListOperator((JList)comp);
    }
    
    
    public void expandAllGroups(){
        for(String group: GROUPS){
            expandGroup(group);
        }
    }
    
    public Point pressMouseOnItem(String patternName) throws ElementNotFoundException{
        JListOperator listOperator = findGroupList(patternName);
        eventTool.waitNoEvent(500);
       
        int itemIndex = listOperator.findItemIndex(new PaletteListItemChooser(patternName));
        listOperator.scrollToItem(itemIndex);
        listOperator.getTimeouts().setTimeout("JScrollBarOperator.WholeScrollTimeout",  500);
        Point point = listOperator.getClickPoint(itemIndex);
        
        MouseRobotDriver dr = new MouseRobotDriver(new Timeout("", 50));
        dr.moveMouse(listOperator, point.x, point.y);
        dr.pressMouse(listOperator, point.x,  point.y, KeyEvent.BUTTON1_MASK, 0);
        //eventTool.waitNoEvent(1500);
        
        listOperator.getTimeouts().setTimeout("JScrollBarOperator.WholeScrollTimeout",  30000);
        SwingUtilities.convertPointToScreen(point, listOperator.getSource());
        return point;
    }
    
    
    public void selectPatternByType(Enum type) throws ElementNotFoundException{
        selectPattern(getPatternNameByType(type));
    }
    
    public Point pressMouseOnItem(Enum type) throws ElementNotFoundException{
        return pressMouseOnItem(getPatternNameByType(type));
    }
    
    
    public String getPatternNameByType(Enum patternType) {
        return TypeSpecificProperties.getCurrentPatternPaletteName(patternType);
    }
    
    
    public void expandGroup(String groupName){
        JToggleButtonOperator groupBtn = new JToggleButtonOperator(this, groupName);
        if (!groupBtn.isSelected()){
            groupBtn.pushNoBlock();
            groupBtn.waitSelected(true);
        }
    }
    
    
    public void collapseGroup(String groupName){
        JToggleButtonOperator groupBtn = new JToggleButtonOperator(this, groupName);
        if (groupBtn.isSelected()){
            groupBtn.pushNoBlock();
            groupBtn.waitSelected(false);
        }
    }
    
    public String getPatternGroupName(Enum patternType) throws ElementNotFoundException{
        String patternName = getPatternNameByType(patternType);
        JListOperator listOperator = findGroupList(patternName);
        Container group = listOperator.getSource().getParent();
//        JToggleButtonOperator groupBtn = new JToggleButtonOperator(new ContainerOperator(group));
        return new JToggleButtonOperator(new ContainerOperator(group)).getText();
    }
    
    
    
    private class PaletteListItemChooser implements JListOperator.ListItemChooser{
        String itemLabel;
        
        public PaletteListItemChooser(String itemLabel) {
            this.itemLabel = itemLabel;
        }
        
        public boolean checkItem(JListOperator oper, int index) {
            Object obj = oper.getModel().getElementAt(index);
            return itemLabel.equals(obj.toString());
        }
        
        public String getDescription() {
            return("Item equal to \"" + itemLabel + "\" string");
        }
    }
    
    
    
    class JListByItemChooser implements ComponentChooser {
        String itemName = null;
        
        
        JListByItemChooser(String itemName) {
            this.itemName = itemName.trim();
        }        
        
        /*
         * (non-Javadoc)
         *
         * @see org.netbeans.jemmy.ComponentChooser#checkComponent(java.awt.Component)
         */
        public boolean checkComponent(Component comp){
            
            if (comp instanceof JList){
                JList list = (JList)comp;
                ListModel model = list.getModel();
//                    getOutput().printLine("JList contains "+model.getSize()+" items");
                for(int i = 0; i < model.getSize(); i++) {
                    Object obj = model.getElementAt(i);
//                        getOutput().printLine("item "+i+": "+obj.toString());
                    if(itemName.equals(obj.toString()))
                        return true;
                }
            }
            return false;
        }
        
        public String getDescription(){
            return "Tool list by tool with name \""+itemName+"\" chooser";
        }
    }
    
}
