/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.orchlib.core.verifiers;

import java.io.PrintStream;
import javax.swing.tree.TreePath;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.utils.TypeSpecificProperties;
import org.netbeans.jemmy.operators.JButtonOperator;
import org.netbeans.jemmy.operators.JDialogOperator;
import org.netbeans.jemmy.operators.JTabbedPaneOperator;
import org.netbeans.jemmy.operators.JTableOperator;
import org.netbeans.jemmy.operators.JTextFieldOperator;
import org.netbeans.jemmy.operators.JTreeOperator;
import org.netbeans.jemmy.operators.Operator;
import org.netbeans.jemmy.util.Dumper;
import org.netbeans.modules.bpel.model.impl.CorrelationSetImpl;


/**
 * @author Andrey.Yamkovoy@sun.com
 */
public class CorrelationTabVerifier{
    
    private PatternVerifier pv = null;
    
    public CorrelationTabVerifier(PatternVerifier patVerifier){
	this.pv = patVerifier;
    }

    private void createCorrelationSet(String nameCS) throws ElementNotFoundException{
        PatternOperator process = pv.designView.getRootPatternOperator();
	process.clickActionPoint(2);
	String nameProcessType = TypeSpecificProperties.getCurrentPatternPaletteName(PatternTypes.PROCESS);
	JDialogOperator dlgEditor = new JDialogOperator(nameProcessType);
	
        JTabbedPaneOperator tabs = new JTabbedPaneOperator(dlgEditor);
	tabs.selectPage("Correlation Sets");
	pv.eventTool.waitNoEvent(500);
	JTreeOperator tree = new JTreeOperator(tabs);
	TreePath treePath = tree.getPathForRow(0);
	pv.log("treePath=" + treePath.toString());
	tree.selectPath(treePath);
	pv.eventTool.waitNoEvent(500);
	
	new JButtonOperator(tabs, "Add").pushNoBlock();
	
	JDialogOperator dlgAdd = new JDialogOperator("Add Correlation Set");
	JTextFieldOperator fieldName = new JTextFieldOperator(dlgAdd, 0);
	fieldName.setText(nameCS);
	new JButtonOperator(dlgAdd, "Ok").pushNoBlock();
	
	new JButtonOperator(dlgEditor, "Ok").pushNoBlock();
    }
    
    private JDialogOperator openCustomEditor_CST(PatternOperator element){
	element.getPopup().pushMenu(pv.PATTERN_CONTEXT_PROPERTY_EDITOR);
//	JDialogOperator editor = new JDialogOperator(pv.patternTypeName);
	JDialogOperator editor = new JDialogOperator("Property Editor");
	
        JTabbedPaneOperator tabs = new JTabbedPaneOperator(editor);
	tabs.selectPage("Correlations");
	
	return editor;
    }
    
    private void addCorrelationSetFromPropertiesDialog(String nameCS){
//	JDialogOperator dlgEditor = new JDialogOperator(pv.patternTypeName);
	JDialogOperator dlgEditor = new JDialogOperator("Property Editor");
	JTabbedPaneOperator tabs = new JTabbedPaneOperator(dlgEditor);
	JTableOperator table = new JTableOperator(tabs);
	int countCS = table.getRowCount();
	pv.log("Row count = " + countCS);
	new JButtonOperator(tabs, "Add").pushNoBlock();

	JDialogOperator dlgChoose = new JDialogOperator("Choose a Correlation Set");
	JTreeOperator tree = new JTreeOperator(dlgChoose);
	TreePath treePath = tree.findPath("Correlation Sets|" + nameCS, new Operator.DefaultStringComparator(true, true));
	pv.log("SelectedPath = " + tree.getSelectionPath().toString());
	tree.selectPath(treePath);
	new JButtonOperator(dlgChoose, "Ok").pushNoBlock();
	dlgChoose.waitClosed();
    }

    private void editCorrelationSetFromPropertiesDialog(String nameCS, String nameCSNew){
//	JDialogOperator dlgEditor = new JDialogOperator(pv.patternTypeName);
	JDialogOperator dlgEditor = new JDialogOperator("Property Editor");
	JTabbedPaneOperator tabs = new JTabbedPaneOperator(dlgEditor);
	JTableOperator table = new JTableOperator(tabs);
	int countCS = table.getRowCount();
	pv.log("Row count = " + countCS);
	
	table.selectCell(0, 0);
	new JButtonOperator(tabs, "Edit").push();

	JDialogOperator dlgEdit = new JDialogOperator("Edit Correlation Set");
	JTextFieldOperator fieldName = new JTextFieldOperator(dlgEdit, 0);
	fieldName.setText(nameCSNew);
	
	new JButtonOperator(dlgEdit, "Ok").pushNoBlock();
	dlgEdit.waitClosed();
    }
    
    private void deleteCorrelationSetFromPropertiesDialog(String nameCS){
//	JDialogOperator dlgEditor = new JDialogOperator(pv.patternTypeName);
	JDialogOperator dlgEditor = new JDialogOperator("Property Editor");
	JTabbedPaneOperator tabs = new JTabbedPaneOperator(dlgEditor);
	JTableOperator table = new JTableOperator(tabs);
	int countCS = table.getRowCount();
	pv.log("Row count = " + countCS);

	table.selectCell(0, 0);
	new JButtonOperator(tabs, "Delete").push();
    }
    
// ------- Check methods ---------------------------    
    
    public boolean checkAddCorrelationSet(){
	String patName;
	PatternOperator element = null;
	String nameCS = "MyCorSet1";
	
	try{
	    createCorrelationSet(nameCS);
	    element = pv.createPattern();
	    patName = element.getName();
	    JDialogOperator dlgEditor = openCustomEditor_CST(element);
	    
            JTabbedPaneOperator tabs = new JTabbedPaneOperator(dlgEditor);
	    JTableOperator table = new JTableOperator(tabs);
	    int countCS = table.getRowCount();
	    pv.log("Row count = " + countCS);

	    addCorrelationSetFromPropertiesDialog(nameCS);
	    
	    pv.log("Row count = " + (table.getRowCount()));
	    
	    if (countCS != (table.getRowCount() - 1)){
		pv.log("Was - " + countCS + ", after add one more we have - " + table.getRowCount());
		return false;
	    }
	    
	    CorrelationSetImpl corSet = (CorrelationSetImpl)table.getValueAt(0,0);
	    pv.log("Cell value = " + corSet.getName());
	    
	    if (!nameCS.equals(corSet.getName())){
		pv.log("Wrong Correlation Set name. Found " + corSet.getName() + " but expected " + nameCS);
		return false;
	    }
	    
	    new JButtonOperator(dlgEditor, "Ok").pushNoBlock();
	    dlgEditor.waitClosed();
	    
	    return true;
	}catch(Exception e){
	    if (pv.log != null){
		e.printStackTrace(pv.log);
	    }
	    return false;
	}
    }

    public boolean checkEditCorrelationSet(){
	String patName;
	PatternOperator element = null;
	String nameCS = "MyCorSet2";
	String nameCSNew = "MyCorSet2_New";
	
	try{
	    createCorrelationSet(nameCS);
	    element = pv.createPattern();
	    patName = element.getName();
	    JDialogOperator dlgEditor = openCustomEditor_CST(element);
	    
            JTabbedPaneOperator tabs = new JTabbedPaneOperator(dlgEditor);
	    JTableOperator table = new JTableOperator(tabs);
	    int countCS = table.getRowCount();
	    pv.log("Row count = " + countCS);

	    addCorrelationSetFromPropertiesDialog(nameCS);
	    
	    editCorrelationSetFromPropertiesDialog(nameCS, nameCSNew);
	    
	    pv.log("Row count = " + (table.getRowCount()));
	    if (countCS != (table.getRowCount() - 1)){
		pv.log("Was - " + countCS + ", after add one more and edit it we have - " + table.getRowCount());
		return false;
	    }
	    
	    CorrelationSetImpl corSet = (CorrelationSetImpl)table.getValueAt(0,0);
	    pv.log("Cell value = " + corSet.getName());
	    
	    if (!nameCSNew.equals(corSet.getName())){
		pv.log("Wrong Correlation Set name. Found " + corSet.getName() + " but expected " + nameCSNew);
		return false;
	    }
	    
	    new JButtonOperator(dlgEditor, "Ok").pushNoBlock();
	    dlgEditor.waitClosed();
	    
	    return true;
	}catch(Exception e){
	    if (pv.log != null){
		e.printStackTrace(pv.log);
	    }
	    return false;
	}
    }    
    
    public boolean checkDeleteCorrelationSet(){
	String patName;
	PatternOperator element = null;
	String nameCS = "MyCorSet3";
	
	try{
	    createCorrelationSet(nameCS);
	    element = pv.createPattern();
	    patName = element.getName();
	    JDialogOperator dlgEditor = openCustomEditor_CST(element);
	    
            JTabbedPaneOperator tabs = new JTabbedPaneOperator(dlgEditor);
	    JTableOperator table = new JTableOperator(tabs);
	    int countCS = table.getRowCount();
	    pv.log("Row count = " + countCS);

	    addCorrelationSetFromPropertiesDialog(nameCS);
	    
	    deleteCorrelationSetFromPropertiesDialog(nameCS);
	    
	    pv.log("Row count = " + (table.getRowCount()));
	    if (countCS != table.getRowCount()){
		pv.log("Was - " + countCS + ", after add one more and delete we have - " + table.getRowCount());
		return false;
	    }
	    if (table.getRowCount()!=0 && nameCS.equals(table.getValueAt(0,0).toString())){
		pv.log("After deletion still have Correlation Set with name " + table.getValueAt(0,0).toString());
		return false;
	    }
	    
	    new JButtonOperator(dlgEditor, "Ok").pushNoBlock();
	    dlgEditor.waitClosed();
	    
	    return true;
	}catch(Exception e){
	    if (pv.log != null){
		e.printStackTrace(pv.log);
	    }
	    return false;
	}
    }    
}

