/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.orchlib.core.verifiers;

import java.awt.event.KeyEvent;
import java.io.PrintStream;
import org.netbeans.orchlib.core.operators.ConnectionOperator;
import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.VisualElementOperator;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.TimeoutExpiredException;
import org.netbeans.jemmy.operators.Operator;

/**
 * @author Andrey.Yamkovoy@sun.com
 */
public class SubPatternVerifier extends PatternVerifier{
    
    private PatternTypes patternRootType = null;
    private String patternRootName = null;

    public SubPatternVerifier(MultiviewEditorOperator editor, PatternTypes patRootType, PatternTypes patType){
	this(editor, patRootType, patType, null);
    }

    public SubPatternVerifier(MultiviewEditorOperator editor, PatternTypes patRootType, PatternTypes patType, PrintStream log){
	this(editor, patRootType, patType, "", log);
    }
    
    public SubPatternVerifier(MultiviewEditorOperator editor, PatternTypes patRootType, PatternTypes patType, String nameRootPref, PrintStream log) {
	super(editor, patType, nameRootPref, log);
	this.patternRootType = patRootType;
	this.patternRootName = nameRootPref + "_C";
	
	log("patternTypeName = " + patternTypeName);
    }

    public PatternOperator createPattern() throws ElementNotFoundException{
	PatternOperator patProcess = designView.getRootPatternOperator();
	PatternOperator patRoot = null;
        long waitPatternTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
        JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 2000);
	try{
	    patRoot = new PatternOperator(patProcess , patternRootType, 0);
	}catch(TimeoutExpiredException e){
	    PatternOperator patTarget = new PatternOperator(patProcess, PatternTypes.SEQUENCE, 0);
	    VisualElementOperator el1 = new VisualElementOperator(patProcess, ElementTypes.START_EVENT, 0);
	    VisualElementOperator el2 = new VisualElementOperator(patTarget, ElementTypes.ANY, 0);             
	    ConnectionOperator conn = new ConnectionOperator(el1, el2);

	    designView.createPatternFromPalette(patternRootType, conn, patTarget);
	    
	    patRoot = new PatternOperator(patTarget, patternRootType, 0);
	}finally{
	    JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatternTime);
	}
        patRoot.select();
	patRoot.getPopup().pushMenuNoBlock("Add " + patternTypeName);
	eventTool.waitNoEvent(1000);
	
	return (patRoot.getAllChildPatternsCount(patternType) > 0)?new PatternOperator(patRoot, patternType, patRoot.getAllChildPatternsCount(patternType) - 1):null;
    }

    
// ------- Check methods ---------------------------    
    
    
}
