/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * FlowCreateAction.java
 *
 */

package org.netbeans.orchlib.ext.creator.actions;
import java.awt.Point;
import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.VisualElementOperator;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.ext.creator.exceptions.CreationVerificationException;
import org.netbeans.jemmy.EventTool;

/**
 *
 * @author Mikhail.Kondratyev@sun.com
 */
public class FlowCreateAction extends CreateAction{
    
    /** Creates a new instance of FlowCreateAction */
    public FlowCreateAction() {
    }
    
    public PatternOperator perform(PatternOperator pattern, PatternTypes type) throws ElementNotFoundException, CreationVerificationException{
        int countTypeBefore = pattern.getChildPatternsCount(type);
        int countPatterns = pattern.getChildPatternsCount();
        
        if (countPatterns==0){            
            VisualElementOperator placeHolder = pattern.getDirectChildElements(ElementTypes.PLACEHOLDER_ELEMENT).get(0);
            int connectionsCountBefore = pattern.getFirstElement().getOutgoingConnections().size();
            pattern.getDesignView().createPatternFromPalette(type, placeHolder);
            
            //verification
            int countTypeAfter = pattern.getChildPatternsCount(type);
            if (countTypeAfter-countTypeBefore!=1){
                throw new CreationVerificationException("No pattern of type "+type.name()+"was added to Flow.");
            }
            return pattern.getDirectChildPattern(type, countTypeAfter-1);
        }else{
            //this is a difficult case as we do not know where the place holder will appear. There are
            //several possible solutions. For example we can start dragging, understand the position and then
            //drag there
            //In the implementation below we chose another way and just try to predict the point based on the 
            //flow pattern coordinates
            int y = (int)(pattern.getBoundingRect().getCenterY());
            int x = (int)(pattern.getBoundingRect().getMaxX()-10);
            Point point = new Point(x,y);
            pattern.getDesignView().createPatternFromPalette(type, point);
            
            //verification
            int countTypeAfter = pattern.getChildPatternsCount(type);
            if (countTypeAfter-countTypeBefore!=1){
                throw new CreationVerificationException("No pattern of type "+type.name()+"was added to Flow.");
            }
            return pattern.getDirectChildPattern(type, countTypeAfter-1);
        }
    }
}
