/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * LogActionFactory.java 
 */

package org.netbeans.orchlib.ext.creator.actions;

import java.util.Hashtable;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.w3c.dom.Element;

/**
 * The factory is used to create appropriate log action 
 * every time new tag is read from BPEL source
 * @author Mikhail.Kondratyev@sun.com
 */
public class LogActionFactory {
    
    public static LogActionFactory getInstance(){
        return instance;
    }
    
    
    public LogAction createAction(PatternOperator parentPattern, Element element){
        Hashtable<String, LogAction> hash = actionsByPattern.get(parentPattern.getTypeString());
        if (hash!=null){
            LogAction action = hash.get(element.getTagName());            
            return action;
        }else{
            return null;
        }
    }
    
    
    /** Creates a new instance of CreateActionFactory */
    private LogActionFactory() {
        Hashtable<String, LogAction> actionsByTag = new Hashtable<String, LogAction>();        
        actionsByPattern.put(PatternTypes.IF.toString(), actionsByTag);
        actionsByTag.put("elseif", new ElseIfLogAction());
        
        actionsByTag = new Hashtable<String, LogAction>();
        actionsByPattern.put(PatternTypes.PICK.toString(), actionsByTag);
        actionsByTag.put("onMessage", new OnMessageLogAction());
        actionsByTag.put("onAlarm", new OnAlarmLogAction());
        
        actionsByTag = new Hashtable<String, LogAction>();
        actionsByPattern.put(PatternTypes.PROCESS.toString(), actionsByTag);
        actionsByTag.put("faultHandlers", new FaultHandlersLogAction());
        
        actionsByTag = new Hashtable<String, LogAction>();
        actionsByPattern.put(PatternTypes.EVENT_HANDLER.toString(), actionsByTag);
        actionsByTag.put("onMessage", new OnMessageLogAction());
        actionsByTag.put("onAlarm", new OnAlarmLogAction());
        
        actionsByTag = new Hashtable<String, LogAction>();
        actionsByPattern.put(PatternTypes.FAULT_HANDLER.toString(), actionsByTag);
        actionsByTag.put("catch", new CatchLogAction());
        actionsByTag.put("catchAll", new CatchAllLogAction());
        
    }
    
    private static LogActionFactory instance = new LogActionFactory();
    
    private Hashtable<String, Hashtable<String, LogAction>> actionsByPattern = new Hashtable<String, Hashtable<String, LogAction>>();
    private Hashtable<String, LogAction> actionsByTag = new Hashtable<String, LogAction>();
    
    
}
