/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * NameActionFactory.java
 *
 */

package org.netbeans.orchlib.ext.creator.actions;

import java.util.Hashtable;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.w3c.dom.Element;

/**
 * The factory used to create appropriate namers for each of the patterns (activities)
 * @author Mikhail.Kondratyev@sun.com
 */
public class NameActionFactory {
    
    public static NameActionFactory getInstance(){
        return instance;
    }    
    
    /**
     * Creates the appropriate action. Currently default NameAction is returned if
     * no specific action is specified for an activity
     * @param parentPattern Parent pattern
     * @param element Dom element that was created
     * @return Returns the appropriate action. NameAction is returned by default
     */
    public NameAction createAction(PatternOperator parentPattern, Element element){
        Hashtable<String, NameAction> hash = actionsByPattern.get(parentPattern.getTypeString());
        if (hash!=null){
            NameAction action = hash.get(element.getTagName());
            if (action==null){
                action = new NameAction();
            }
            return action;
        }else{
            return new NameAction();
        }
    }
    
  /** Creates a new instance of CreateActionFactory */
    private NameActionFactory() {
        Hashtable<String, NameAction> actionsByTag = new Hashtable<String, NameAction>();        
        actionsByPattern.put(PatternTypes.SWITCH.toString(), actionsByTag);
        actionsByTag.put("case", new DummyNameAction());
        
        actionsByTag = new Hashtable<String, NameAction>();
        actionsByPattern.put(PatternTypes.PICK.toString(), actionsByTag);
        actionsByTag.put("onMessage", new DummyNameAction());
        actionsByTag.put("onAlarm", new DummyNameAction());
        
        actionsByTag = new Hashtable<String, NameAction>();
        actionsByPattern.put(PatternTypes.PROCESS.toString(), actionsByTag);
        actionsByTag.put("eventHandlers", new DummyNameAction());
        actionsByTag.put("compensationHandler", new DummyNameAction());
        actionsByTag.put("eventHandlers", new DummyNameAction());
        
        actionsByTag = new Hashtable<String, NameAction>();
        actionsByPattern.put(PatternTypes.EVENT_HANDLER.toString(), actionsByTag);
        actionsByTag.put("onMessage", new DummyNameAction());
        actionsByTag.put("onAlarm", new DummyNameAction());
        
        actionsByTag = new Hashtable<String, NameAction>();
        actionsByPattern.put(PatternTypes.FAULT_HANDLER.toString(), actionsByTag);
        actionsByTag.put("catch", new DummyNameAction());
        actionsByTag.put("catchAll", new DummyNameAction());
        
    }
    
    private static NameActionFactory instance = new NameActionFactory();
    
    private Hashtable<String, Hashtable<String, NameAction>> actionsByPattern = new Hashtable<String, Hashtable<String, NameAction>>();
    private Hashtable<String, NameAction> actionsByTag = new Hashtable<String, NameAction>();
    
}
