/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * PickOnMessageCreateAction.java
 *
 */

package org.netbeans.orchlib.ext.creator.actions;

import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.ext.creator.exceptions.CreationVerificationException;
import org.netbeans.jemmy.EventTool;

/**
 *
 * @author Mikhail.Kondratyev@sun.com
 */
public class PickOnMessageCreateAction extends CreateAction{
    
    /** Creates a new instance of PickOnMessageCreateAction */
    public PickOnMessageCreateAction() {
    }
    
    private EventTool eventTool = new EventTool();
    
    
    public PatternOperator perform(PatternOperator pattern, PatternTypes type) throws ElementNotFoundException, CreationVerificationException{
        
        int count = pattern.getChildPatternsCount();
        
        //the first case if this is a new pick. That means the pick has exactly one onMessage
        //and that message has exactly 2 elements and one of them is placeholder
        boolean hasEmptyOnMessage = false;
        int initialMessagePatternCount = pattern.getChildPatternsCount(PatternTypes.ON_MESSAGE);
        if (initialMessagePatternCount==1){
            //checking this message has placeholder inside
            PatternOperator casePattern = pattern.getDirectChildPattern(PatternTypes.ON_MESSAGE, 0);
            int placeHolderElementsCount = casePattern.getDirectChildElements(ElementTypes.PLACEHOLDER_ELEMENT).size();
            if (placeHolderElementsCount==1){
                hasEmptyOnMessage = true;
            }
        }
        
        if (hasEmptyOnMessage){
            PatternOperator casePattern = pattern.getDirectChildPattern(PatternTypes.ON_MESSAGE, 0);
            return casePattern;
        }else{
            pattern.select();
            eventTool.waitNoEvent(1000);
            pattern.getPopup().pushMenuNoBlock("Add|On Message");
            eventTool.waitNoEvent(1000);
            //verification
            int onMessagePatternsCount = pattern.getChildPatternsCount(PatternTypes.ON_MESSAGE);
            if (onMessagePatternsCount-initialMessagePatternCount!=1){
                throw new CreationVerificationException("Failed to add new onMessage to pick called "+pattern.getName());
            }
            return pattern.getDirectChildPattern(PatternTypes.ON_MESSAGE, onMessagePatternsCount-1);
        }
    }    
}
