/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * SequenceCreateAction.java
 *
 */

package org.netbeans.orchlib.ext.creator.actions;

import java.awt.Point;
import org.netbeans.orchlib.core.operators.ConnectionOperator;
import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.VisualElementOperator;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.ext.creator.exceptions.CreationVerificationException;
import org.netbeans.jemmy.EventTool;

/**
 *
 * @author Mikhail.Kondratyev@sun.com
 */
public class SequenceCreateAction extends CreateAction{
    
    /** Creates a new instance of SequenceCreateAction */
    public SequenceCreateAction() {
    }
    
    
    public PatternOperator perform(PatternOperator pattern, PatternTypes type) throws ElementNotFoundException, CreationVerificationException{
        int count = pattern.getChildPatternsCount();
        if (count==0){
            VisualElementOperator placeHolder = new VisualElementOperator(pattern, ElementTypes.PLACEHOLDER_ELEMENT, 0);
            pattern.getDesignView().createPatternFromPalette(type, placeHolder);
            
            //verification
            if (pattern.getChildPatternsCount()==0){
                throw new CreationVerificationException("Failed to create first activity (activity type - "+type.name()+") in Sequence called "+pattern.getName());
            }
            return new PatternOperator(pattern, type, 0);
        }else{
            System.err.println("want to create"+type);
            PatternOperator lastPattern = pattern.getDirectChildPattern(PatternTypes.ANY, count-1);
            System.err.println("last is "+lastPattern.getName());
            System.err.println("count "+count);
            VisualElementOperator lastElement = lastPattern.getLastElement();
            System.err.println("element"+lastElement.getName()+" "+lastElement.getTypeString());
            int size = lastElement.getOutgoingConnections().size();
            if (size>0){
                //usual case
                ConnectionOperator connection = lastElement.getOutgoingConnections().get(0);
                pattern.getDesignView().createPatternFromPalette(type, connection, pattern);                
            } else{
                //rare case when container is a handler
                int yPos = (((int)lastPattern.getBoundingRect().getMaxY())+((int)pattern.getBoundingRect().getMaxY()))/2;
                int xPos = (int)lastPattern.getBoundingRect().getCenterX();
                Point target = new Point(xPos, yPos);
                pattern.getDesignView().createPatternFromPalette(type, target);
            }
            
            //verification
            int newCount = pattern.getChildPatternsCount();
            if (newCount-count!=1){
                throw new CreationVerificationException("Failed to add activity (activity type - "+type.name()+") to Sequence called "+pattern.getName());
            }
            return pattern.getDirectChildPattern(PatternTypes.ANY, newCount-1);
        }
        
    }
    
    
    
}
