/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * SwitchCaseCreateAction.java
 *
 */

package org.netbeans.orchlib.ext.creator.actions;

import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.ext.creator.exceptions.CreationVerificationException;
import org.netbeans.jemmy.EventTool;

/**
 *
 * @author Mikhail.Kondratyev@sun.com
 */
public class SwitchCaseCreateAction extends CreateAction{
    
    /** Creates a new instance of SequenceCreateAction */
    public SwitchCaseCreateAction() {
    }
    
    private EventTool eventTool = new EventTool();
    
    public PatternOperator perform(PatternOperator pattern, PatternTypes type) throws ElementNotFoundException, CreationVerificationException{
        int count = pattern.getChildPatternsCount();
        
        //the first case if this is a new switch. That means the switch has exactly one case
        //and that case has exactly one element (placeholder) inside
        boolean hasEmptyCase = false;
        int initialCasePatternCount = pattern.getChildPatternsCount(PatternTypes.SWITCH_CASE);
        if (initialCasePatternCount==1){
            //checking this case has placeholder inside
            PatternOperator casePattern = pattern.getDirectChildPattern(PatternTypes.SWITCH_CASE, 0);
            int totalElementsCount = casePattern.getChildElementsCount();
            int placeHolderElementsCount = casePattern.getDirectChildElements(ElementTypes.PLACEHOLDER_ELEMENT).size();
            if (totalElementsCount==placeHolderElementsCount && totalElementsCount==1){
                hasEmptyCase = true;
            }
        }            
        
        if (hasEmptyCase){
            PatternOperator casePattern = pattern.getDirectChildPattern(PatternTypes.SWITCH_CASE, 0);
            return casePattern;
        }else{
            pattern.getPopup().pushMenuNoBlock("Add Case");
            eventTool.waitNoEvent(1000);
            
            //verification            
            int casePatternsCount = pattern.getChildPatternsCount(PatternTypes.SWITCH_CASE);
            if (casePatternsCount-initialCasePatternCount!=1){
                throw new CreationVerificationException("Failed to add new case to switch called "+pattern.getName());
            }
            return pattern.getDirectChildPattern(PatternTypes.SWITCH_CASE, casePatternsCount-1);            
        }        
    }
}
    