/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * ExtJTreeOperator.java
 */

package org.netbeans.orchlib.ext.operators;
import javax.swing.JPopupMenu;
import javax.swing.tree.TreePath;
import org.netbeans.jemmy.operators.ContainerOperator;
import org.netbeans.jemmy.operators.JTreeOperator;
import org.netbeans.jemmy.operators.Operator;
import org.netbeans.jellytools.nodes.Node;


/**
 * This is an utility class and is an extension of the standard Jemmy JTreeOperator with a few
 * additional path management methods
 * @author Alexander.Pepin@sun.com
 */
public class ExtJTreeOperator extends JTreeOperator{
    
    public ExtJTreeOperator(ContainerOperator cont) {
        super(cont);
    }
    
    public TreePath findPathEx(String... path ){
        return findPath(path, new Operator.DefaultStringComparator(true, true));
    }
    
    public TreePath findPathEx(TreePath parentPath, String... subpath){
        Object[] path = parentPath.getPath();
        getOutput().printTrace("parentPath = "+path);
        int length = path.length - 1;    
        String[] fullpath = new String[path.length-1+subpath.length];

        for(int i=0; i<fullpath.length; i++){
            fullpath[i] = i<length? path[i+1].toString(): subpath[i-length];
        }       

        return findPathEx(fullpath);
    }
    
    public TreePath findPathEx(StringComparator comparator, String... path){
        return findPath(path, comparator);
    }
    
    public void selectPath(String... path ){
        selectPath(findPathEx(path));
    }
    
    public void selectPath(TreePath parentPath, String... subpath){
        selectPath(findPathEx(parentPath, subpath));
    }
    
    public void expandPath(String... path ){
        expandPath(findPathEx(path));
    }

    public void expandPath(TreePath parentPath, String... path ){
        expandPath(findPathEx(parentPath, path));
    }
    
    public JPopupMenu callPopupOnPath(String... path){
        return callPopupOnPath(findPathEx(path));
    }
    
    public JPopupMenu callPopupOnPath(TreePath parentPath, String... path){
        return callPopupOnPath(findPathEx(parentPath, path));
    }
    
    public String getNodeFilename(TreePath path){
        Node node = new Node(this, path);
        org.openide.nodes.Node nbNode = (org.openide.nodes.Node)node.getOpenideNode();
        String name = nbNode.getName();
        getOutput().printTrace("nbNode filename is "+name);
        return name;
    }
    
}
