/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.util.Hashtable;

public class OpcodeTable {

	final static Class _NULL = null;
	final static Class _N = OpcodeNoArg.class;
	final static Class _S1 = OpcodeSignedOneArg.class;
	final static Class _U1 = OpcodeUnsignedOneArg.class;
	final static Class _U1CP = OpcodeUnsignedOneArgCP.class;
	final static Class _U1LV = OpcodeUnsignedOneArg.class;
	final static Class _S2 = OpcodeSignedTwoArg.class;
	final static Class _S2JMP = OpcodeSignedTwoArg.class;
	final static Class _U2 = OpcodeUnsignedTwoArg.class;
	final static Class _U2CP = OpcodeUnsignedTwoArgCP.class;
	final static Class _S4 = OpcodeSignedFourArg.class;
	final static Class _S4JMP = OpcodeSignedFourArg.class;
	final static Class _II = OpcodeIInc.class;
	final static Class _T = OpcodeTableSwitch.class;
	final static Class _L = OpcodeLookupSwitch.class;
	final static Class _IN = OpcodeInvokeInterface.class;
	final static Class _M = OpcodeMultiANewArray.class;
	final static Class _NA = OpcodeNewArray.class;
	
	final static int OPCODE_WIDE = 196;
	final static int OPCODE_IINC = 132;
	final static int OPCODE_WIDE_LOAD_LOW = 21;
	final static int OPCODE_WIDE_LOAD_HIGH = 25;
	final static int OPCODE_WIDE_STORE_LOW = 54;
	final static int OPCODE_WIDE_STORE_HIGH = 58;

	public final static Object[] opcodes = {
	// 0
		"nop", _N,
		"aconst_null", _N,
		"iconst_m1", _N,
		"iconst_0", _N,
		"iconst_1", _N,
	// 5
		"iconst_2", _N,
		"iconst_3", _N,
		"iconst_4", _N,
		"iconst_5", _N,
		"lconst_0", _N,
	// 10
		"lconst_1", _N,
		"fconst_0", _N,
		"fconst_1", _N,
		"fconst_2", _N,
		"dconst_0", _N,
	// 15
		"dconst_1", _N,
		"bipush", _S1,
		"sipush", _S2,
		"ldc", _U1CP,
		"ldc_w", _U2CP,
	// 20
		"ldc2_w", _U2CP,
		"iload", _U1LV,
		"lload", _U1LV,
		"fload", _U1LV,
		"dload", _U1LV,
	// 25
		"aload", _U1LV,
		"iload_0", _N,
		"iload_1", _N,
		"iload_2", _N,
		"iload_3", _N,
	// 30
		"lload_0", _N,
		"lload_1", _N,
		"lload_2", _N,
		"lload_3", _N,
		"fload_0", _N,
	// 35
		"fload_1", _N,
		"fload_2", _N,
		"fload_3", _N,
		"dload_0", _N,
		"dload_1", _N,
	// 40
		"dload_2", _N,
		"dload_3", _N,
		"aload_0", _N,
		"aload_1", _N,
		"aload_2", _N,
	// 45
		"aload_3", _N,
		"iaload", _N,
		"laload", _N,
		"faload", _N,
		"daload", _N,
	// 50
		"aaload", _N,
		"baload", _N,
		"caload", _N,
		"saload", _N,
		"istore", _U1LV,
	// 55
		"lstore", _U1LV,
		"fstore", _U1LV,
		"dstore", _U1LV,
		"astore", _U1LV,
		"istore_0", _N,
	// 60
		"istore_1", _N,
		"istore_2", _N,
		"istore_3", _N,
		"lstore_0", _N,
		"lstore_1", _N,
	// 65
		"lstore_2", _N,
		"lstore_3", _N,
		"fstore_0", _N,
		"fstore_1", _N,
		"fstore_2", _N,
	// 70
		"fstore_3", _N,
		"dstore_0", _N,
		"dstore_1", _N,
		"dstore_2", _N,
		"dstore_3", _N,
	// 75
		"astore_0", _N,
		"astore_1", _N,
		"astore_2", _N,
		"astore_3", _N,
		"iastore", _N,
	// 80
		"lastore", _N,
		"fastore", _N,
		"dastore", _N,
		"aastore", _N,
		"bastore", _N,
	// 85
		"castore", _N,
		"sastore", _N,
		"pop", _N,
		"pop2", _N,
		"dup", _N,
	// 90
		"dup_x1", _N,
		"dup_x2", _N,
		"dup2", _N,
		"dup2_x1", _N,
		"dup2_x2", _N,
	// 95
		"swap", _N,
		"iadd", _N,
		"ladd", _N,
		"fadd", _N,
		"dadd", _N,
	// 100
		"isub", _N,
		"lsub", _N,
		"fsub", _N,
		"dsub", _N,
		"imul", _N,
	// 105
		"lmul", _N,
		"fmul", _N,
		"dmul", _N,
		"idiv", _N,
		"ldiv", _N,
	// 110
		"fdiv", _N,
		"ddiv", _N,
		"irem", _N,
		"lrem", _N,
		"frem", _N,
	// 115
		"drem", _N,
		"ineg", _N,
		"lneg", _N,
		"fneg", _N,
		"dneg", _N,
	// 120	
		"ishl", _N,
		"lshl", _N,
		"ishr", _N,
		"lshr", _N,
		"iushr", _N,
	// 125
		"lushr", _N,
		"iand", _N,
		"land", _N,
		"ior", _N,
		"lor", _N,
	// 130	
		"ixor", _N,
		"lxor", _N,
		"iinc", _II,
		"i2l", _N,
		"i2f", _N,
	// 135	
		"i2d", _N,
		"l2i", _N,
		"l2f", _N,
		"l2d", _N,
		"f2i", _N,
	// 140	
		"f2l", _N,
		"f2d", _N,
		"d2i", _N,
		"d2l", _N,
		"d2f", _N,
	// 145	
		"i2b", _N,
		"i2c", _N,
		"i2s", _N,
		"lcmp", _N,
		"fcmpl", _N,
	// 150	
		"fcmpg", _N,
		"dcmpl", _N,
		"dcmpg", _N,
		"ifeq", _S2JMP,
		"ifne", _S2JMP,
	// 155	
		"iflt", _S2JMP,
		"ifge", _S2JMP,
		"ifgt", _S2JMP,
		"ifle", _S2JMP,
		"if_icmpeq", _S2JMP,
	// 160	
		"if_icmpne", _S2JMP,
		"if_icmplt", _S2JMP,
		"if_icmpge", _S2JMP,
		"if_icmpgt", _S2JMP,
		"if_icmple", _S2JMP,
	// 165	
		"if_acmpeq", _S2JMP,
		"if_acmpne", _S2JMP,
		"goto", _S2JMP,
		"jsr", _S2JMP,
		"ret", _U1LV,
	// 170	
		"tableswitch", _T,
		"lookupswitch", _L,
		"ireturn", _N,
		"lreturn", _N,
		"freturn", _N,
	// 175	
		"dreturn", _N,
		"areturn", _N,
		"return", _N,
		"getstatic", _U2CP,
		"putstatic", _U2CP,
	// 180	
		"getfield", _U2CP,
		"putfield", _U2CP,
		"invokevirtual", _U2CP,
		"invokespecial", _U2CP,
		"invokestatic", _U2CP,
	// 185	
		"invokeinterface", _IN,
		null, _NULL,
		"new", _U2CP,
		"newarray", _NA,
		"anewarray", _U2CP,
	// 190	
		"arraylength", _N,
		"athrow", _N,
		"checkcast", _U2CP,
		"instanceof", _U2CP,
		"monitorenter", _N,
	// 195	
		"monitorexit", _N,
		"wide", _N,
		"multianewarray", _M,
		"ifnull", _S2JMP,
		"ifnonnull", _S2JMP,
	// 200	
		"goto_w", _S4JMP,
		"jsr_w", _S4JMP,
		"breakpoint", _NULL,
		"ldc_quick", _NULL,
		"ldc_w_quick", _NULL,
	// 205
		"ldc2_w_quick", _NULL,
		"getfield_quick", _NULL,
		"putfield_quick", _NULL,
		"getfield2_quick", _NULL,
		"putfield2_quick", _NULL,
	// 210
		"getstatic_quick", _NULL,
		"putstatic_quick", _NULL,
		"getstatic2_quick", _NULL,
		"putstatic2_quick", _NULL,
		"invokevirtual_quick", _NULL,
	// 215
		"invokenonvirtual_quick", _NULL,
		"invokesuper_quick", _NULL,
		"invokestatic_quick", _NULL,
		"invokeinterface_quick", _NULL,
		"invokevirtualobject_quick", _NULL,
	// 220
		null, _NULL,
		"new_quick", _NULL,
		"anewarray_quick", _NULL,
		"multianewarray_quick", _NULL,
		"checkcast_quick", _NULL,
	// 225
		"instanceof_quick", _NULL,
		"invokevirtual_quick_w", _NULL,
		"getfield_quick_w", _NULL,
		"putfield_quick_w", _NULL,
		null, _NULL,
	// 230
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
	// 235
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
	// 240
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
	// 245
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
	// 250
		null, _NULL,
		null, _NULL,
		null, _NULL,
		null, _NULL,
		"impdep1", _NULL,
	// 255
		"impdep2", _NULL
	};
	
	static Hashtable table;

	public static String getNameByOpcode (int opcode) {
		if (opcode < 0  ||  opcode >= 256  ||  opcodes[opcode * 2 + 1] == _NULL)
			return null;
		return (String) opcodes[opcode * 2];
	}

	public static Class getClassByOpcode (int opcode) {
		if (opcode < 0  ||  opcode >= 256)
			return null;
		return (Class) opcodes[opcode * 2 + 1];
	}
	
	public static int getOpcodeByName (String name) {
		Integer i = (Integer) table.get (name);
		if (i == null)
			return -1;
		return i.intValue ();
	}
	
	static {
		table = new Hashtable ();
		for (int a = 0; a < (opcodes.length >> 1); a ++) {
			Object o = getClassByOpcode (a);
			if (o == _NULL)
				continue;
			o = getNameByOpcode (a);
			if (o == null)
				continue;
			table.put (o, new Integer (a));
		}
	}

}
