/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.io.*;

public class OpcodeTableSwitch extends Opcode {

	int def;
	int low;
	int high;
	int count;
	int offsets[];

	public int fromClass (DataInputStream dis) throws Exception {
		int a = (4 - ((pos + 1) & 3)) & 3;
		dis.skip (a);
		def = dis.readInt ();
		low = dis.readInt ();
		high = dis.readInt ();
		count = high - low + 1;
		offsets = new int[count];
		for (int b = 0; b < count; b ++)
			offsets[b] = dis.readInt ();
		return a + 12 + 4 * count;
	}
	
	public void toClass (DataOutputStream dos) throws Exception {
		dos.writeByte (opcode);
		int a = (4 - ((pos + 1) & 3)) & 3;
		for (; a > 0; a --)
			dos.writeByte (0);
		dos.writeInt (def);
		dos.writeInt (low);
		dos.writeInt (high);
		for (int b = 0; b < count; b ++)
			dos.writeInt (offsets[b]);
	}
	
	public int fromJRAW (Lexan lex) throws Exception {
		int a = (4 - ((pos + 1) & 3)) & 3;
		lex.cmpLexel (OpcodeTable.getNameByOpcode (opcode));
		def = (int) lex.getLong ();
		lex.cmpLexel ();
		low = (int) lex.getLong ();
		lex.cmpLexel ();
		high = (int) lex.getLong ();
		lex.cmpLexel ();
		count = high - low + 1;
		lex.cmpLexel ("{");
		offsets = new int[count];
		for (int b = 0; b < count; b ++) {
			offsets[b] = (int) lex.getLong ();
			lex.cmpLexel ();
		}
		lex.cmpLexel ("}");
		return a + 12 + 4 * count;
	}

	public void toJRAW (ExtStringBuffer esb) {
		esb.println (OpcodeTable.getNameByOpcode (opcode) + " " + def + " " + low + " " + high + " { // " + opcode);
		esb.incIndent ();
		for (int a = 0; a < count; a ++)
			esb.println (offsets[a] + " // " + (a + low));
		esb.decIndent ();
		esb.println ("}");
	}

}
