/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.syntax;

import org.netbeans.editor.Syntax;
import org.netbeans.editor.TokenID;

public class JRAWSyntax extends Syntax {

    private static final int MAX_IDENT_POS = 15;
    private static final char[] identBuffer = new char[MAX_IDENT_POS];
    private static int identPos = 0;

    // Internal states
    private static final int START = 2;
    private static final int WHITESPACE = 3;
    private static final int SLASH = 4;
    private static final int LINE_COMMENT = 5;
    private static final int BLOCK_COMMENT = 6;
    private static final int BLOCK_COMMENT_ASTERISK = 7;
    private static final int IDENTIFIER = 8;
    private static final int NUMBER = 9;
    private static final int NUMBER_POINT = 10;
    private static final int NUMBER_EXPONENT = 11;
    private static final int NUMBER_AFTER_EXPONENT_MINUS = 12;
    private static final int NUMBER_AFTER_EXPONENT = 13;
    private static final int STRING = 14;
    private static final int BSLASH = 15;
    private static final int INVALID_BSLASH = 16;
    private static final int STRING_NUMBER1 = 17;
    private static final int STRING_NUMBER2 = 18;
    private static final int LABEL = 19;
    private static final int HASH = 20;
    
    public JRAWSyntax() {
        tokenContextPath = JRAWTokenContext.contextPath;
    }
    
    protected TokenID parseToken() {
        while (offset < stopOffset) {
            switch (state) {
                case INIT:
/*                    state = START;
                    break;*/
                case START:
                    switch (buffer[offset]) {
                        case '/':
                            offset ++;
                            state = SLASH;
                            break;
                        case '{':
                            offset ++;
                            return JRAWTokenContext.LBRACE;
                        case '}':
                            offset ++;
                            return JRAWTokenContext.RBRACE;
                        case ',':
                            offset ++;
                            return JRAWTokenContext.COMMA;
                        case '"':
                            offset ++;
                            state = STRING;
                            break;
                        case '#':
                            offset ++;
                            state = HASH;
                            break;
                        default:
                            if (Character.isLetter(buffer[offset])) {
                                identPos = 1;
                                identBuffer[0] = buffer[offset];
                                offset ++;
                                state = IDENTIFIER;
                                break;
                            }
                            if (Character.isWhitespace(buffer[offset])) {
                                offset ++;
                                state = WHITESPACE;
                                break;
                            }
                            if (Character.isDigit(buffer[offset])  ||  buffer[offset] == '-') {
                                offset ++;
                                state = NUMBER;
                                break;
                            }
                            offset ++;
                            return JRAWTokenContext.INVALID_CHAR;
                    }
                    break;
                case SLASH:
                    switch (buffer[offset]) {
                        case '/':
                            offset ++;
                            state = LINE_COMMENT;
                            break;
                        case '*':
                            offset ++;
                            state = BLOCK_COMMENT;
                            break;
                        default:
                            state = START;
                            return JRAWTokenContext.INVALID_CHAR;
                    }
                    break;
                case LINE_COMMENT:
                    if (buffer[offset] == '\n') {
                        state = START;
                        return JRAWTokenContext.LINE_COMMENT;
                    }
                    offset ++;
                    break;
                case BLOCK_COMMENT:
                    if (buffer[offset] == '*') {
                        state = BLOCK_COMMENT_ASTERISK;
                    }
                    offset ++;
                    break;
                case BLOCK_COMMENT_ASTERISK:
                    if (buffer[offset] == '/') {
                        offset ++;
                        state = START;
                        return JRAWTokenContext.BLOCK_COMMENT;
                    }
                    state = BLOCK_COMMENT;
                    break;
                case LABEL:
                case IDENTIFIER:
                    if (Character.isLetterOrDigit(buffer[offset])  ||  buffer[offset] == '_') {
                        if (identPos < MAX_IDENT_POS) {
                            identBuffer[identPos] = buffer[offset];
                            identPos ++;
                        }
                        state = IDENTIFIER;
                        offset ++;
                    } else if (buffer[offset] == ':') {
                        state = LABEL;
                        offset ++;
                    } else {
                        boolean b = state == LABEL;
                        state = START;
                        if (identPos < MAX_IDENT_POS) {
                            TokenID token = JRAWTokenContext.getKeyword (new String (identBuffer, 0, identPos));
                            if (token != null) {
                                return token;
                            }
                        }
                        return (b) ? JRAWTokenContext.LABEL : JRAWTokenContext.IDENTIFIER;
                    }
                    break;
                case HASH:
                    if (Character.isLetterOrDigit(buffer[offset])  ||  buffer[offset] == '_') {
                        offset ++;
                    } else {
                        state = START;
                        return JRAWTokenContext.LABEL;
                    }
                    break;
                case WHITESPACE:
                    if (!(Character.isWhitespace(buffer[offset]))) {
                        state = START;
                        return JRAWTokenContext.WHITESPACE;
                    }
                    offset ++;
                    break;
                case NUMBER:
                    switch (buffer[offset]) {
                        case '.':
                            state = NUMBER_POINT;
                            break;
                        case 'e':
                        case 'E':
                            state = NUMBER_EXPONENT;
                            break;
                        default:
                            if (!(Character.isDigit(buffer[offset]))) {
                                state = START;
                                return JRAWTokenContext.INT_LITERAL;
                            }
                    }
                    offset ++;
                    break;
                case NUMBER_POINT:
                    switch (buffer[offset]) {
                        case 'e':
                        case 'E':
                            state = NUMBER_EXPONENT;
                            break;
                        default:
                            if (!(Character.isDigit (buffer[offset]))) {
                                state = START;
                                return JRAWTokenContext.FLOAT_LITERAL;
                            }
                    }
                    offset ++;
                    break;
                case NUMBER_EXPONENT:
                    if (Character.isDigit(buffer[offset])) {
                        state = NUMBER_AFTER_EXPONENT;
                        offset ++;
                        break;
                    }
                    if (buffer[offset] == '-') {
                        state = NUMBER_AFTER_EXPONENT_MINUS;
                        offset ++;
                        break;
                    }
                    state = START;
                    return JRAWTokenContext.INVALID_CHAR;
                case NUMBER_AFTER_EXPONENT_MINUS:
                    if (Character.isDigit(buffer[offset])) {
                        state = NUMBER_AFTER_EXPONENT;
                        offset ++;
                        break;
                    }
                    state = START;
                    return JRAWTokenContext.INVALID_CHAR;
                case NUMBER_AFTER_EXPONENT:
                    if (!(Character.isDigit (buffer[offset]))) {
                        state = START;
                        return JRAWTokenContext.FLOAT_LITERAL;
                    }
                    offset ++;
                    break;
                case STRING:
                    switch (buffer[offset]) {
                        case '\\':
                            state = BSLASH;
                            break;
                        case '"':
                            offset ++;
                            state = START;
                            return JRAWTokenContext.STRING_LITERAL;
                    }
                    offset ++;
                    break;
                case BSLASH:
                    switch (buffer[offset]) {
                        case '\\':
                        case 'n':
                        case 't':
                        case '"':
                            state = STRING;
                            offset ++;
                            break;
                        case '0':
                            state = STRING_NUMBER1;
                            offset ++;
                            break;
                        default:
                            state = INVALID_BSLASH;
                            return JRAWTokenContext.STRING_LITERAL;
                    }
                    break;
                case STRING_NUMBER1:
                    if (buffer[offset] >= '0'  &&  buffer[offset] <= '7') {
                        state = STRING_NUMBER2;
                        offset ++;
                        break;
                    }
                    state = INVALID_BSLASH;
                    return JRAWTokenContext.STRING_LITERAL;
                case STRING_NUMBER2:
                    if (buffer[offset] >= '0'  &&  buffer[offset] <= '7') {
                        state = STRING;
                        offset ++;
                        break;
                    }
                    state = INVALID_BSLASH;
                    return JRAWTokenContext.STRING_LITERAL;
                case INVALID_BSLASH:
                    offset ++;
                    state = STRING;
                    return JRAWTokenContext.INVALID_CHAR;
            }
        }
        if (lastBuffer) {
            switch (state) {
                case INIT:
                case START:
                    return null;
                case WHITESPACE:
                    return JRAWTokenContext.WHITESPACE;
                case LINE_COMMENT:
                    return JRAWTokenContext.LINE_COMMENT;
                case BLOCK_COMMENT:
                case BLOCK_COMMENT_ASTERISK:
                    return JRAWTokenContext.BLOCK_COMMENT;
                case LABEL:
                    return JRAWTokenContext.LABEL;
                case IDENTIFIER:
                    if (identPos < MAX_IDENT_POS) {
                        TokenID token = JRAWTokenContext.getKeyword (new String (identBuffer, 0, identPos));
                        if (token != null) {
                            return token;
                        }
                    }
                    return JRAWTokenContext.IDENTIFIER;
                case HASH:
                    return JRAWTokenContext.LABEL;
                case NUMBER:
                    return JRAWTokenContext.INT_LITERAL;
                case NUMBER_POINT:
                case NUMBER_EXPONENT:
                case NUMBER_AFTER_EXPONENT_MINUS:
                case NUMBER_AFTER_EXPONENT:
                    return JRAWTokenContext.FLOAT_LITERAL;
                case STRING:
                case BSLASH:
                case STRING_NUMBER1:
                case STRING_NUMBER2:
                    return JRAWTokenContext.STRING_LITERAL;
                case INVALID_BSLASH:
                    return JRAWTokenContext.INVALID_CHAR;
            }
            return JRAWTokenContext.INVALID_CHAR;
        }
        return null;
    }

}
