package org.netbeans.modules.jasm.syntax;

import java.awt.Font;
import java.awt.Color;
import java.util.Map;
import java.util.HashMap;
import java.util.TreeMap;
import org.netbeans.editor.Acceptor;
import org.netbeans.editor.AcceptorFactory;
import org.netbeans.editor.Coloring;
import org.netbeans.editor.Settings;
import org.netbeans.editor.SettingsNames;
import org.netbeans.editor.SettingsDefaults;
import org.netbeans.editor.SettingsUtil;
import org.netbeans.editor.TokenCategory;
import org.netbeans.editor.TokenContextPath;
import org.netbeans.editor.ext.ExtSettingsNames;
import org.netbeans.editor.ext.ExtSettingsDefaults;


class JRAWTokenColoringInitializer extends SettingsUtil.TokenColoringInitializer {

        Font boldFont = SettingsDefaults.defaultFont.deriveFont(Font.BOLD);
        Font italicFont = SettingsDefaults.defaultFont.deriveFont(Font.ITALIC);
        Settings.Evaluator boldSubst = new SettingsUtil.FontStylePrintColoringEvaluator(Font.BOLD);
        Settings.Evaluator italicSubst = new SettingsUtil.FontStylePrintColoringEvaluator(Font.ITALIC);
        Settings.Evaluator lightGraySubst = new SettingsUtil.ForeColorPrintColoringEvaluator(Color.lightGray);

        Coloring commentColoring = new Coloring(italicFont, Coloring.FONT_MODE_APPLY_STYLE,
                            Color.gray, null);

        Coloring keywordsColoring = new Coloring(boldFont, Coloring.FONT_MODE_APPLY_STYLE, Color.blue, null);
        Coloring numbersColoring = new Coloring(null, Color.red, null);
        Coloring stringsColoring = new Coloring(null, Color.magenta, null);
        Coloring invalidCharsColoring = new Coloring(null, Color.white, Color.red);
        
        Coloring labelsColoring = new Coloring(null, Color.green, null);

        public JRAWTokenColoringInitializer() {
            super(JRAWTokenContext.context);
        }

        public Object getTokenColoring(TokenContextPath tokenContextPath,
        TokenCategory tokenIDOrCategory, boolean printingSet) {
            if (!printingSet) {
                switch (tokenIDOrCategory.getNumericID()) {
                    case JRAWTokenContext.WHITESPACE_ID:
                    case JRAWTokenContext.IDENTIFIER_ID:
                    case JRAWTokenContext.OPERATORS_ID:
                        return SettingsDefaults.emptyColoring;

                    case JRAWTokenContext.LABEL_ID:
                        return labelsColoring;

                    case JRAWTokenContext.INVALID_CHAR_ID:
                        return invalidCharsColoring;

                    case JRAWTokenContext.KEYWORDS_ID:
                        return keywordsColoring;

                    case JRAWTokenContext.LINE_COMMENT_ID:
                    case JRAWTokenContext.BLOCK_COMMENT_ID:
                        return commentColoring;

                    case JRAWTokenContext.STRING_LITERAL_ID:
                        return stringsColoring;

                    case JRAWTokenContext.INT_LITERAL_ID:
                    case JRAWTokenContext.FLOAT_LITERAL_ID:
                        return numbersColoring;

                }

            } else { // printing set
                switch (tokenIDOrCategory.getNumericID()) {
                    case JRAWTokenContext.LINE_COMMENT_ID:
                    case JRAWTokenContext.BLOCK_COMMENT_ID:
                         return lightGraySubst; // print fore color will be gray

                    default:
                         return SettingsUtil.defaultPrintColoringEvaluator;
                }

            }

            return null;

        }

    }
