
package org.netbeans.server.componentsmatch;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import org.netbeans.modules.exceptions.entity.InnocentClass;
import org.netbeans.server.uihandler.DatabaseTestCase;

/**
 *
 * @author pzajac
 */
public class MatcherTest extends DatabaseTestCase{
    
    /** Creates a new instance of MatcherTest */
    public MatcherTest(String name) {
        super(name);
    }
    
    public void testSimple() throws IOException {
        Matcher.read(new StringReader(""));//clean Matcher
        Package fsPack = new Package("org.openide.filesystems");
        Component fsComp = new Component("openide","filesystems",4);
        Component openideCodeComp = new Component("openide","code",2);
        fsPack.addComponent(fsComp);
        fsPack.addComponent(openideCodeComp);
        Matcher.addGlobalPackage(fsPack);
        
        assertEquals("Components size",2,fsPack.getComponents().size());
        assertSame(fsComp,fsPack.getFirstComponent());
        
        
        Matcher matcher = Matcher.getDefault();
        Component comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.openide.filesystems.LocalFileSystem","getName(String)","file",45)
        });
        assertSame(fsComp,comp);
        
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("java.lang.String","getName(String)","file",45),
            new StackTraceElement("org.openide.filesystems.LocalFileSystem","getName(String)","file",45)
        });
        assertSame(fsComp,comp);
        
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("java.lang.String","getName(String)","file",45),
        });
        assertNull(comp);
        
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("java.lang.String","getName(String)","${java.home}/lib/rt.jar",45),
        });
        assertNull(comp);
        
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("java.lang.String","getName(String)","${netbeans.home}/lib/org-openide-util.jar",45),
        });
        assertEquals(comp.getComponent(), "netbeans.home");
        assertEquals(comp.getSubComponent(), "org-openide-util.jar");

        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.nothing.nothing","getName(String)","${netbeans.home}/lib/org-openide-util.jar",45),
            new StackTraceElement("org.openide.filesystems.LocalFileSystem","getName(String)","file",45)
        });
        assertEquals(comp.getComponent(), "netbeans.home");
        assertEquals(comp.getSubComponent(), "org-openide-util.jar");

        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("java.lang.String","getName(String)","file",45),
            new StackTraceElement("org.nothing.nothing","getName(String)","${user.dir}/lib/some-user-module.jar",45),
            new StackTraceElement("org.openide.filesystems.LocalFileSystem","getName(String)","file",45)
        });
        assertEquals(comp.getComponent(), "user.dir");
        assertEquals(comp.getSubComponent(), "some-user-module.jar");

    }
    
    public void testRead() throws IOException {
        Matcher.read(new InputStreamReader (Matcher.class.getResourceAsStream("componentmapping.txt")));
        // org.netbeans.api.java.source;27;java;retouche;22;editor;code completion;2;java;j2seproject;1;refactoring;retouche;1;java;srcmodel;1;
        Package pack = Matcher.getDefault().packages.get("org.netbeans.api.java.source");
        assertNotNull("package exists",pack);
        Component first = pack.getFirstComponent();
        assertEquals("component","java",first.getComponent());
        assertEquals("subcomponent","retouche",first.getSubComponent());
        assertEquals("issues count",22,first.getIssues());
        assertEquals("components count",5,pack.getComponents().size());
        
        Matcher matcher = Matcher.getDefault();
        Component comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("java.lang.String","getName(String)","file",45),
            new StackTraceElement("org.openide.ErrorManager","getDefault()","file",45)
        });
        // more components
        assertEquals("openide",comp.getComponent());
        assertEquals("wizards",comp.getSubComponent());
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("java.lang.String","getName(String)","file",45),
            new StackTraceElement("java.util.Vector","size()","file",45),
            new StackTraceElement("org.netbeans.modules.j2ee.refactoring.unknown.MyClass","getDefault()","file",45)
        });
        assertEquals("j2ee",comp.getComponent());
        assertEquals("refactoring",comp.getSubComponent());
        
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.netbeans.modules.reallyunknownmodule.Aaa","getDefault()","file",45)
        });
        assertNull(comp);
    }
    
    public void testOrgNetBeansLib() throws IOException {
        Matcher.read(new InputStreamReader(Matcher.class.getResourceAsStream("componentmapping.txt")));
        Matcher matcher = Matcher.getDefault();
 
        Component comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.netbeans.lib.xxx","checkLexerInputFinished()","file",45),
        });
        assertNull(comp);
   }
    public void testVisualWebInsync() throws IOException {
        Matcher.read(new InputStreamReader(Matcher.class.getResourceAsStream("septemberComponents.txt")));
        Matcher matcher = Matcher.getDefault();
 
        Component comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.netbeans.modules.visualweb.insync.ModelSet","modelSet()","file",45),
        });
        assertEquals("visualweb",comp.getComponent());
   }
    public void testXml() throws IOException {
        Matcher.read(new InputStreamReader(Matcher.class.getResourceAsStream("septemberComponents.txt")));
        Matcher matcher = Matcher.getDefault();
 
        Component comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.netbeans.modules.xml.wsdl.ui.navigator.WSDLNavigatorContent.","modelSet()","file",45),
            new StackTraceElement("java.awt.event.InvocationEvent.","dispatch()","file",45),
        });
        // http://installer.netbeans.org/issues/show_bug.cgi?id=112532 
        assertFalse("core".equals(comp.getComponent()));
   }
 
    public void testInnocent() throws IOException{
        Matcher.read(new InputStreamReader (Matcher.class.getResourceAsStream("componentmapping.txt")));
        Matcher matcher = Matcher.getDefault();
        
        Component comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.openide.util.lookup.InstanceContent$SimpleItem","<init>","file",45)
        });
        assertNotNull(comp);//no Innocent
        
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.openide.util.lookup.SimpleLookup","<init>","file",45)
        });
        assertNotNull(comp);//no Innocent

        perUtils.persist(new InnocentClass("org.openide.util.lookup")); // look up is innocent

        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.openide.util.lookup.InstanceContent$SimpleItem","<init>","file",45)
        });
        assertNull(comp);
        
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.openide.util.lookup.SimpleLookup","<init>","file",45)
        });
        assertNull(comp);
        
        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.openide.util.lookup.Lookups","<init>","file",45)
        });
        assertNull(comp);

        comp = matcher.match(new StackTraceElement[] {
            new StackTraceElement("org.openide.util.Enumerations","<init>","file",45)
        });
        assertNotNull(comp);
    }
}
