/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.server.uihandler;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import org.netbeans.modules.exceptions.entity.Logfile;

/**
 *
 * @author Jindrich Sedek
 */
public class AddLogFileWithoutConfigurationTest extends DatabaseTestCase{
    private String userDir = "NB01214251321";
    private static Map<Integer, Logfile> filesSet = new HashMap<Integer, Logfile>(50);
    
    public AddLogFileWithoutConfigurationTest(String str)  {
        super(str);
    }
    
    public void testGetLogInfo()throws Exception{
        List<Thread> threadList = new ArrayList<Thread>(90);
        for (int uploadnumber=0;uploadnumber<30;uploadnumber++){
            threadList.add(new Thread(new GetLog(uploadnumber)));
            threadList.add(new Thread(new GetLog(uploadnumber)));
            threadList.add(new Thread(new GetLog(uploadnumber)));
        }
        for (Thread thread : threadList) {
            thread.start();
        }
        boolean finished = false;
        while (!finished){
            finished = true;
            for (Thread thread : threadList) {
                if (thread.isAlive())finished=false;
            }
        }
        List<Logfile> list = AddLogFileTest.logFiles(perUtils);
        assertEquals("LOGS SHOULD BE INSERTED",30, list.size());
    }
    
    private class GetLog implements Runnable{
        Integer upload;
        public GetLog(int upload){
            this.upload = upload;
        }
        public void run() {
            Logfile log = LogsManager.getDefault().getLogInfo(userDir, upload);
            assertNotNull(log);
            if (filesSet.get(upload) == null)filesSet.put(upload, log);
            else assertEquals(filesSet.get(upload), log);
        }
    }
    
    public void testAddLogWithoutConfiguration()throws Exception{
        Logger.getLogger("").addHandler(new THandler());
        File log = LogsManagerTest.extractResourceAs(data, "1.log", "NB2040811605.0");
        logsManager.addLog(log, "11.0.0.51");
    }
    
    private class THandler extends Handler{
        
        public void publish(LogRecord record) {
            String message = record.getMessage();
            if ("METADATA = NULL".equals(message)){
                List<Logfile> logFiles = perUtils.getAll(Logfile.class);
                assertEquals("LOG file is INSERTED", 1, logFiles.size());
                Logfile logFile  = logFiles.get(0);
                assertNull("Version", logFile.getProductVersionId());
                assertEquals("IP Addr", "11.0.0.51", logFile.getIpAddr());
            }
        }
        
        public void flush() {
            //
        }
        
        public void close() throws SecurityException {
            //
        }
        
    }
    
}
