/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is Forte for Java, Community Edition. The Initial
 * Developer of the Original Software is Sun Microsystems, Inc. Portions
 * Copyright 1997-2006 Sun Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.mdrxml.looks;

import java.lang.ref.WeakReference;
import java.io.IOException;
import java.awt.datatransfer.Transferable;
import org.openide.actions.PropertiesAction;
import org.openide.nodes.*;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.netbeans.api.looks.*;
import org.netbeans.spi.looks.*;
import org.netbeans.modules.mdrxml.looks.actions.DeleteAction;
import org.netbeans.modules.mdrxml.looks.actions.GenerateTextAction;
/**
 *
 * @author  Tomas Zezula
 */
public class NodeLook extends DefaultLook {
    
    private static class NameProperty extends PropertySupport.ReadWrite {
        
        private WeakReference ref;
        
        public NameProperty(Look.NodeSubstitute substitute) {
            super(NbBundle.getMessage(NodeLook.class,"TXT_Name"),String.class,NbBundle.getMessage(NodeLook.class,"TXT_Name"),NbBundle.getMessage(NodeLook.class,"TIP_Name"));
            this.ref = new WeakReference (substitute);
        }
        
        public Object getValue() {
	    Look.NodeSubstitute val = (Look.NodeSubstitute) this.ref.get ();
            if (val != null && (val.getRepresentedObject() instanceof xmlmodel.Node)) {
                return ((xmlmodel.Node)val.getRepresentedObject()).getName();
            }
            else {
                return null;
            }
        }
        
        public void setValue(Object value) {
	    Look.NodeSubstitute val = (Look.NodeSubstitute) this.ref.get ();
            if (value instanceof String && val != null && val.getRepresentedObject() instanceof xmlmodel.Node) {
                ((xmlmodel.Node)val.getRepresentedObject()).setName((String)value);
                val.fireNameChange(null,(String)value);
                val.fireDisplayNameChange(null,(String)value);
            }
        }
        
    }
    
    /** Creates a new instance of NodeLook */
    public NodeLook(String name) {
        super(name);
    }
    
    public String getName(Look.NodeSubstitute substitute) {
        try {
            xmlmodel.Node en = (xmlmodel.Node) substitute.getRepresentedObject();
            return en.getName();
        }catch (javax.jmi.reflect.InvalidObjectException ioe) {
            return new String ();
        }
    }
    
    public String getDisplayName(Look.NodeSubstitute substitute) {
        return this.getName(substitute);
    }
    
    public Node.PropertySet[] getPropertySets(Look.NodeSubstitute substitute) {
        Sheet.Set set = new Sheet.Set();
        String name = NbBundle.getMessage(NodeLook.class,"TXT_Properties");
        set.setName(name);
        set.setDisplayName(name);
        set.put(new NameProperty(substitute));
        return new Node.PropertySet[] {set};
    }
    
    public javax.swing.Action[] getActions(Look.NodeSubstitute substitute) {
        return new javax.swing.Action[] {
            SystemAction.get(DeleteAction.class),
            SystemAction.get(PropertiesAction.class),
            SystemAction.get(GenerateTextAction.class)
        };
    }
    
    public boolean canCopy (Look.NodeSubstitute substitute) {
      return true;  
    }
    
    public Transferable drag (Look.NodeSubstitute substitute) throws IOException {
        return this.clipboardCopy (substitute);
    }
    
}
