/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.mdrxml.looks.actions;

import org.openide.util.NbBundle;
import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.nodes.Node;
import org.openide.util.*;
import org.openide.util.actions.NodeAction;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileObject;
import org.openide.loaders.*;

import org.netbeans.api.looks.*;
import org.netbeans.api.mdr.*;
import org.netbeans.modules.mdrxml.util.XMLGenerator;

import javax.jmi.reflect.*;
import xmlmodel.*;
import java.util.*;
import java.io.*;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

public class GenerateXMLAction extends NodeAction {
    
    private static ResourceBundle bundle;

    protected void performAction (Node[] nodes) {
        Node node = nodes[0];
        LookNode lookNode = (LookNode) node;
        if ( lookNode != null ) {
            generateXML( lookNode );
        }
    }

    private void generateXML( LookNode lookNode ) {
        File temp = null;
        final TopManager tm = TopManager.getDefault();
        // selects one folder from data systems
        JFileChooser chooser = new JFileChooser();
        chooser.setFileFilter(new XMLFileFilter());
	java.awt.Component parent = tm.getWindowManager().getMainWindow();
        int returnVal = org.openide.util.Utilities.showJFileChooser(chooser, parent, getLocalizedString("TXT_GenerateXML"));
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            temp = chooser.getSelectedFile();
        }
        
        final File file = temp;
        final ElementNode elementNode = (ElementNode) lookNode.getRepresentedObject();
        
        if (file != null) {
            RequestProcessor.postRequest(new Runnable() {
                public void run() {                    
                    tm.setStatusText (getLocalizedString("TXT_StartGenerate"));
                    try {
                        FileOutputStream fos = new FileOutputStream( file );
                        new XMLGenerator ().generateXML (fos, elementNode);
                        tm.setStatusText (getLocalizedString("TXT_FinishGeneration"));
                    } catch (Exception e) {
                        TopManager.getDefault().getErrorManager().notify (e);
                    }
                } // run
            });
        } // if
    }
    
    private static String getLocalizedString (String message) {
        if (bundle == null)
            bundle = NbBundle.getBundle (GenerateXMLAction.class);
        return bundle.getString (message);
    }

    protected boolean enable (Node[] nodes) {
        if (nodes.length != 1)
            return false;
        if (!(nodes[0] instanceof LookNode))
            return false;
        LookNode lookNode = (LookNode) nodes[0];
        return lookNode.getRepresentedObject() instanceof ElementNode;
    }

    public String getName () {
        return getLocalizedString ("TXT_GenerateAction");
    }

    protected String iconResource () {
        return null;
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
    }

    private static final class XMLFileFilter extends FileFilter {
        public boolean accept(File f) {
            return f.getName().toUpperCase().endsWith(".XML") || f.isDirectory(); // No I18N
        }
        
        public String getDescription() {
            return getLocalizedString ("TXT_XMLFiles");
        }
    }
        
}