/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.handlers;

import org.netbeans.mdr.util.ImplGenerator;

import java.io.*;
import java.lang.reflect.Method;
import java.util.*;

/**
 *
 * @author Martin Matula
 * @version
 */
public class StructGenerator extends ImplGenerator {
    private static final String GET_PREFIX = "get"; //NOI18N
    private static final String IS_PREFIX = "is"; //NOI18N

    private static final String M_GET_NAME = "handleGet"; //NOI18N
    private static final String M_GET_DESC = "(Ljava/lang/String;)Ljava/lang/Object;"; //NOI18N

    private static final String CONSTRUCTOR_DESCRIPTOR = "(Ljava/util/List;Ljava/util/Map;Ljava/util/List;)V"; //NOI18N

    private StructGenerator(String className, Class ifc) {
        super(className, ifc, StructImpl.class);
    }

    public static byte[] generateStruct(final String name, Class ifc) {
	StructGenerator gen = new StructGenerator(name, ifc);
	final byte[] classFile = gen.generateClassFile();

	return classFile;
    }

    protected Method[] getMethodsToImplement() {
        Method ifcMethods[] = ifc.getMethods();
        Method objMethods[] = javax.jmi.reflect.RefStruct.class.getMethods();
        HashSet excludeMethods = new HashSet();
        ArrayList methodsToImplement = new ArrayList();
        
        for (int i = 0; i < objMethods.length; i++) {
            excludeMethods.add(objMethods[i].getName() + getMethodDescriptor(objMethods[i].getParameterTypes(), objMethods[i].getReturnType()));
        }
        
        for (int i = 0; i < ifcMethods.length; i++) {
            if (!excludeMethods.contains(ifcMethods[i].getName() + getMethodDescriptor(ifcMethods[i].getParameterTypes(), ifcMethods[i].getReturnType()))) {
                methodsToImplement.add(ifcMethods[i]);
            }
        }
        
        return (Method[]) methodsToImplement.toArray(new Method[methodsToImplement.size()]);
    }
    
    /**
     * Generate the constructor method for the proxy class.
     */
    protected MethodInfo generateConstructor() throws IOException {
	MethodInfo minfo = new MethodInfo(
	    "<init>", CONSTRUCTOR_DESCRIPTOR, //NOI18N
	    ACC_PUBLIC);

	DataOutputStream out = new DataOutputStream(minfo.code);

        // this
	code_aload(0, out);
        // list containing field names
        code_aload(1, out);
        // map containing fields and their values
        code_aload(2, out);
        // type qualified name
        code_aload(3, out);

        // just call super.<init>(String);
	out.writeByte(opc_invokespecial);
	out.writeShort(cp.getMethodRef(
	    dotToSlash(superclass.getName()),
	    "<init>", CONSTRUCTOR_DESCRIPTOR)); //NOI18N

	out.writeByte(opc_return);

	minfo.maxStack = 10;
	minfo.maxLocals = 4;
	minfo.declaredExceptions = new short[0];

	return minfo;
    }

    protected ClassMethod getClassMethod(Method method, Class fromClass) {
        String methodName = method.getName();
        ClassMethod am;

        if (methodName.startsWith(GET_PREFIX)) {
            am = new ClassMethod(method, cp.getMethodRef(dotToSlash(superclass.getName()), M_GET_NAME, M_GET_DESC), firstLower(strip(methodName, GET_PREFIX)));
        } else if (methodName.startsWith(IS_PREFIX)) {
            am = new ClassMethod(method, cp.getMethodRef(dotToSlash(superclass.getName()), M_GET_NAME, M_GET_DESC), firstLower(methodName));
        } else {
            am = super.getClassMethod(method, fromClass);
        }

        return am;
    }
}
