/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.btreeimpl.btreestorage;

import java.lang.ref.*;

import org.netbeans.mdr.persistence.*;

/** A member of the MDR cache.  This is a soft reference to an object
* identified by a MOFID.
*/
//class CacheReference extends SoftReference {
// BHM - lets try more memory friendly
class CacheReference extends WeakReference {

    /* The MOF ID of the object we reference */
    private Object key;

    private Object hardRef;

    /* Whether the reference is currently in the cache */
    // boolean inCache;

    /** Create a reference 
    * @param m the MOF ID of the referenced object
    * @param o the referenced object
    * @param q our reference queue
    */
    CacheReference(Object m, Object o, ReferenceQueue q) {
        super(o, q);
        key = m;
    }

    /** A CacheReference is represented by the same string as its MOF ID */
    public String toString() {
        return key.toString();
    }

    /** A CacheReference hashes to the same number as its MOF ID */
    public int hashCode() {
        return key.hashCode();
    }

    /** Two CacheReferences are equal if their MOF IDs are equal */
    public boolean equals(Object o) {
        if (!(o instanceof CacheReference))
            return false;

        CacheReference cr = (CacheReference)o;
        return cr.key.equals(key);
    }
    
    /** Calling this method makes the reference to be a hard reference
     */
    public void harden() {
        hardRef = get();
    }
    
    /** Calling this method makes the reference weak again
     */
    public void weaken() {
        hardRef = null;
    }
    
    /** Returns true if the reference is hard
     */
    public boolean isHard() {
        return hardRef != null;
    }
    
    
    /** Overriden clear to weaken possibly hardened reference
     */
    public void clear() {
        weaken();
        key = null;
        super.clear();
    }
    
    
    /** Returns the key of the referece */
    Object getKey() {
        return key;
    }
    
}
