/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.btreeimpl.btreestorage;

import java.io.*;
import java.text.*;
import java.util.*;
import org.netbeans.mdr.persistence.*;
import org.netbeans.mdr.persistence.btreeimpl.btreeindex.*;

/**
* This is an index from names (Strings) to Integer.  Its primary use is to
* store the codes for classes stored in the repository.
*/
public class CounterIndex extends NameIndex {

    /* next integer to be assigned */
    int nextInt = 0;

    /** Create a new CounterIndex  */
    public CounterIndex() {
    }

    /** get an integer by its name.  If none exists, throw an exception
    * @param name the name associated with the integer
    */
    public synchronized int get(String name) throws StorageException  {
    	return ((Integer)getObj(name)).intValue();
    }

    /** get an integer by its name.  If none exists, return null
    * @param name the name associated with the integer
    */
    public synchronized Integer getIf(String name) throws StorageException  {
    	return (Integer)getObjIf(name);
    }

    /**
    * Add a name-MOFID pair to the MofidIndex.  If one already existed,
    * throw an exception.
    * @param name name of object to add to index
    * @param id MOFID of object to add
    */
    public synchronized int add(String name) throws StorageException {
	int i = nextInt++;
    	addObj(name, new Integer(i));
	return i;
    }

    /* write to a DataOutputStream
    */
    public synchronized void write(DataOutputStream dstrm) throws StorageException{
	super.write(dstrm);
	try {
	    dstrm.writeInt(nextInt);
	}
	catch (IOException ex) {
	    throw new StorageIOException(ex);
	}
    }

    /** read from a DataOutputStream
    */
    public synchronized void read(DataInputStream dstrm) throws StorageException{
	super.read(dstrm);
	try {
	    nextInt = dstrm.readInt();
	}
	catch (IOException ex) {
	    throw new StorageIOException(ex);
	}
    }

    /**
    * write object to stream.  Used by serialization.
    * @param obj object to write
    * @param strm stream to write it to
    */
    protected void writeObjectToStream(Object obj, DataOutputStream strm) 
    	throws StorageException {
	try {
	    strm.writeInt(((Integer)obj).intValue());
	}
	catch (IOException ex) {
	    throw new StorageIOException(ex);
	}
    }

    /**
    * read object from stream.  Used by deserialization.
    * @param strm stream to read from
    * @return obj object read
    */
    protected Object readObjectFromStream(DataInputStream strm) 
    	throws StorageException {
	try {
	    return new Integer(strm.readInt());
	}
	catch (IOException ex) {
	    throw new StorageIOException(ex);
	}
	
    }
}



