/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.memoryimpl;

import java.util.HashMap;
import java.util.Map;
import org.netbeans.mdr.persistence.*;

/**
 *
 * @author  pbuzek
 * @version
 */
public class StorageFactoryImpl extends Object implements StorageFactory {
    public static final String STORAGE_ID = "org.netbeans.mdr.persistence.memoryimpl.id";
    public static final String STORAGE_NAME = "org.netbeans.mdr.persistence.memoryimpl.fileName";
    static final String NULL_STORAGE_ID = ".";
    private static final MOFID NULL_MOFID = new MOFID(0, NULL_STORAGE_ID);

    private StorageImpl nullStorage; 
    
    private static final HashMap storages = new HashMap();

    /** Creates new StorageFactoryImpl */
    public StorageFactoryImpl() {
    }

    /** Creates instance of class that implements Storage interface.
     * throws StorageException if the name is not valid name of a Storage
     */
    public synchronized Storage createStorage(Map properties) throws StorageException {
        String name = (String) properties.get(STORAGE_ID);  // Not mandatory
        if (name == null || name.equals(NULL_STORAGE_ID)) {
            if (nullStorage == null) {
                nullStorage = new StorageImpl(NULL_STORAGE_ID, null);
            }
            return nullStorage;
        } else {
            synchronized (storages) {
                if (storages.containsKey(name)) {
                    throw new RuntimeException("Storage '" + name + "' already created.");
                } else {
                    Storage result = new StorageImpl(name, (String) properties.get(STORAGE_NAME));
                    storages.put(name, result);
                    return result;
                }                
            }
        }
    }
    
    /** Serializes content of the given storage. Returns false if the storage was not found. 
     */
    public static boolean serialize(String storageId) throws StorageException {
        StorageImpl storage;
        synchronized (storages) {
            storage = (StorageImpl) storages.get(storageId);
        }
        if (storage == null) {
            return false;
        }
        storage.serialize();
        return true;
    }
    
    public org.netbeans.mdr.persistence.MOFID createNullMOFID() throws StorageException {
        return NULL_MOFID;
    }
}
