/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.storagemodel;

import java.util.*;
import javax.jmi.reflect.*;

import org.netbeans.mdr.persistence.StorageException;
import org.netbeans.mdr.persistence.MOFID;
import org.netbeans.mdr.util.DebugException;
import org.netbeans.mdr.util.Logger;

/**
 *
 * @author Martin Matula
 */
public class AttrList extends AttrCollection implements List {
    protected final List innerList;

    public AttrList() {
        innerList = (List) inner;
    }

    AttrList(StorableFeatured mdrObject, StorableClass.AttributeDescriptor desc) throws StorageException {
        this(mdrObject, desc, null);
    }
    
    protected AttrList(StorableFeatured mdrObject, StorableClass.AttributeDescriptor desc, List values) throws StorageException {
        super(mdrObject, desc, values);
        innerList = (List) inner;
    }
    
    protected AttrList(StorableFeatured mdrObject, List values, int maxSize, Class type, String attrName, boolean isRefObject, MOFID metaMofId) {
        super(mdrObject, values, maxSize, type, attrName, isRefObject, metaMofId);
        innerList = values;
    }
    
    public void add(int param, Object obj) {
        checkType(obj);
        checkMaxSize(1);
        checkUnwrap();
        mdrObject.objectWillChange();
        
        if (isIndexed)
            ((StorableObject) mdrObject).removeFromIndex (metaMofId);
        innerList.add(param, obj);
        if (isRefObject) {
            try {
                setAttribComposite((RefObject) obj);
            } catch (StorageException e) {
                throw (DebugException) Logger.getDefault().annotate(new DebugException(), e);
            }
        }
        
        if (isIndexed)
            ((StorableObject) mdrObject).addToIndex (metaMofId);
        
        mdrObject.objectChanged();
    }    
    
    public boolean addAll(int param, Collection collection) {
        // should not be called
        throw new DebugException();
    }
    
    public Object get(int param) {
        checkUnwrap();
        return innerList.get(param);
    }
    
    public int indexOf(Object obj) {
        checkUnwrap();
        return innerList.indexOf(obj);
    }
    
    public int lastIndexOf(Object obj) {
        checkUnwrap();
        return innerList.lastIndexOf(obj);
    }
    
    public ListIterator listIterator() {
        checkUnwrap();
        return new AttrListIterator(innerList.listIterator());
    }
    
    public ListIterator listIterator(int param) {
        checkUnwrap();
        return new AttrListIterator(innerList.listIterator(param));
    }
    
    public Object remove(int param) {
        checkUnwrap();
        mdrObject.objectWillChange();
        if (isIndexed)
            ((StorableObject) mdrObject).removeFromIndex (metaMofId);

        Object result = innerList.remove(param);
        if (isRefObject) {
            try {
                clearAttribComposite((RefObject) result);
            } catch (StorageException e) {
                throw (DebugException) Logger.getDefault().annotate(new DebugException(), e);
            }
        }
        
        if (isIndexed)
            ((StorableObject) mdrObject).addToIndex (metaMofId);
        
        mdrObject.objectChanged();
        return result;
    }
    
    public Object set(int param, Object obj) {
        checkType(obj);
        checkUnwrap();
        mdrObject.objectWillChange();
        if (isIndexed)
            ((StorableObject) mdrObject).removeFromIndex (metaMofId);
        
        Object result = innerList.set(param, obj);
        if (isRefObject) {
            try {
                clearAttribComposite((RefObject) result);
                setAttribComposite((RefObject) obj);
            } catch (StorageException e) {
                throw (DebugException) Logger.getDefault().annotate(new DebugException(), e);
            }
        }
        
        if (isIndexed)
            ((StorableObject) mdrObject).addToIndex (metaMofId);
        
        mdrObject.objectChanged();
        return result;
    }
    
    public List subList(int param, int param1) {
        checkUnwrap();
        return new AttrList(mdrObject, innerList.subList(param, param1), maxSize, type, attrName, isRefObject, metaMofId);
    }
    
    protected class AttrListIterator extends AttrIterator implements ListIterator {
        private final ListIterator innerIterator;
        
        protected AttrListIterator(ListIterator iterator) {
            super(iterator);
            innerIterator = iterator;
        }
        
        public void add(Object obj) {
            checkType(obj);
            checkMaxSize(1);
            
            mdrObject.objectWillChange();
            
            if (isIndexed)
                ((StorableObject) mdrObject).removeFromIndex (metaMofId);
            
            innerIterator.add(obj);
            if (isRefObject) {
                try {
                    setAttribComposite((RefObject) obj);
                } catch (StorageException e) {
                    throw (DebugException) Logger.getDefault().annotate(new DebugException(), e);
                }
            }
            
            if (isIndexed)
                ((StorableObject) mdrObject).addToIndex (metaMofId);
            
            mdrObject.objectChanged();
        }
        
        public boolean hasPrevious() {
            return innerIterator.hasPrevious();
        }
        
        public int nextIndex() {
            return innerIterator.nextIndex();
        }
        
        public Object previous() {
            return (lastRead = innerIterator.previous());
        }
        
        public int previousIndex() {
            return innerIterator.previousIndex();
        }
        
        public void set(Object obj) {
            checkType(obj);
            
            mdrObject.objectWillChange();
            
            if (isIndexed)
                ((StorableObject) mdrObject).removeFromIndex (metaMofId);
            
            innerIterator.set(obj);
            if (isRefObject) {
                try {
                    clearAttribComposite((RefObject) lastRead);
                    setAttribComposite((RefObject) obj);
                } catch (StorageException e) {
                    throw (DebugException) Logger.getDefault().annotate(new DebugException(), e);
                }
            }
            
            if (isIndexed)
                ((StorableObject) mdrObject).addToIndex (metaMofId);
            
            mdrObject.objectChanged();
        }
    }
}
