/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.mdr.storagemodel.transientimpl;

import java.lang.ref.ReferenceQueue;
import java.util.HashMap;
import org.netbeans.mdr.persistence.*;
import org.netbeans.mdr.storagemodel.TransientStorableObject;
import org.netbeans.mdr.storagemodel.TransientStorableClass;
/**
 *
 * @author Tomas Zezula
 */

public class TransientObjectResolverIndex extends TransactionalIndex implements SinglevaluedIndex {

    public static final String NAME = "TRANSIENT_OBJECTS_RESOLVER";

    private HashMap map;
    private ReferenceQueue queue;
    
    /** Creates a new instance of ObjectResolverIndex */
    public TransientObjectResolverIndex() {
        this.map = new HashMap ();
        this.queue = new ReferenceQueue ();
    }
    
    public void add(Object key, Object value) throws StorageException {
        this.addNoTx (key, value);
        this.txlog.push (new CompensatingTransaction.AddCTx (key, value));
    }
    
    public void addNoTx (Object key, Object value) throws StorageException {
        this.map.put (key, new TransientIndex.KeyedReference (value, this.queue, (MOFID)key));
    }
    
    public boolean put (Object key, Object value) throws StorageException {
        throw new UnsupportedOperationException ();
    }
    
    public void replace (Object key, Object value) throws StorageException {
        throw new UnsupportedOperationException ();
    }
    
    public Object get (Object key) throws StorageException {
        Object result = this.getIfExists (key);
        if (result == null)
            throw new StorageBadRequestException ();
        return result;
    }
    
    public Object getIfExists (Object key) throws StorageException {
        TransientIndex.KeyedReference entry = (TransientIndex.KeyedReference) this.map.get (key);
        if (entry == null)
            return null;
        return entry.get ();
    }
    
    public Object getObjectIfExists (Object key, SinglevaluedIndex index) throws StorageException {
        TransientIndex.KeyedReference entry = (TransientIndex.KeyedReference) this.map.get (key);
        if (entry == null)
            return null;
        Object result = entry.get ();
        if (result == null)
            return null;
        return index.get (result);
    }
    
    public Object getObject (Object key, SinglevaluedIndex index) throws StorageException {
        Object result = this.getObjectIfExists (key, index);
        if (result == null)
            throw new StorageBadRequestException ();
        return result;
    }
    

    public Storage.EntryType getKeyType() throws StorageException {
        return Storage.EntryType.MOFID;
    }
    
    public String getName () throws StorageException {
        return NAME;
    }
    
    public Storage.EntryType getValueType() throws StorageException {
        return Storage.EntryType.STREAMABLE;
    }
    
    public java.util.Set keySet() throws StorageException {
        throw new UnsupportedOperationException ();
    }
    
    public java.util.Collection values () throws StorageException {
        throw new UnsupportedOperationException ();
    }
    
    public boolean remove(Object key) throws StorageException {
        Object result = this.removeNoTx (key, null);
        if (result != null) {
            this.txlog.push ( new CompensatingTransaction.RemoveCTx (key, result));
            return true;
        }
        else {
            return false;
        }
    }
    
    public java.util.Collection queryByKeyPrefix (Object prefix, SinglevaluedIndex repos) {
        throw new UnsupportedOperationException ();
    }
    
    protected Object removeNoTx(Object key, Object value) throws StorageException {
        return this.map.remove (key);
    }
    
}
