/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.test;

import org.openide.util.*;
import org.netbeans.mdr.NBMDRManagerImpl;
import org.netbeans.lib.jmi.xmi.*;
import org.netbeans.lib.jmi.mapping.JMIMapperImpl;

import java.util.*;

/**
 *
 * @author  mmatula
 * @version
 */
public class MDRTestLookup extends Lookup {
    private ArrayList instances = new ArrayList();

    /** Creates new RepositoryLookup */
    public MDRTestLookup() {
        instances.add(new NBMDRManagerImpl());
        instances.add(new XMISaxReaderImpl());
        instances.add(new JMIMapperImpl());
        instances.add(new XMIWriterImpl());
    }

    public Object lookup(Class cls) {
        for (Iterator it = instances.iterator(); it.hasNext();) {
            Object instance = it.next();
            if (cls.isAssignableFrom(instance.getClass())) {
                return instance;
            }
        }
        return null;
    }
    
    public Lookup.Result lookup(Lookup.Template template) {
        return new Result(lookup(template.getType()));
    }
    
    private static class Result extends Lookup.Result {
        private final ArrayList result = new ArrayList();
        
        public Result(Object o) {
            if (o != null) result.add(o);
        }
        
        /** Registers a listener that is invoked when there is a possible
         * change in this result.
         *
         * @param l the listener to add
         */
        public void addLookupListener(LookupListener l) {
            // this lookup never changes so nothing need to be registered
        }
        
        /** Get all instances in the result.
         * @return collection of all instances
         */
        public java.util.Collection allInstances() {
            return result;
        }
        
        /** Unregisters a listener previously added.
         * @param l the listener to remove
         */
        public void removeLookupListener(LookupListener l) {
            // nothing was registered
        }
    }
}
