/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 */
package org.foo.wavsupport;

import java.io.File;
import java.io.IOException;
import javax.sound.sampled.AudioFormat;
import org.foo.wavsupport.api.*;
import org.foo.wavutils.WrappedAudioInputStream;
import org.openide.DialogDisplayer;
import org.openide.NotifyDescriptor;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.nodes.Node;

public class WavDataObject extends MultiDataObject implements WavCookie {
    
    private final SaveCookie saveCookie;
    
    /** The actual audio data */
    private transient WrappedAudioInputStream audioStream;

    public WavDataObject(FileObject pf, WavDataLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        saveCookie = new WavSaveCookie(this);
        getCookieSet().add(new WavOpenSupport(getPrimaryEntry()));
    }
    
    protected Node createNodeDelegate() {
        return new WavDataNode(this);
    }
    
    public void setModified(boolean modified)
    {
        super.setModified(modified);
        if (modified)
        {
            addSaveCookie();
        }
        else
        {
            removeSaveCookie();
        }
    }
    
    public AudioFormat getAudioFormat()
    {
        return getAudioInputStream().getFormat();
    }

    public WrappedAudioInputStream getAudioInputStream()
    {
        if( audioStream == null )
        {
            FileObject promptFO = getPrimaryFile();
            File promptFile = FileUtil.toFile(promptFO);
            try
            {
                audioStream = WrappedAudioInputStream.createFromFile(promptFile);
            }
            catch (Exception ex)
            {
                NotifyDescriptor nd = new NotifyDescriptor.Message(
                        "Can't get audio stream for " + getName() 
                        + " because " + ex.toString());
                DialogDisplayer.getDefault().notify(nd);
            }
        }
        return audioStream;
    }

    /**
     * @param is The AudioInputStream for this prompt, or null to revert to disk.
     */
    public void setAudioInputStream( WrappedAudioInputStream is )
    {
        final WrappedAudioInputStream oldStream = audioStream;
        audioStream = is;
        // If the audio input stream is not null, the wav file is modified.
        setModified(is != null);
        firePropertyChange(PROP_WAVEFORM, oldStream, is);
    }

    private void addSaveCookie()
    {
        if( getCookie( SaveCookie.class ) == null ) {
            getCookieSet().add( saveCookie );
        }
    }

    private void removeSaveCookie()
    {
        SaveCookie save = (SaveCookie) getCookie( SaveCookie.class );
        if( save != null ) {
            getCookieSet().remove( save );
        }
    }

}
