/*
 * CustomerRootChildren.java
 *
 * Created on September 21, 2006, 3:40 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.netbeans.examples.masterdetail.nodes;

import java.beans.IntrospectionException;
import java.util.Collections;
import org.netbeans.examples.masterdetail.api.Customer;
import org.openide.ErrorManager;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;

/**
 *
 * @author Tim
 */
final class CustomerRootChildren extends Children.Keys {
    final Customer[] customers;
    
    /** Creates a new instance of CustomerRootChildren */
    public CustomerRootChildren() {
        this (generateCustomers());
    }
    
    CustomerRootChildren (Customer[] customers) {
        this.customers = customers;
    }
    
    protected void addNotify() {
        setKeys (customers);
    }
    
    protected void removeNotify() {
        //allow the nodes to be disposed
        setKeys (Collections.EMPTY_LIST);
    }

    protected Node[] createNodes(Object object) {
        Customer c = (Customer) object;
        Node result;
        try {
            result = new CustomerNode (c);
        } catch (IntrospectionException ex) {
            ErrorManager.getDefault().notify (ex);
            result = new AbstractNode (Children.LEAF);
            result.setDisplayName (NbBundle.getMessage (CustomerRootChildren.class,
                    "ERR_Exception", ex.getMessage()));
        }
        return new Node[] { result };
    }

    /** Generate some sample Customer objects */
    private static Customer[] generateCustomers() {
        Customer[] result = new Customer[] {
            new Customer ("Joe", "NetBeans", "21 NetBeans Way", "Foodbar", "MN",
                    "23406"),
            new Customer ("Martha", "NetBeans", "22 Java Street", "Foodbar", 
                    "MN", "05383"),
            new Customer ("Egloreptidus", "Tiddlewaddle", "31 Pathology St.", 
                    "Snorkelbreath", "MI", "29423"),
            new Customer ("Hector", "Hedgehog", "201 Brush St.", "Noodlebarn", 
                    "TN", "32204"),
            new Customer ("Billy", "Poodlehoover", "12 Ambush Rd.", "Snikt", 
                    "CA", "94306"),
            new Customer ("David", "Mumblebunny", "2931 Food St.", "Hunger", 
                    "MN", "23041"),
        };
        return result;
    }
}
