/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.customexplorer;

import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * Custom implementation of a rectangle with support for property changes
 * and a name.  A CustomRectangle is contained by a ShapeBusinessObject.
 *
 * @author twheeler
 */
public class CustomRectangle {

    // custom data flavor used for DnD support
    public static final DataFlavor DATA_FLAVOR
            = new DataFlavor(CustomRectangle.class, "CustomRectangle");

    private static final String PROP_X = "x";
    private static final String PROP_Y = "y";
    private static final String PROP_WIDTH = "width";
    private static final String PROP_HEIGHT = "height";
    private static final String PROP_NAME = "name";

    private PropertyChangeSupport pcs;

    private double x, y, width, height;
    private String name;

    public CustomRectangle(String name, double x, double y, double width, double height) {
        this.name = name;
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;

        pcs = new PropertyChangeSupport(this);
    }

    public Rectangle getRectangle() {
        Rectangle rect = new Rectangle();
        rect.setRect(x, y, width, height);
        return rect;
    }

    public double getHeight() {
        return height;
    }

    public void setHeight(double height) {
        double old = this.height;
        this.height = height;
        pcs.firePropertyChange(PROP_HEIGHT, old, height);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        String old = this.name;
        this.name = name;
        pcs.firePropertyChange(PROP_NAME, old, name);
    }

    public double getWidth() {
        return width;
    }

    public void setWidth(double width) {
        double old = this.width;
        this.width = width;
        pcs.firePropertyChange(PROP_WIDTH, old, width);
    }

    public double getX() {
        return x;
    }

    public void setX(double x) {
        double old = this.x;
        this.x = x;
        pcs.firePropertyChange(PROP_X, old, x);
    }

    public double getY() {
        return y;
    }

    public void setY(double y) {
        double old = this.y;
        this.y = y;
        pcs.firePropertyChange(PROP_Y, old, y);
    }

    public void addPropertyChangeListener(PropertyChangeListener pcl) {
        pcs.addPropertyChangeListener(pcl);
    }

    public void removePropertyChangeListener(PropertyChangeListener pcl) {
        pcs.removePropertyChangeListener(pcl);
    }
}
