/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.explservlet;

import java.awt.Graphics;
import java.beans.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.openide.nodes.*;
import java.awt.Image;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import org.openide.ErrorManager;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.Repository;
import org.openide.loaders.DataFilter;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.loaders.RepositoryNodeFactory;

import org.openide.xml.XMLUtil;

/*
import java.awt.image.BufferedImage;
import java.awt.Graphics;
import com.sun.image.codec.jpeg.JPEGCodec;
 */

/** Serve the NetBeans Explorer over HTTP.
 * @author Jesse Glick
 */
public class ExplorerServlet extends HttpServlet {
    private Node root;
    public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        Node n = findNode(getRoot(), req.getPathInfo());
        if (n == null) {
            res.sendError(404, "Could not find any node named: " + req.getPathInfo());
            return;
        }
        if (Boolean.valueOf(req.getParameter("icon")).booleanValue()) {
            // Node icon.
            Image icon = n.getIcon(BeanInfo.ICON_COLOR_16x16);
            BufferedImage bi = new BufferedImage(icon.getWidth(null), icon.getHeight(null), BufferedImage.TYPE_INT_ARGB);
            Graphics g = bi.getGraphics();
            g.drawImage(icon, 0, 0, null);
            g.dispose();
            res.setContentType("image/png");
            ImageIO.write(bi, "png", res.getOutputStream());
            res.setContentType("image/jpeg");
        } else if (req.getParameter("prop") != null) {
            // Property editor.
            res.setContentType("text/html");
            PrintWriter pw = res.getWriter();
            String propname = req.getParameter("prop");
            Node.PropertySet[] propsets = n.getPropertySets();
        FOUND:
            for (int i = 0; i < propsets.length; i++) {
                Node.Property[] props = propsets[i].getProperties();
                for (int j = 0; j < props.length; j++) {
                    Node.Property prop = props[j];
                    if (prop.getName().equals(propname)) {
                        pw.println("Back to node: <img src=\"?icon=true\"> <a href=\"?\">" + XMLUtil.toElementContent(n.getDisplayName()) + "</a>");
                        pw.println("<hr>Property: " + XMLUtil.toElementContent(prop.getDisplayName()));
                        String tooltip = prop.getShortDescription();
                        if (tooltip != null && tooltip.length() > 0 && !tooltip.equals(prop.getDisplayName())) {
                            pw.println("<br><i>" + XMLUtil.toElementContent(tooltip) + "</i>");
                        }
                        pw.println("<hr><form action=\"\" method=\"post\">");
                        PropertyEditor ed = props[j].getPropertyEditor();
                        Object val = null;
                        try {
                            val = props[j].getValue();
                            ed.setValue(val);
                            String text = ed.getAsText();
                            if (text != null) {
                                String[] tags = ed.getTags();
                                if (tags != null) {
                                    pw.println("<select name=\"" + propname + "\">");
                                    for (int k = 0; k < tags.length; k++) {
                                        pw.print("<option");
                                        if (text.equals(tags[k])) pw.print(" selected");
                                        pw.println(">" + XMLUtil.toElementContent(tags[k]) + "</option>");
                                    }
                                    pw.println("</select>");
                                } else {
                                    pw.print("<input type=\"text\" name=\"" + propname + "\" value=\"");
                                    pw.print(XMLUtil.toAttributeValue(text));
                                    pw.println("\">");
                                }
                                pw.println("<input type=\"submit\" name=\"__submit\" value=\"Change\">");
                            } else {
                                pw.print("<hr><i>object of type <code>" + props[j].getValueType() + "</code></i>");
                            }
                        } catch (Exception e) {
                            pw.print("<hr><i>thrown <code>" + e + "</code> with value <code>" + val + "</code></i>");
                        }
                        pw.println("</form>");
                        break FOUND;
                    }
                }
            }
        } else {
            // Info about the node and its children.
            res.setContentType("text/html");
            PrintWriter pw = res.getWriter();
            try {
                pw.println("Node: <img src=\"?icon=true\"> " + XMLUtil.toElementContent(n.getDisplayName()));
                if (n.getParentNode() != null) {
                    // XXX show full hierarchy down to here...
                    pw.println("<br>Parent: <img src=\"../?icon=true\"> <a href=\"../\">" + XMLUtil.toElementContent(n.getParentNode().getDisplayName()) + "</a>");
                }
                if (!n.isLeaf()) {
                    Children ch = n.getChildren();
                    pw.println("<hr>Children:<ol>");
                    Node[] kids = ch.getNodes(true);
                    Set names = new HashSet(100); // Set<String>
                    for (int i = 0; i < kids.length; i++) {
                        String kidname = encode(kids[i].getName());
                        if (!names.add(kidname)) {
                            kidname = "";
                        }
                        if (kidname.length() > 0) {
                            pw.println("<li><img src=\"" + kidname + "/?icon=true\"> <a href=\"" + kidname + "/\">" + XMLUtil.toElementContent(kids[i].getDisplayName()) + "</a>");
                            if (!kids[i].isLeaf()) pw.println("...");
                        } else {
                            pw.println("<li><i>" + XMLUtil.toElementContent(kids[i].getDisplayName()) + "</i>");
                        }
                    }
                    pw.println("</ol>");
                }
                Node.PropertySet[] propsets = n.getPropertySets();
                for (int i = 0; i < propsets.length; i++) {
                    pw.println("<hr>" + XMLUtil.toElementContent(propsets[i].getDisplayName()) + ":");
                    Node.Property[] props = propsets[i].getProperties();
                    pw.println("<table border=1><tr><th>Name</th><th>Value</th></tr>");
                    for (int j = 0; j < props.length; j++) {
                        if (!props[j].canRead()) continue;
                        pw.print("<tr><td>");
                        boolean writable = props[j].canWrite();
                        if (writable) pw.print("<a href=\"?prop=" + props[j].getName() + "\">");
                        pw.print(XMLUtil.toElementContent(props[j].getDisplayName()));
                        if (writable) pw.print("</a>");
                        pw.print("</td><td>");
                        PropertyEditor ed = props[j].getPropertyEditor();
                        Object val = null;
                        try {
                            val = props[j].getValue();
                            ed.setValue(val);
                            String text = ed.getAsText();
                            if (text != null) {
                                if (text.length() == 0) {
                                    pw.print("&nbsp;");
                                } else {
                                    pw.print(XMLUtil.toElementContent(text));
                                }
                            } else {
                                pw.print("<i>object of type <code>" + props[j].getValueType() + "</code></i>");
                            }
                        } catch (Exception e) {
                            pw.print("<i>thrown <code>" + e + "</code> with value <code>" + val + "</code></i>");
                        }
                        pw.println("</td></tr>");
                    }
                    pw.println("</table>");
                }
            } catch (RuntimeException re) {
                pw.println("<hr><pre>");
                re.printStackTrace(pw);
                pw.println("</pre>");
            }
            pw.close();
        }
    }
    public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        Node n = findNode(getRoot(), req.getPathInfo());
        if (n == null) {
            res.sendError(404, "Could not find any node named: " + req.getPathInfo());
            return;
        }
        Node.PropertySet[] propsets = n.getPropertySets();
        for (int i = 0; i < propsets.length; i++) {
            Node.Property[] props = propsets[i].getProperties();
            for (int j = 0; j < props.length; j++) {
                Node.Property prop = props[j];
                String nue = req.getParameter(prop.getName());
                if (nue != null) {
                    PropertyEditor ed = prop.getPropertyEditor();
                    try {
                        ed.setAsText(nue);
                        prop.setValue(ed.getValue());
                    } catch (Exception e) {
                        res.setContentType("text/html");
                        PrintWriter pw = res.getWriter();
                        pw.print("Problem setting value " + XMLUtil.toElementContent(nue) + " on property " + prop.getName() + ": <pre>");
                        e.printStackTrace(pw);
                        pw.println("</pre>");
                        pw.println("<hr><a href=\"\">Back to node</a>");
                        return;
                    }
                    // For some reason this does not work, nor "", nor "./", nor even "../nodename/"!
                    //res.sendRedirect(req.getRequestURI());
                    // Fine, just handle it as a get request then:
                    doGet(req, res);
                }
            }
        }
        res.setContentType("text/html");
        PrintWriter pw = res.getWriter();
        pw.println("No such property found");
    }
    private static String encode(String in) {
        if (in.length() == 0) {
            return "%";
        } else {
            return URLEncoder.encode(in);
        }
    }
    private static String decode(String in) {
        if (in.equals("%")) {
            return "";
        } else {
            return URLDecoder.decode(in);
        }
    }
    private Node getRoot() {
        if (root == null) {
            Children.Array kids = new Children.Array();
            FileSystem sfs = Repository.getDefault().getDefaultFileSystem();
            try {
                kids.add(new Node[] {
                    RepositoryNodeFactory.getDefault().repository(DataFilter.ALL).cloneNode(),
                    DataObject.find(sfs.findResource("UI/Runtime")).getNodeDelegate().cloneNode(),
                    DataObject.find(sfs.findResource("UI/Services")).getNodeDelegate().cloneNode(),
                    // XXX project desktop / workplace?
                });
            } catch (DataObjectNotFoundException e) {
                ErrorManager.getDefault().notify(e);
            }
            root = new AbstractNode(kids);
            root.setName("Root Node");
            ((AbstractNode)root).setIconBase("org/netbeans/examples/modules/explservlet/explorer");
        }
        return root;
    }
    private Node findNode(Node root, String path) {
        if (path == null) return null;
        if (path.equals("/")) return root;
        if (!path.startsWith("/")) return null;
        int idx = path.indexOf('/', 1);
        if (idx == -1) return null;
        Node first = root.getChildren().findChild(decode(path.substring(1, idx)));
        if (first == null) return null;
        return findNode(first, path.substring(idx));
    }
}
