/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.beans.BeanInfo;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.openide.ErrorManager;
import org.openide.cookies.EditorCookie;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataObject;
import org.openide.util.Utilities;
import org.openide.util.WeakListeners;
import org.openide.util.WeakSet;
import org.openide.util.datatransfer.ExTransferable;
import org.openide.util.datatransfer.NewType;
import org.openide.util.datatransfer.PasteType;

public class ScoreDataNode extends DataNode implements ChangeListener, PropertyChangeListener {
    private static final ErrorManager err =
        ErrorManager.getDefault().getInstance("org.netbeans.examples.modules.minicomposer.ScoreDataNode"); // NOI18N
    /** Whether this score has actually been opened in an editor.
     * If not, we will not bother to ask for a parse yet.
     */
    private boolean inUse = false;
    /** Files to which a listener has already been attached.
     */
    private final Set listenedFiles = Collections.synchronizedSet(new WeakSet()); // Set<FileObject>
    public ScoreDataNode(ScoreDataObject obj) {
        this(obj, (ScoreCookie)obj.getCookie(ScoreCookie.class));
    }
    private ScoreDataNode(ScoreDataObject obj, ScoreCookie score) {
        this(obj, score, new ScoreChildren(score));
    }
    private ScoreDataNode(ScoreDataObject obj, ScoreCookie score, ScoreChildren children) {
        super(obj, children);
        getCookieSet().add(children.getIndex());
        score.addChangeListener(WeakListeners.change(this, score));
        EditorCookie ed = (EditorCookie) getCookie(EditorCookie.class);
        if (ed != null && (ed instanceof EditorCookie.Observable)) {
            ((EditorCookie.Observable) ed).addPropertyChangeListener(WeakListeners.propertyChange(this, ed));
        } else {
            err.log(ErrorManager.WARNING, "Warning: no CloneableEditorSupport found on " + getDisplayName());
        }
    }
    /* XXX when per-file Player choice is implemented, do something like:
    protected Sheet createSheet() {
        Sheet sheet = super.createSheet();
        Sheet.Set properties = sheet.get(Sheet.PROPERTIES);
        properties.put(new ClipLengthProperty((ScoreDataObject)getDataObject(), this));
        Sheet.Set set = new Sheet.Set();
        set.setName("execution");
        set.setDisplayName(NbBundle.getMessage(ScoreDataNode.class, "LBL_Execution"));
        set.setShortDescription(NbBundle.getMessage(ScoreDataNode.class, "HINT_Execution"));
        ScoreExecSupport support = (ScoreExecSupport)getCookie(ScoreExecSupport.class);
        support.addProperties(set);
        sheet.put(set);
        return sheet;
    }
     */
    public Image getIcon(int type) {
        if (type == BeanInfo.ICON_COLOR_16x16 || type == BeanInfo.ICON_MONO_16x16) {
            Image icon = Utilities.loadImage("org/netbeans/examples/modules/minicomposer/resources/ScoreDataIcon.gif");
            err.log("getIcon: " + getDisplayName());
            ScoreCookie cookie = (ScoreCookie)getCookie(ScoreCookie.class);
            if (cookie != null) {
                if (inUse) {
                    cookie.prepare();
                }
                if (!cookie.isValid()) {
                    err.log("getIcon: error");
                    Image badge = Utilities.loadImage("org/netbeans/examples/modules/minicomposer/resources/error-badge.gif");
                    icon = Utilities.mergeImages(icon, badge, 8, 8);
                }
            }
            DataObject obj = getDataObject();
            try {
                icon = obj.getPrimaryFile().getFileSystem().getStatus().annotateIcon(icon, type, obj.files());
            } catch (FileStateInvalidException fsie) {
                // Forget about it.
                err.notify(ErrorManager.INFORMATIONAL, fsie);
            }
            return icon;
        } else {
            return null;
        }
    }
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }
    public void stateChanged(ChangeEvent ev) {
        if (!getDataObject().isValid()) return;
        Object src = ev.getSource();
        err.log("state change: source " + src);
        assert src instanceof ScoreCookie;
        err.log("icon change: state: " + getDisplayName());
        fireIconChange();
    }
    public void propertyChange(PropertyChangeEvent ev) {
        assert ev.getSource() instanceof EditorCookie;
        if (!inUse) {
            err.log("editor change: " + getDisplayName());
            inUse = true;
            fireIconChange();
        }
    }
    // Access from ClipLengthProperty:
    void fireClipLengthChange() {
        firePropertyChange("clipLength", null, null);
    }
    public Transferable clipboardCopy() throws IOException {
        Transferable deflt = super.clipboardCopy();
        ExTransferable enriched = ExTransferable.create(deflt);
        enriched.put(new ExTransferable.Single(NoteTransfer.SCORE_FLAVOR) {
            protected Object getData() throws IOException {
                return ((ScoreCookie)getCookie(ScoreCookie.class)).getScore();
            }
        });
        return enriched;
    }
    protected void createPasteTypes(Transferable t, List s) {
        super.createPasteTypes(t, s);
        PasteType p = NoteTransfer.createNotePasteType(t, (ScoreCookie)getCookie(ScoreCookie.class), null);
        if (p != null) {
            s.add(p);
        }
    }
    public NewType[] getNewTypes() {
        return new NewType[] {new NoteTransfer.NewNote((ScoreCookie)getCookie(ScoreCookie.class))};
    }
}
