/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.qreporter;

import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.BufferedReader;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;
import org.netbeans.qa.reports.*;

/**
 * Tool for creation of q-build report.
 *
 * @author dkaspar
 */
public class QReporter {
    
    public final static boolean DEBUG = true;
    public final static String PATH = QReporter.class.getProtectionDomain ().getCodeSource ().getLocation ().getFile ();
    
    private final static class ModuleDesc {
        public String name;
        public String[] modules;
        public ModuleDesc (String _name, String[] _modules) { name = _name; modules = _modules; }
        public void addToSet (Set set) {
            for (int a = 0; a < modules.length; a ++)
            set.add (modules[a]);
        }
        public boolean isIn (String name) {
            for (int a = 0; a < modules.length; a ++)
                if (modules[a].equals (name))
                    return true;
            return false;
        }
    };
    
    public final static ModuleDesc[] modules = new ModuleDesc[] {
        new ModuleDesc ("Ant", new String[] {"ant"}),
        new ModuleDesc ("AutoUpdate", new String[] {"autoupdate"}),
        new ModuleDesc ("Beans", new String[] {"beans"}),
        new ModuleDesc ("Clazz", new String[] {"clazz"}),
        new ModuleDesc ("Core", new String[] {"core"}),
        new ModuleDesc ("Debugger", new String[] {"debugger", "debuggercore", "debuggerjpda", "debuggertools"}),
        new ModuleDesc ("Editor", new String[] {"editor"}),
        new ModuleDesc ("Form", new String[] {"form"}),
        new ModuleDesc ("I18N", new String[] {"i18n"}),
        new ModuleDesc ("JarPackager", new String[] {"jarpackager"}),
        new ModuleDesc ("Java", new String[] {"java"}),
        new ModuleDesc ("JavaDoc", new String[] {"javadoc"}),
        new ModuleDesc ("Jini", new String[] {"jini"}),
        new ModuleDesc ("JNDI", new String[] {"jndi"}),
        new ModuleDesc ("OpenIDE", new String[] {"openide", "openidex"}),
        new ModuleDesc ("Projects", new String[] {"projects"}),
        new ModuleDesc ("Properties", new String[] {"properties"}),
        new ModuleDesc ("RMI", new String[] {"rmi"}),
        new ModuleDesc ("Scripting", new String[] {"scripting"}),
        new ModuleDesc ("Text", new String[] {"text"}),
        new ModuleDesc ("Utilities", new String[] {"utilities"}),
        new ModuleDesc ("VCS", new String[] {"javacvs", "vcscore", "vcscvs", "vcsgeneric"}),
        new ModuleDesc ("Web", new String[] {"web"}),
    };
    public final static Set modulesSet;
    
    PrintWriter out;
    boolean closeOutput;
    String buildNumber;
    String builtDate;
    boolean recommended;
    String comments;
    HashSet version;
    List issuesTestResults;
    List issuesLast;
    ArrayList issuesFix;
    String lastDate;
    String testDate;
    String nowDate;
    String fixDate;
    Date testDateDate;
    
    static {
        modulesSet = new HashSet ();
        for (int a = 0; a < modules.length; a ++)
            modules[a].addToSet (modulesSet);
    }
    
    public static void debug (String message) {
        if (DEBUG)
            System.out.println(message);
    }
    
    public QReporter () {
        if (System.getProperty ("http.proxyHost") == null)
            System.setProperty ("http.proxyHost", "webcache.uk.sun.com");
        if (System.getProperty ("http.proxyPort") == null)
            System.setProperty ("http.proxyPort", "8080");
        version = null;
    }
    
    public void queryIssuesLast () throws Exception {
        IssueQuery iq = new IssueQuery ();
        iq.status = 0; // overrides the default value
        iq.type = Issue.TYPE_DEFECT;
        iq.priority = Issue.PRIORITY_P1 | Issue.PRIORITY_P2;
        iq.product = modulesSet;
        iq.changedField = IssueQuery.CHANGED_FIELD_ISSUE_CREATION;
        iq.changedFrom = lastDate;
        iq.changedTo = testDate;
        debug ("Quering last issues from " + iq.changedFrom + " to " + iq.changedTo);
        issuesLast = Queries.getIssues (iq.getIssueNumbers ());
    }
    
    public void queryIssuesTestResults () throws Exception {
        IssueQuery iq = new IssueQuery ();
        iq.status = 0; // overrides the default value
        iq.type = Issue.TYPE_DEFECT;
        iq.priority = Issue.PRIORITY_P1 | Issue.PRIORITY_P2 | Issue.PRIORITY_P3 | Issue.PRIORITY_P4 | Issue.PRIORITY_P5;
        iq.product = modulesSet;
        iq.changedField = IssueQuery.CHANGED_FIELD_ISSUE_CREATION;
        iq.changedFrom = testDate;
        iq.changedTo = nowDate;
        debug ("Quering test results issues from " + iq.changedFrom + " to " + iq.changedTo);
        issuesTestResults = Queries.getIssues (iq.getIssueNumbers ());
    }
    
    public void printBegin () {
        out.println ("<HTML>");
        out.println ("<HEAD>");
        out.println ("<TITLE>Q-build Verification Report</TITLE>");
        out.println ("<LINK REL=\"Stylesheet\" HREF=\"http://www.netbeans.org/netbeans.css\" TYPE=\"text/css\" TITLE=\"NetBeans OpenSource Style\">");
        out.println ("</HEAD>");
        out.println ("<BODY>");
        out.println ("<H1>Q-build Verification Report</H1>");
    }
    
    void printHeader () {
        out.println ("<B>Build number:</B> <B>" + buildNumber + "</B>");
        out.println ("<BR><B>Date built/tested:</B> <I>" + builtDate + "</I>");
        out.println ("<BR><B>Recommended: <FONT COLOR=\"#FF0000\">" + ((recommended) ? "YES" : "NO") + "</FONT></B>");
        out.println ("<P><A HREF=\"index.html\">Q-builds index</A>");
        out.println (comments);
    }
    
    void printStoppers () {
        out.println ("<H3>Q-build stoppers</H3>");
        out.println ("<UL>");
        out.println ("<LI>|PENDING:Dont|");
        out.println ("</UL>");
    }
    
    boolean allowedLast (Issue issue) {
        if (version != null  &&  !version.contains (issue.version))
            return false;
        if (issue.resolution == Issue.RESOLUTION_DUPLICATE  ||  issue.resolution == Issue.RESOLUTION_INVALID  ||  issue.resolution == Issue.RESOLUTION_WORKSFORME  ||  issue.resolution == Issue.RESOLUTION_WONTFIX)
            return false;
        if (issue.resolution == Issue.RESOLUTION_FIXED) {
            Date date = null;
            String val = null;
            Iterator it;
            try {
                debug ("Testing activity of bug #" + issue.number);
                it = issue.getActivity().iterator();
            } catch (IOException e) {
                debug ("IOException raised when getting activity from bug #" + issue.number + " - adding into fix list!");
                return true;
            }
            while (it.hasNext()) {
                Change ch = (Change) it.next ();
//                debug ("Change date of bug #" + issue.number + " is " + ch.when + " : debug: " + testDateDate + " : " + ch.when.before (testDateDate));
//                debug (ch.what);
                debug(">>" + ch.what);
                if (ch.what.equals (IssueQuery.CHANGED_FIELD_RESOLUTION_S))
                    if (ch.when.before (testDateDate))
                        val = ch.newValue;
            }
            debug("VAL: "+val);
//            if (val == null)
//                debug ("No changed field resolution of fixed bug #" + issue.number);
            if (IssueQuery.RESOLUTION_FIXED.equals (val)) {
                System.out.println("DROP");
                return false;
            }
        }
        return true;
    }
    
    boolean allowedTest (Issue issue) {
        if (version != null  &&  !version.contains (issue.version))
            return false;
        if ((issue.priority & (Issue.PRIORITY_P1 | Issue.PRIORITY_P2)) == 0)
            return false;
        if (issue.resolution == Issue.RESOLUTION_DUPLICATE  ||  issue.resolution == Issue.RESOLUTION_INVALID  ||  issue.resolution == Issue.RESOLUTION_WORKSFORME  ||  issue.resolution == Issue.RESOLUTION_WONTFIX)
            return false;
        return true;
    }
    
    void printToFix () {
        issuesFix = new ArrayList ();
        Iterator iter = issuesLast.iterator ();
        while (iter.hasNext ()) {
            Issue issue = (Issue) iter.next ();
            if (allowedLast (issue)) {
                issuesFix.add (issue);
                System.out.println("ADD: " + issue.number);
            }
        }
        iter = issuesTestResults.iterator ();
        while (iter.hasNext ()) {
            Issue issue = (Issue) iter.next ();
            if (allowedTest (issue))
                issuesFix.add (issue);
        }
        out.print ("<H3>Bugs to Be Fixed");
        if (fixDate != null)
            out.print (" by " + fixDate);
        out.println ("</H3>");
        boolean br = false;
        StringBuffer link = null;
        for (int a = 0; a < issuesFix.size (); a ++) {
            Issue issue = (Issue) issuesFix.get (a);
            if ((issue.priority & Issue.PRIORITY_P1) == 0)
                continue;
            if (br)
                out.println ("<BR>");
            else
                br = true;
            out.println ("<A HREF=\"http://www.netbeans.org/issues/show_bug.cgi?id=" + issue.number + "\">" + issue.number + "</A>");
            out.print ("- <B>P1</B>: " + issue.summary);
            if (link != null)
                link.append("%2C+");
            else
                link = new StringBuffer ();
            link.append (issue.number);
        }
        for (int a = 0; a < issuesFix.size (); a ++) {
            Issue issue = (Issue) issuesFix.get (a);
            if ((issue.priority & Issue.PRIORITY_P2) == 0)
                continue;
            if (br)
                out.print ("<BR>");
            else
                br = true;
            out.println ("<A HREF=\"http://www.netbeans.org/issues/show_bug.cgi?id=" + issue.number + "\">" + issue.number + "</A>");
            out.print ("- <B>P2</B>: " + issue.summary);
            if (link != null)
                link.append("%2C+");
            else
                link = new StringBuffer ();
            link.append (issue.number);
        }
        if (link != null) {
            out.print ("<p>Use <a href=\"http://www.netbeans.org/issues/buglist.cgi?email1=&amp;emailtype1=substring&amp;emailassigned_to1=1&amp;email2=&amp;emailtype2=substring&amp;emailreporter2=1&amp;issueidtype=include&amp;issue_id=");
            out.print (link.toString());
            out.println ("&amp;changedin=&amp;votes=&amp;chfieldfrom=&amp;chfieldto=Now&amp;chfieldvalue=&amp;short_desc=&amp;short_desc_type=substring&amp;long_desc=&amp;long_desc_type=substring&amp;issue_file_loc=&amp;issue_file_loc_type=substring&amp;status_whiteboard=&amp;status_whiteboard_type=substring&amp;keywords=&amp;keywords_type=anywords&amp;field0-0-0=noop&amp;type0-0-0=noop&amp;value0-0-0=&amp;cmdtype=doit&amp;order=Reuse+same+sort+as+last+time\">this query</a> to check the current status of the bugs from the above list. </p>");
        }
    }
    
    void printDont () {
        out.println ("<H3>&quot;Don't&quot;s for This Build</H3>");
        out.println ("<UL>");
        out.println ("<LI>|PENDING:Dont|");
        out.println ("</UL>");
    }
    
    void printResults () {
        out.println ("<H3>Test Results</H3>");
        out.println ("The following table provides a complete list of all defects discovered during the testing of the Q-build.");
        out.println ("<P><TABLE BORDER CELLSPACING=0 CELLPADDING=0 WIDTH=\"90%\" CLASS=\"tablebg\">");
        out.println ("<TR><TD><TABLE BORDER=0 CELLPADDING=3 WIDTH=\"100%\"></TR>");
        out.println ("<TR><TD COLSPAN=\"6\"><FONT COLOR=\"white\">Build #" + buildNumber + " Results</FONT></TD></TR>");
        out.println ("<TR CLASS=\"tablecbg\">");
        out.println ("<TD ALIGN=CENTER WIDTH=\"20%\">Module</TD>");
        out.println ("<TD ALIGN=CENTER WIDTH=\"16%\">P1</TD>");
        out.println ("<TD ALIGN=CENTER WIDTH=\"16%\">P2</TD>");
        out.println ("<TD ALIGN=CENTER WIDTH=\"16%\">P3</TD>");
        out.println ("<TD ALIGN=CENTER WIDTH=\"16%\">P4</TD>");
        out.println ("<TD ALIGN=CENTER WIDTH=\"16%\">P5</TD>");
        out.println ("</TR>");
        for (int mod = 0; mod < modules.length; mod ++) {
            out.println ("<TR CLASS=\"tablerbg\">");
            out.println ("<TD ALIGN=CENTER>" + modules[mod].name + "</td>");
            for (int pri = 1; pri <= 5; pri ++) {
                ArrayList issues = new ArrayList ();
                Iterator iter = issuesTestResults.iterator ();
                while (iter.hasNext ()) {
                    Issue issue = (Issue) iter.next ();
                    if (modules[mod].isIn (issue.product)  &&  (issue.priority & (1 << (pri - 1))) != 0  &&  (version == null  ||  version.contains(issue.version)))
                        issues.add (issue);
                }
                if (issues.size () <= 0)
                    out.println ("<TD>&nbsp;</TD>");
                else {
                    out.print ("<TD>");
                    for (int cnt = 0; cnt < issues.size (); cnt ++) {
                        if (cnt != 0)
                            out.println (", ");
                        out.print ("<A HREF=\"http://www.netbeans.org/issues/show_bug.cgi?id=" + ((Issue) issues.get (cnt)).number + "\">" + ((Issue) issues.get (cnt)).number + "</A>");
                    }
                    out.println ("</TD>");
                }
            }
            out.println ("</TR>");
        }
        out.println ("</TABLE></TD></TR></TABLE>");
    }
    
    public void printEnd () {
        out.println ("</BODY>");
        out.println ("</HTML>");
    }
    
    public String getArg (String[] args, String param) {
        for (int a = 0; a < args.length; a ++) {
            if (args[a].startsWith (param))
                return args[a].substring (param.length ());
        }
        return null;
    }
    
    public String getArg (String[] args, String param, String def) {
        String str = getArg (args, param);
        return (str == null) ? def : str;
    }
    
    public void prepare (String[] args) {
        String arg;
        long curMillis = System.currentTimeMillis ();
        Date dat = new Date (curMillis);
        long midnight = curMillis - (curMillis % 86400000);
        arg = getArg (args, "-buildNumber=");
        if (arg == null) {
            buildNumber = new SimpleDateFormat ("yyyyMMdd'0100'").format (dat);
        } else {
            buildNumber = arg;
        }
        if (buildNumber.length () >= 8)
            builtDate = buildNumber.substring (4, 6) + "/" + buildNumber.substring (6, 8) + "/" + buildNumber.substring (0, 4);
        else {
            buildNumber = "|PENDING:buildNumber|";
            builtDate = "|PENDING:builtDate|";
        }
        debug ("Build number: " + buildNumber);
        debug ("built date: " + builtDate);
        arg = getArg (args, "-recommended=");
        if (arg != null)
            arg = arg.toLowerCase ();
        recommended = args == null  ||  "true".equals (arg)  ||  "yes".equals (arg)  ||  "on".equals (arg)  ||  "1".equals (arg); // |PENDING:recommended|
        debug ("Recommended: " + recommended);
        comments = "<P>|PENDING:comments|";
        arg = getArg (args, "-date=");
        SimpleDateFormat sdf = new SimpleDateFormat ("yyyy'-'MM'-'dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat ("MMMMM dd");
        nowDate = sdf.format (dat) + "+09%3A00%3A00";
        if (arg == null) {
            testDate = sdf.format (dat) + "+01%3A00%3A00";
        } else {
            try {
                dat = sdf.parse (arg);
            } catch (Exception e) {
                e.printStackTrace ();
                dat = null;
            }
            testDate = arg + "+01%3A00%3A00";
        }
        Date lastDateDate = new Date (midnight - 7 * 24 * 60 * 60 * 1000);
        testDateDate = new Date (dat.getTime () - (dat.getTime () % 86400000) - 60 * 60 * 1000);
        lastDate = sdf.format (lastDateDate) + "+01%3A00%3A00";
        debug ("Last date: " + lastDate);
        debug ("Test date: " + testDate);
        debug ("Now date: " + nowDate);
        debug ("Test date Date: " + testDateDate.toString());
        if (dat != null) {
            dat = new Date (dat.getTime () + 6 * 24 * 60 * 60 * 1000);
            fixDate = sdf2.format (dat);
            if (fixDate != null  &&  fixDate.length () >= 2)
                switch (fixDate.charAt (fixDate.length () - 1)) {
                    case '1':
                        if (fixDate.charAt (fixDate.length () - 2) != '1')
                            fixDate += "st";
                        else
                            fixDate += "th";
                        break;
                    case '2':
                        if (fixDate.charAt (fixDate.length () - 2) != '1')
                            fixDate += "nd";
                        else
                            fixDate += "th";
                        break;
                    case '3':
                        if (fixDate.charAt (fixDate.length () - 2) != '1')
                            fixDate += "rd";
                        else
                            fixDate += "th";
                        break;
                    default:
                        fixDate += "th";
                }
        } else
            fixDate = null;
        debug ("FixDate: " + fixDate);
        String versionStr = getArg (args, "-version=");
        if (versionStr != null) {
            version = new HashSet ();
            StringTokenizer st = new StringTokenizer (versionStr, ",");
            while (st.hasMoreTokens())
                version.add (st.nextToken());
        }
        if (version != null)
            debug ("Versions: " + version);
    }
    
    public void print (String[] args) {
        debug ("Preparing");
        prepare (args);
        try {
            out = new PrintWriter (new FileWriter (PATH + "/Q-build-report-" + buildNumber + ".html"));
            closeOutput = true;
        } catch (IOException e) {
            e.printStackTrace ();
            out = new PrintWriter (System.out, true);
            closeOutput = false;
        }
        try {
            queryIssuesLast ();
            queryIssuesTestResults ();
            debug ("Printing report");
            printBegin ();
            printHeader ();
            printStoppers ();
            printToFix ();
            printDont ();
            printResults ();
            printEnd ();
            debug ("Report was created");
        } catch (Exception e) {
            e.printStackTrace ();
        } finally {
            if (closeOutput) {
                try {
                    out.close ();
                } catch (Exception e) {
                    e.printStackTrace ();
                }
            }
        }
    }
    
    public static void main (String[] args) {
        new QReporter ().print (args);
    }
    
}
