/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.reports.example;

import java.io.*;
import java.util.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import org.netbeans.qa.reports.*;

import org.apache.tools.ant.*;
import org.apache.tools.ant.types.*;

/**
 * Tool for creation of bug summaries.
 *
 * @author Jan Stola
 */
public class Reporter extends Task {
    /** Determines whether category for High priority bugs should be included. */
    private boolean listP12Bugs=true;
    /** Determines whether category for Enhancements should be included. */
    private boolean enhancements=true;
    /** Determines whether category for Features should be included. */
    private boolean features=false;
    /** Determines whether category for Patches should be included. */
    private boolean patches=false;
    /** Determines whether category for Tasks should be included. */
    private boolean tasks=false;
    /** Determines whether list of bugs assigned to individual developers should be included. */
    private boolean listAssignees=false;
    /** Determines whether list of New bugs should be included. */
    private boolean listNew=false;
    /** Determines whether list of Resolved bugs should be included. */
    private boolean listResolved=false;
    /** Set of keywords for which a category should be generated. */
    private Set keywords=new HashSet();
    /** Set of components for which a category should be generated. */
    private Set components=new HashSet();
    /** set of versions */
    private Set versions=new HashSet();
    /** Determines whether the reported should run in a verbose mode. */
    private boolean verbose=false;
    /** Output stream. */
    private PrintWriter pw;
    /** Reference date. */
    private Date date;
    /** Set of modules for which the report should be generated. */
    private Set modules=new HashSet();
    /** Counter of sections. */
    private int section=1;
    
    /**
     * Prints usage of the reporter.
     *
     * @param msg message printed as a first line (usually error message).
     */
    public void printUsage(String msg) {
        System.out.println(msg);
        System.out.println();
        System.out.println("USAGE Reporter [-v] [-o <file>] <date> <module>...");
        System.out.println("----------------------------------------------------");
        System.out.println("PARAMETERS:");
        System.out.println("<date>        date in yyyy-MM-dd-HH:mm:ss format");
        System.out.println("<module>...   list of modules");
        System.out.println("-v            verbose mode");
        System.out.println("-o            output file (default is Report.html)");
    }
    
    /**
     * Parses arguments (when used as a standalone application).     
     *
     * @param args command line arguments.
     */
    public void readArguments(String args[]) throws IOException {
        int i=0;
        
        if (args.length<2) {
            printUsage("Not enough arguments.");
            System.exit(1);
        }
        while (args[i].startsWith("-")) {
            boolean ok=false;
            if (args[i].startsWith("-v")) {
                Queries.verbose=true;
                ok=true;
                verbose=true;
            }
            if (args[i].startsWith("-o")||args[i].startsWith("-vo")) {
                i++;
                pw=new PrintWriter(new FileWriter(args[i]));
                ok=true;
            }
            if (!ok) {
                printUsage("Unknown switch: "+args[i]);
                System.exit(1);
            }
            i++;
            if (i+2>args.length) {
                printUsage("Not enough arguments.");
                System.exit(1);
            }
        }
        if (pw==null) {
            pw=new PrintWriter(new FileWriter("Report.html"));
        }
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss");
        try {
            date=sdf.parse(args[i]);
        } catch (ParseException e) {
            e.printStackTrace();
            return;
        }
        date.setHours(date.getHours()-9);
        i++;
        modules=new HashSet();
        for (;i<args.length; i++) {
            modules.add(args[i]);
        }
        components.add("ui");
    }
    
    // ANT
    /**
     * Setter for <code>listP12Bugs</code>.
     *
     * @param bool new value of the property.
     */
    public void setListp12bugs(String bool) {
        if (bool.equalsIgnoreCase("false")||bool.equalsIgnoreCase("no")) {
            listP12Bugs=false;
        }
    }
    
    // ANT
    /**
     * Setter for <code>enhancements</code>.
     *
     * @param bool new value of the property.
     */     
    public void setEnhancements(String bool) {
        if (bool.equalsIgnoreCase("false")||bool.equalsIgnoreCase("no")) {
            enhancements=false;
        }
    }
    
    // ANT
    /**
     * Setter for <code>features</code>.
     *
     * @param bool new value of the property.
     */
    public void setFeatures(String bool) {
        if (bool.equalsIgnoreCase("true")||bool.equalsIgnoreCase("yes")) {
            features=true;
        }
    }
    
    // ANT
    /**
     * Setter for <code>patches</code>.
     *
     * @param bool new value of the property.
     */
    public void setPatches(String bool) {
        if (bool.equalsIgnoreCase("true")||bool.equalsIgnoreCase("yes")) {
            patches=true;
        }
    }
    
    // ANT
    /**
     * Setter for <code>tasks</code>.
     *
     * @param bool new value of the property.
     */
    public void setTasks(String bool) {
        if (bool.equalsIgnoreCase("true")||bool.equalsIgnoreCase("yes")) {
            tasks=true;
        }
    }
    
    // ANT
    /**
     * Setter for <code>listAssignees</code>.
     *
     * @param bool new value of the property.
     */
    public void setListAssignees(String bool) {
        if (bool.equalsIgnoreCase("true")||bool.equalsIgnoreCase("yes")) {
            listAssignees=true;
        }
    }
    
    // ANT
    /**
     * Setter for <code>listNew</code>.
     *
     * @param bool new value of the property.
     */
    public void setListNew(String bool) {
        if (bool.equalsIgnoreCase("true")||bool.equalsIgnoreCase("yes")) {
            listNew=true;
        }
    }
    
    // ANT
    /**
     * Setter for <code>listResolved</code>.
     *
     * @param bool new value of the property.
     */
    public void setListResolved(String bool) {
        if (bool.equalsIgnoreCase("true")||bool.equalsIgnoreCase("yes")) {
            listResolved=true;
        }
    }
    
    // ANT
    /**
     * Setter for <code>keywords</code>.
     *
     * @param words comma separated list of keywords.
     */
    public void setKeywords(String words) {
        StringTokenizer st = new StringTokenizer(words, ",");
        keywords = new HashSet();
        while (st.hasMoreElements()) {
            keywords.add(st.nextElement());
        }
    }
    
    // ANT
    /**
     * Setter for <code>components</code>.
     *
     * @param comps comma separated list of components.
     */     
    public void setComponents(String comps) {
        StringTokenizer st = new StringTokenizer(comps, ",");
        components = new HashSet();
        while (st.hasMoreElements()) {
            components.add(st.nextElement());
        }
    }
    
    // ANT
    /**
     * Setter for <code>versions</code>.
     *
     * @param comps comma separated list of versions.
     */     
    public void setVersions(String vers) {
        StringTokenizer st = new StringTokenizer(vers, ",");
        versions = new HashSet();
        while (st.hasMoreElements()) {
            versions.add(st.nextElement());
        }
    }
    
    // ANT
    /**
     * Setter for <code>verbose</code>.
     *
     * @param verb should be <code>true</code> or <code>yes</code> for verbose mode.
     */
    public void setVerbose(String verb) {
        if ( verb.equalsIgnoreCase("true") || verb.equalsIgnoreCase("yes") ) {
            Queries.verbose=true;
            verbose=true;
        }
    }
    
    // ANT
    /**
     * Setter for <code>modules</code>.
     *
     * @param mods comma separated list of modules.
     */
    public void setModules(String mods) {
        StringTokenizer st = new StringTokenizer(mods,",");
        modules = new HashSet();
        while (st.hasMoreElements()) {
            modules.add(st.nextElement());
        }
    }
    
    // ANT
    /**
     * Setter for output stream.
     *
     * @param file name of the output file.
     */
    public void setOutputfile(String file) {
        try {
            pw = new PrintWriter(new FileWriter(file));
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }
    
    // ANT
    /**
     * Setter for reference date.
     *
     * @param dt reference date.
     */
    public void setDate(String dt) {
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss");
        try {
            date=sdf.parse(dt);
        } catch (ParseException e) {
            e.printStackTrace();
            return;
        }
        date.setHours(date.getHours()-9);
    }
    
    // ANT
    /**
     * Setter for proxy host.
     *
     * @param host proxy host.
     */
    public void setProxyhost(String host) {
        System.setProperty("http.proxyHost", host);
    }
    
    // ANT
    /**
     * Setter for proxy port.
     *
     * @param port proxy port.
     */
    public void setProxyport(String port) {
        System.setProperty("http.proxyPort", port);
    }
    
    /**
     * Generates header of the report.
     */
    public void generateHeader() throws IOException {
        SimpleDateFormat sdf=new SimpleDateFormat("dd/MM/yyyy");
        String dates=sdf.format(date)+" - "+sdf.format(new Date(System.currentTimeMillis()));
        
        StringBuffer sbvers=new StringBuffer("Versions:");
        Iterator itervers = versions.iterator();
        int numVers=0;
        while (itervers.hasNext()) {
            numVers++;
            sbvers.append(" ");
            sbvers.append(((String) itervers.next()).replace('+', ' '));
            sbvers.append(",");
        }
        sbvers.deleteCharAt(sbvers.length()-1);        
        
        pw.println("<HTML><HEAD><TITLE>Bug summary "+dates+"</TITLE>");
        pw.print("<link rel=\"Stylesheet\" href=\"http://www.netbeans.org/netbeans.css\" ");
        pw.println("type=\"text/css\" title=\"NetBeans OpenSource Style\"></HEAD>");
        pw.println("<BODY><HR>");
        StringBuffer sb=new StringBuffer("<center><h1>Bugs summary for ");
        Iterator iter=modules.iterator();
        if (modules.size()==1) {
            sb.append(iter.next()).append(" module.");
        } else {
            sb.append("modules:");
            while (iter.hasNext()) {
                sb.append(' ').append(iter.next()).append(',');
            }
            sb.setCharAt(sb.length()-1, '.');
        }
        sb.append("</h1></center>");
        pw.println(sb.toString());
        pw.println("<center><h2>"+dates+"</h2></center>");
        if (numVers>0) {
            pw.println("<center><h3>"+sbvers.toString()+"</h3></center>");
        }
        pw.println("<hr>");
        pw.println("<ul>\n<li><b><a href=\"#counts\">Bug counts</a></b></li>");
        if(listNew) {
            pw.println("<li><b><a href=\"#newBugs\">New bugs</a></b></li>");
        }
        if(listResolved){
            pw.println("<li><b><a href=\"#resolvedBugs\">Resolved bugs</a></b></li>");
        }
        if (listP12Bugs) {
            pw.println("<li><b><a href=\"#highPriority\">High priority bugs (P1 and P2)</a></b></li>");
        }
        if (enhancements) {
            pw.println("<li><b><a href=\"#Enhancements\">Enhancements</a></b></li>");
        }
        if (features) {
            pw.println("<li><b><a href=\"#Features\">Features</a></b></li>");
        }
        if (patches) {
            pw.println("<li><b><a href=\"#Patches\">Patches</a></b></li>");
        }
        if (tasks) {
            pw.println("<li><b><a href=\"#Tasks\">Tasks</a></b></li>");
        }
        iter=components.iterator();
        while (iter.hasNext()) {
            Object o=iter.next();
            pw.println("<li><b><a href=\"#"+o+"\">"+o+" bugs </a></b></li>");
        }
        iter=keywords.iterator();
        while (iter.hasNext()) {
            Object o=iter.next();
            pw.println("<li><b><a href=\"#"+o+"\">"+o+" bugs</a></b></li>");
        }
        if (listAssignees) {
            pw.println("<li><b><a href=\"#assignee\">Bugs by assignee</a></b></li>");
        }
        pw.println("</ul>");
    }
    
    /**
     * Generates footer of the report.
     */
    public void generateFooter() throws IOException {
        pw.println("</BODY></HTML>");
        pw.close();
    }
    
    /**
     * Generates list of issues.
     *
     * @param bugs collection of bugs for which the list should be generated.
     * @param color color of the summary.
     * @param status determines whether the status should be printed.
     */
    public void generateBugList(Collection bugs, String color, boolean status) {

        // sort bug collection
        ArrayList bugList = new ArrayList(bugs);
        Collections.sort(bugList, new BugComparator());
        
        pw.println("<TABLE cellpadding=\"3\">");        
        Iterator iter=bugList.iterator();
        while (iter.hasNext()) {
            Issue issue=(Issue)iter.next();
            pw.print("<tr><td ALIGN=\"RIGHT\"><a href=\"http://www.netbeans.org/issues/show_bug.cgi?id="+issue.number+"\">");
            pw.print(issue.number+"</a></td><td ALIGN=\"CENTER\">");
            String s=null;
            switch (issue.priority) {
                case (Issue.PRIORITY_P1): s="P1"; break;
                case (Issue.PRIORITY_P2): s="P2"; break;
                case (Issue.PRIORITY_P3): s="P3"; break;
                case (Issue.PRIORITY_P4): s="P4"; break;
                case (Issue.PRIORITY_P5): s="P5"; break;
                default: s="?";
            }
            pw.print("<b>"+s+"</b></td>");
            pw.println("<td>"+issue.product+"</td>");
            if (status) {
                pw.println("<td ALIGN=\"CENTER\">");
                switch (issue.resolution) {
                    case (Issue.RESOLUTION_DUPLICATE): s="DUPLICATE"; break;
                    case (Issue.RESOLUTION_FIXED): s="FIXED"; break;
                    case (Issue.RESOLUTION_INVALID): s="INVALID"; break;
                    case (Issue.RESOLUTION_LATER): s="LATER"; break;
                    case (Issue.RESOLUTION_MOVED): s="MOVED"; break;
                    case (Issue.RESOLUTION_REMIND): s="REMIND"; break;
                    case (Issue.RESOLUTION_WONTFIX): s="WONTFIX"; break;
                    case (Issue.RESOLUTION_WORKSFORME): s="WORKSFORME"; break;
                    default: s="?";
                }
                pw.print("["+s+"]");
                pw.println("</td>");
            }
            pw.println("<td><font color=\""+color+"\">"+issue.summary+"</font></td></tr>");
        }
        pw.println("</TABLE>");
    }
    
    /**
     * Generates main parts of the report.
     */     
    public void generateMainParts() throws IOException {
        /* Obtain absolute issue counts. */
        if (verbose) System.out.println("Obtaining bug counts.");
        Map issueCounts=new HashMap();
        Map relIssueCounts=new HashMap();
        Iterator iter=modules.iterator();
        while (iter.hasNext()) {
            String module=(String)iter.next();
            issueCounts.put(module, Queries.getBugCounts(module, versions));
            relIssueCounts.put(module, new IssueCounts());
        }
        /* Obtain relative issue counts, resolved and opened issues. */
        if (verbose) System.out.println("Obtaining changed issues.");
        List changedIssues=Queries.getChangedIssueNumbers(date);
        //System.out.println("changedIssues contains: " + changedIssues.size() + " bugs");
        IssueQuery iq=new IssueQuery();
        List ourChangedIssues=new LinkedList();
        List part=new LinkedList();
        iq.product=modules;
        iq.version=versions;
        int p=0;
        while (p<changedIssues.size()) {
            part.add(changedIssues.get(p));
            p++;
            if ((p%200)==0) {                
                iq.issueNumbers=part;
                ourChangedIssues.addAll(iq.getIssues());
                part=new LinkedList();
            }
        }
        iq.issueNumbers=part;
        ourChangedIssues.addAll(iq.getIssues());
        Set[] newIssues=new TreeSet[5];
        Set[] resolvedIssues=new TreeSet[5];
        for (int i=0; i<5; i++) {
            newIssues[i]=new TreeSet(new Queries.IssueComparator());
            resolvedIssues[i]=new TreeSet(new Queries.IssueComparator());
        }
        iter=ourChangedIssues.iterator();
        //System.out.println("ourChangedIssues contains: " + ourChangedIssues.size() + " bugs");
        while (iter.hasNext()) {
            int change=0;
            Issue issue=(Issue)iter.next();
            int newSince=issue.newSince(date, modules);
            int resolvedSince=issue.resolvedSince(date, modules);
            int priorityChangeSince=issue.priorityChangeSince(date);
            int priority=-1;
            switch (issue.priority) {
                case Issue.PRIORITY_P1: priority=0; break;
                case Issue.PRIORITY_P2: priority=1; break;
                case Issue.PRIORITY_P3: priority=2; break;
                case Issue.PRIORITY_P4: priority=3; break;
                case Issue.PRIORITY_P5: priority=4; break;
            }
            if (0!=newSince) {
                switch (issue.type) {
                    case Issue.TYPE_DEFECT: newIssues[0].add(issue); change++; break;
                    case Issue.TYPE_ENHANCEMENT: newIssues[1].add(issue); break;
                    case Issue.TYPE_FEATURE: newIssues[2].add(issue); break;
                    case Issue.TYPE_PATCH: newIssues[3].add(issue); break;
                    case Issue.TYPE_TASK: newIssues[4].add(issue); break;
                }
            }
            if (0!=resolvedSince) {
                switch (issue.type) {
                    case Issue.TYPE_DEFECT: resolvedIssues[0].add(issue); change--; break;
                    case Issue.TYPE_ENHANCEMENT: resolvedIssues[1].add(issue); break;
                    case Issue.TYPE_FEATURE: resolvedIssues[2].add(issue); break;
                    case Issue.TYPE_PATCH: resolvedIssues[3].add(issue); break;
                    case Issue.TYPE_TASK: resolvedIssues[4].add(issue); break;
                }
            }
            if ((priorityChangeSince!=0)&&(newSince==0)&&(issue.type==Issue.TYPE_DEFECT)) {
                IssueCounts ic=(IssueCounts)relIssueCounts.get(issue.product);
                ic.priorities[priority-priorityChangeSince]-=1;
                ic.priorities[priority]+=1;
            }
            if (change!=0) {
                IssueCounts ic=(IssueCounts)relIssueCounts.get(issue.product);
                if (ic==null) {
                System.out.println(issue.product);
                System.out.println(relIssueCounts.keySet());
                }
                ic.priorities[priority]+=change;
            }
        }
        /* Header of the table. */
        if (verbose) System.out.println("Generating bug counts.");
        pw.println("<HR><H4><a name=\"counts\"></a>"+section+++". Bug counts</H4>");
        pw.println("<table width=\"60%\" border=\"1\" cellspacing=\"0\" cellpadding=\"3\"> <tr>");
        pw.println("<td ALIGN=LEFT>Module / Priority</td>");
        for (int i=1; i<6; i++) {
            pw.println("<td ALIGN=CENTER WIDTH=\"50\"><b>P"+i+"</b></td>");
        }
        pw.println("<td ALIGN=CENTER><b>All</b></td><td ALIGN=CENTER><b>CQI</b></td></tr>");
        iter=modules.iterator();
        int[][] priorities=new int[7][2];
        while (iter.hasNext()) {
            String module=(String)iter.next();
            pw.println("<tr><td ALIGN=LEFT><b>"+module+"</b></td>");
            IssueCounts absolute=(IssueCounts)issueCounts.get(module);
            IssueCounts relative=(IssueCounts)relIssueCounts.get(module);
            int all=0;
            int rel=0;
            for (int i=0; i<5; i++) {
                priorities[i][0]+=absolute.priorities[i];
                priorities[i][1]+=relative.priorities[i];
                all+=absolute.priorities[i];
                rel+=relative.priorities[i];
                pw.print("<td ALIGN=CENTER>"+absolute.priorities[i]);
                if (relative.priorities[i]>0) {
                    pw.print("<font color=\"red\"> (+"+relative.priorities[i]+")</font>");
                } else if (relative.priorities[i]<0) {
                    pw.print("<font color=\"blue\"> ("+relative.priorities[i]+")</font>");
                }
                pw.println("</td>");
            }
            priorities[5][0]+=all;
            priorities[5][1]+=rel;
            pw.print("<td ALIGN=CENTER>"+all);
            if (rel>0) {
                pw.print("<font color=\"red\"> (+"+rel+")</font>");
            } else if (rel<0) {
                pw.print("<font color=\"blue\"> ("+rel+")</font>");
            }
            pw.println("</td>");
            priorities[6][0]+=absolute.getCQI();
            priorities[6][1]+=relative.getCQI();
            pw.print("<td ALIGN=CENTER>"+absolute.getCQI());
            if (relative.getCQI()>0) {
                pw.print("<font color=\"red\"> (+"+relative.getCQI()+")</font>");
            } else if (relative.getCQI()<0) {
                pw.print("<font color=\"blue\"> ("+relative.getCQI()+")</font>");
            }
            pw.println("</td></tr>");
        }
        if (modules.size()>1) {
            pw.println("<tr><td ALIGN=LEFT><b>all modules</b></td>");
            for (int i=0; i<5; i++) {
                pw.print("<td ALIGN=CENTER>"+priorities[i][0]);
                if (priorities[i][1]>0) {
                    pw.print("<font color=\"red\"> (+"+priorities[i][1]+")</font>");
                } else if (priorities[i][1]<0) {
                    pw.print("<font color=\"blue\"> ("+priorities[i][1]+")</font>");
                }
                pw.println("</td>");
            }
            pw.print("<td ALIGN=CENTER>"+priorities[5][0]);
            if (priorities[5][1]>0) {
                pw.print("<font color=\"red\"> (+"+priorities[5][1]+")</font>");
            } else if (priorities[5][1]<0) {
                pw.print("<font color=\"blue\"> ("+priorities[5][1]+")</font>");
            }
            pw.println("</td>");
            pw.print("<td ALIGN=CENTER>"+priorities[6][0]);
            if (priorities[6][1]>0) {
                pw.print("<font color=\"red\"> (+"+priorities[6][1]+")</font>");
            } else if (priorities[6][1]<0) {
                pw.print("<font color=\"blue\"> ("+priorities[6][1]+")</font>");
            }
            pw.println("</td></tr>");
        }
        pw.println("</table><br/>");
        
        // new bugs
        pw.println("<HR><H4><a NAME=\"newBugs\"></a>"+section+++". New bugs ("+newIssues[0].size()+")</H4>");
        if (newIssues[0].size()==0) {
            pw.println("No new bugs.");
        } else {
            generateBugList(newIssues[0], "color", false);
        }
        
        // resolved bugs
        pw.println("<HR><H4><a NAME=\"resolvedBugs\"></a>"+section+++". Resolved bugs ("+resolvedIssues[0].size()+")</H4>");
        if (resolvedIssues[0].size()==0) {
            pw.println("No resolved bugs.");
        } else {
            generateBugList(resolvedIssues[0], "blue", true);
        }
        generateHighPriorityBugs();
        for (int i=1; i<5; i++) {
            String s=null;
            switch (i) {
                case (1): if (!enhancements) continue; s="Enhancements"; break;
                case (2): if (!features) continue; s="Features"; break;
                case (3): if (!patches) continue; s="Patches"; break;
                case (4): if (!tasks) continue; s="Tasks"; break;
            }
            pw.println("<HR><H4><a NAME=\""+s+"\"></a>"+section+++". "+s+"</H4>");
            if ((newIssues[i].size()|resolvedIssues[i].size())==0) {
                pw.println("No changed "+s.toLowerCase()+".<br>");
                continue;
            }
            pw.println("<b>New ("+newIssues[i].size()+")</b><br>");
            if (newIssues[i].size()==0) {
                pw.println("No new "+s.toLowerCase()+".<br>");
            } else {
                generateBugList(newIssues[i], "red", false);
            }
            pw.println("<br><b>Resolved ("+resolvedIssues[i].size()+")</b><br>");
            if (resolvedIssues[i].size()==0) {
                pw.println("No resolved "+s.toLowerCase()+".<br>");
            } else {
                generateBugList(resolvedIssues[i], "blue", false);
            }
            
        }
    }
    
    /**
     * Generates High Priority Bugs part.     
     */
    public void generateHighPriorityBugs() throws IOException {
        if (verbose) System.out.println("Generating high priority bugs.");
        IssueQuery iq=new IssueQuery();
        iq.status=Issue.STATUS_OPEN;
        iq.product=modules;
        iq.version=versions;
        iq.type=Issue.TYPE_DEFECT;
        iq.priority=Issue.PRIORITY_P1|Issue.PRIORITY_P2;
        List bugs=iq.getIssues();
        pw.println("<HR><H4><a NAME=\"highPriority\"></a>"+section+++". High priority bugs ("+bugs.size()+")</H4>");
        if (bugs.size()==0) {
            pw.println("No high priority bugs");
        } else {
            generateBugList(bugs, "black", false);
        }
    }
    
    /**
     * Generates part for one component.
     *
     * @param component component for which this part is generated.
     */
    public void generateBugsByComponent(String component) throws IOException {
        if (verbose) System.out.println("Generating "+component+" bugs.");
        IssueQuery iq=new IssueQuery();
        Set comp=new HashSet(1);
        comp.add(component);
        iq.status=Issue.STATUS_OPEN;
        iq.product=modules;
        iq.version=versions;
        iq.type=Issue.TYPE_DEFECT;
        iq.component=comp;
        List bugs=iq.getIssues();
        pw.println("<HR><H4><a NAME=\""+component+"\"></a>"+section+++". Bugs assigned to subcomponent - <b>"+component+"</b> ("+bugs.size()+")</H4>");
        if (bugs.size()==0) {
            pw.println("No "+component+" related bugs.");
        } else {
            generateBugList(bugs, "black", false);
        }
    }
    
    /**
     * Generates part for one keyword.
     *
     * @param keyword keyword for which this part is generated.
     */
    public void generateBugsByKeyword(String keyword) throws IOException {
        if (verbose) System.out.println("Generating bugs with keyword: "+keyword);
        List bugs=Queries.getIssues(Queries.getBugNumbers(modules, keyword, versions));
        pw.println("<HR><H4><a NAME=\""+keyword+"\"></a>"+section+++". Bugs with assigned keyword - <b>"+keyword.toUpperCase()+"</b> ("+bugs.size()+")</H4>");
        if (bugs.size()==0) {
            pw.println("No bugs with keyword "+keyword);
        } else {
            generateBugList(bugs, "black", false);
        }
    }
    
    /**
     * Generates lists of bugs for individual developers.     
     */
    public void generateBugsByAssignee() throws IOException {
        if (verbose) System.out.println("Generating bugs by assignee.");
        pw.println("<HR><H4><a NAME=\"assignee\"></a>"+section+++". Bugs by assignee</H4>");
        List issues=Queries.getIssues(Queries.getBugNumbers(modules, versions));
        if (issues.size()==0) {
            pw.println("No bugs.");
            return;
        }
        Map assignedBugs=new HashMap();
        Iterator iter=issues.iterator();
        while (iter.hasNext()) {
            Issue issue=(Issue)iter.next();
            String key=issue.assignedTo;
            if (key.endsWith(".netbeans.org")) {
                key="not assigned";
            }
            if (assignedBugs.containsKey(key)) {
                List bugs=(List)assignedBugs.get(key);
                bugs.add(issue);
            } else {
                List bugs=new LinkedList();
                bugs.add(issue);
                assignedBugs.put(key, bugs);
            }
        }
        iter=assignedBugs.keySet().iterator();
        List notAssigned=new LinkedList();
        while (iter.hasNext()) {
            String assignee=(String)iter.next();
            List bugs=(List)assignedBugs.get(assignee);
            if (assignee.equals("not assigned")) {
                notAssigned=bugs;
                continue;
            }
            pw.println("<b>Bugs assigned to <a href=\"mailto:"+assignee+"\">"+assignee+"</a> ("+bugs.size()+")</b>");
            generateBugList(bugs, "black", false);
            pw.println("<br>");
        }
        if (notAssigned.size()!=0) {
            pw.println("<b>Not assigned bugs. ("+notAssigned.size()+")</b>");
            generateBugList(notAssigned, "black", false);
        }
    }
    
    /**
     * Generates the whole report.
     */
    public void generateReport() throws IOException {
        Iterator iter;
        generateHeader();
        generateMainParts();
        iter=components.iterator();
        while (iter.hasNext()) {
            generateBugsByComponent((String)iter.next());
        }
        iter=keywords.iterator();
        while (iter.hasNext()) {
            generateBugsByKeyword((String)iter.next());
        }
        if (listAssignees) {
            generateBugsByAssignee();
        }
        generateFooter();
    }
    
    /**
     * Entry point when this class is used as a standalone application.
     */
    public static void main(String args[]) throws IOException {
        Reporter reporter=new Reporter();
        reporter.readArguments(args);
        reporter.generateReport();
    }
    
    //ANT
    /**
     * Entry point when this class is used as an ant script.
     */
    public void execute() throws BuildException {
        try {
            generateReport();
        } catch (IOException ioe) {
            throw new BuildException(ioe.toString());
        }
    }
    
    public static class BugComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            Issue i1 = (Issue) o1;
            Issue i2 = (Issue) o2;
            return i1.product.compareTo(i2.product); 
        }
    }
    
}
