/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import javax.swing.JComponent;
import javax.swing.JFrame;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.ui.tree.Node;
import org.netbeans.testspeceditor.ui.tree.menu.PopupSupport;

/**
 *
 * @author ehucka
 */
public class ActionsManager {
    
    static ActionsManager instance = null;
    
    HashMap<Class<? extends Node>, NodeAction[]> actionsType;
    HashMap<Class<? extends Action>, Action> actions;
    
    
    public ActionsManager() {
        actionsType = new HashMap<Class<? extends Node>, NodeAction[]>(16);
        actions = new HashMap<Class<? extends Action>, Action>(16);
    }
    
    public static ActionsManager getInstance() {
        if (instance == null) {
            instance = new ActionsManager();
        }
        return instance;
    }
    
    public Action[] findAllActions() {
        HashMap<Class<? extends Action>, Action> tactions=new HashMap(64);
        tactions.putAll(this.actions);
        for (NodeAction[] act : actionsType.values()) {
            for (int i = 0; i < act.length; i++) {
                if (act[i] != null && tactions.get(act[i].getClass()) == null) {
                    tactions.put(act[i].getClass(), act[i]);
                }
            }
        }
        Action[] act=PopupSupport.getDefault().findAllActions();
        for (int i = 0; i < act.length; i++) {
            if (tactions.get(act[i].getClass()) == null) {
                tactions.put(act[i].getClass(), act[i]);
            }
        }
        return tactions.values().toArray(new Action[tactions.size()]);
    }
    
    public void registerAction(JFrame frame, Action action) {
        if (action.getKeyStroke() != null) {
            frame.getRootPane().registerKeyboardAction(new ActionInvoker(action),
                    action.getKeyStroke(), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
            actions.put(action.getClass(), action);
        }
    }
    
    public void registerActions(Class<? extends Node> node, NodeAction... actions) {
        if (actions != null)
            actionsType.put(node, actions);
    }
    
    public NodeAction[] getActions(Node[] nodes) {
        Class cl=nodes[0].getClass();
        for (int i = 1; i < nodes.length; i++) {
            if (nodes[i].getClass() != cl) {
                if (nodes[i].getClass().isAssignableFrom(cl)) {
                    cl=nodes[i].getClass();
                } else if (!cl.isAssignableFrom(nodes[i].getClass())) {
                    return null;
                }
            }
        }
        return actionsType.get(cl);
    }
    
    static class ActionInvoker implements ActionListener {
        
        Action action;
        
        public ActionInvoker(Action action) {
            this.action=action;
        }
        
        public void actionPerformed(ActionEvent e) {
            if (action instanceof StandaloneAction) {
                ((StandaloneAction)action).perform();
            } else if (action instanceof NodeAction) {
                Node[] sels=Application.getInstance().getUi().getSelectedNodes();
                if (sels != null) {
                    ((NodeAction)action).perform(sels);
                }
            }
        }
    }
}
