/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.edits;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.text.JTextComponent;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.utils.Timer;

/**
 *
 * @author  ehucka
 */
public class CompletionDialog extends javax.swing.JDialog implements WindowListener, ComponentListener {
    
    public static final int PREFERED_WIDTH = 100;
    
    static CompletionDialog instance = null;
    
    JTextComponent invoked;
    CompletionListModel model;
    Runnable visiblingRunnable;
    
    String context="";
    StringBuilder selectionContext;
    boolean selected=false;
    private boolean visibling = false;
    boolean ignoring = false;
    
    boolean debug = false;
    
    public static CompletionDialog getInstance() {
        if (instance == null) {
            instance=new CompletionDialog();
        }
        return instance;
    }
    
    /** Creates new form CompletionDialog */
    public CompletionDialog() {
        super(Application.getInstance().getUi(), false);
        initComponents();
        model=(CompletionListModel)list.getModel();
        selectionContext=new StringBuilder();
        Application.getInstance().getUi().addWindowListener(this);
        Application.getInstance().getUi().addComponentListener(this);
        visiblingRunnable=new Runnable() {
            public void run() {
                setVisibling(false);
            }
        };
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jScrollPane1 = new javax.swing.JScrollPane();
        list = new javax.swing.JList();

        setAlwaysOnTop(true);
        setFocusable(false);
        setFocusableWindowState(false);
        setResizable(false);
        setUndecorated(true);
        list.setForeground(new java.awt.Color(0, 0, 204));
        list.setModel(new CompletionListModel());
        list.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        list.setCellRenderer(new CompletionRenderer());
        jScrollPane1.setViewportView(list);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    public void show(JTextComponent invokedComponent) {
        invoked=invokedComponent;
        move();
        if (selectionContext.length() > 0) {
            selectionContext.delete(0, selectionContext.length());
        }
        if (invoked.getSelectedText() != null)
            selected=true;
        else
            selected=false;
        generateContent();
        setVisibling(true);
        setVisible(true);
        Timer.getInstance().delayAfter(new Runnable() {
            public void run() {
                SwingUtilities.invokeLater(visiblingRunnable);
            }
        }, 333);
    }
    
    public void addInSelectionChar(char ch) {
        selectionContext.append(ch);
        generateContent();
    }
    
    public void removeInSelectionChar() {
        if (selectionContext.length() > 0)
            selectionContext.deleteCharAt(selectionContext.length()-1);
        generateContent();
    }
    
    protected void generateContent() {
        try {
            reparseContext();
            ArrayList<FormatTag> tags;
            tags=FormatSupport.getInstance().getTagForContext(context);
            if (tags.size() == 0) {
                context="";
                tags=FormatSupport.getInstance().getTagForContext(context);
            }
            model.setData(tags);
            list.revalidate();
            if (tags.size() > 0) {
                list.setSelectedIndex(0);
                list.ensureIndexIsVisible(0);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    protected void reparseContext() throws Exception {
        if (!selected) {
            context=((HTMLEditor)invoked).getContext(true);
        } else {
            context=selectionContext.toString();
        }
        context=context.toLowerCase();
    }
    
    public void update() {
        if (isVisible()) {
            generateContent();
            move();
            //revalidate();
            repaint();
        }
    }
    
    protected void move() {
        Point loc=invoked.getCaret().getMagicCaretPosition();
        Point p;
        if (loc == null)
            p= new Point(0, 0);
        else
            p= new Point(loc.x, loc.y);
        SwingUtilities.convertPointToScreen(p, invoked);
        setPreferredSize(new Dimension(PREFERED_WIDTH,100));
        setLocation(p.x,p.y-getHeight());
    }
    
    public void close() {
        setVisible(false);
        if (invoked != null)
            invoked.grabFocus();
    }
    
    public void cancel() {
        close();
    }
    
    public void insert() {
        try {
            FormatTag tag=(FormatTag)(list.getSelectedValue());
            int pos=tag.insertInto(invoked.getDocument(), context, invoked.getCaretPosition(),
                    invoked.getSelectionStart(), invoked.getSelectionEnd());
            invoked.setCaretPosition(pos);
            invoked.setSelectionStart(pos);
            invoked.setSelectionEnd(pos);
            setVisible(false);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void completionDown() {
        int in=list.getSelectedIndex()+1;
        if (in >= list.getModel().getSize())
            in=list.getModel().getSize()-1;
        list.setSelectedIndex(in);
        list.ensureIndexIsVisible(in);
        list.repaint();
    }
    
    public void completionUp() {
        int in=list.getSelectedIndex()-1;
        if (in < 0)
            in=0;
        if (in >= list.getModel().getSize())
            in=list.getModel().getSize()-1;
        list.setSelectedIndex(in);
        list.ensureIndexIsVisible(in);
    }
    
    public void completionPgDown() {
        int in=list.getSelectedIndex()+list.getVisibleRowCount();
        if (in >= list.getModel().getSize())
            in=list.getModel().getSize()-1;
        list.setSelectedIndex(in);
        list.ensureIndexIsVisible(in);
    }
    
    public void completionPgUp() {
        int in=list.getSelectedIndex()-list.getVisibleRowCount();
        if (in < 0)
            in=0;
        if (in >= list.getModel().getSize())
            in=list.getModel().getSize()-1;
        list.setSelectedIndex(in);
        list.ensureIndexIsVisible(in);
    }
    
    protected void processComponentKeyEvent(KeyEvent e) {
        int id = e.getID();
        
        switch(id) {
            case KeyEvent.KEY_TYPED:
                //System.err.println("typed "+e.getKeyCode()+" "+e.getModifiers());
                if (isVisible() && invoked.getSelectedText() != null) {
                    e.consume();
                }
                break;
            case KeyEvent.KEY_PRESSED:
                ignoring=false;
                //System.err.println("presed "+e.getKeyCode()+" "+e.getModifiers());
                if (isVisible()) {
                    if (e.getKeyCode() == KeyEvent.VK_UP) {
                        ignoring=true;
                        completionUp();
                        e.consume();
                    } else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
                        ignoring=true;
                        completionDown();
                        e.consume();
                    } else if (e.getKeyCode() == KeyEvent.VK_PAGE_UP) {
                        ignoring=true;
                        completionPgUp();
                        e.consume();
                    } else if (e.getKeyCode() == KeyEvent.VK_PAGE_DOWN) {
                        ignoring=true;
                        completionPgDown();
                        e.consume();
                    } else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
                        ignoring=true;
                        close();
                        e.consume();
                    } else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                        ignoring=true;
                        insert();
                        e.consume();
                    } else if (invoked.getSelectedText() != null) {
                        e.consume();
                    }
                }
                break;
            case KeyEvent.KEY_RELEASED:
                //System.err.println("released "+e.getKeyCode()+" "+e.getModifiers());
                if (isVisible() && invoked.getSelectedText() != null) {
                    char c=e.getKeyChar();
                    if (Character.isLetterOrDigit(c))
                        addInSelectionChar(c);
                    if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE)
                        removeInSelectionChar();
                    e.consume();
                }
                if (isVisible() && !ignoring) {
                    update();
                }
                break;
        }
    }
    
    public void caretUpdate(CaretEvent e) {
        update();
    }
    
    public void windowOpened(java.awt.event.WindowEvent e) {
    }
    
    public void windowIconified(java.awt.event.WindowEvent e) {
        if (isVisible())
            setVisible(false);
    }
    
    public void windowDeiconified(java.awt.event.WindowEvent e) {
    }
    
    public void windowDeactivated(java.awt.event.WindowEvent e) {
        //setVisible(false);  //windows bad behaviour
    }
    
    public void windowClosing(java.awt.event.WindowEvent e) {
        if (isVisible())
            setVisible(false);
    }
    
    public void windowClosed(java.awt.event.WindowEvent e) {
    }
    
    public void windowActivated(java.awt.event.WindowEvent e) {
    }
    
    public void componentShown(java.awt.event.ComponentEvent e) {
    }
    
    public void componentResized(java.awt.event.ComponentEvent e) {
        setVisible(false);
    }
    
    public void componentMoved(java.awt.event.ComponentEvent e) {
        if (isVisible())
            setVisible(false);
    }
    
    public void componentHidden(java.awt.event.ComponentEvent e) {
        if (isVisible())
            setVisible(false);
    }
    
    public boolean isVisibling() {
        return visibling;
    }
    
    public void setVisibling(boolean value) {
        if (debug) System.out.println("set visibling "+value);
        this.visibling = value;
    }
    
    public void setVisible(boolean b) {
        if (debug) System.out.println("try to set visible "+b);
        if (!b && visibling) {
            return;
        }
        if (debug) System.out.println("set visible "+b);
        super.setVisible(b);
    }
    
    public JTextComponent getInvoked() {
        return invoked;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JList list;
    // End of variables declaration//GEN-END:variables
    
}
