/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree;


import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import javax.swing.Icon;
import javax.swing.event.TreeModelEvent;
import javax.swing.tree.TreePath;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.beans.TestAction;
import org.netbeans.testspeceditor.beans.TestCase;
import org.netbeans.testspeceditor.beans.TestResult;
import org.netbeans.testspeceditor.beans.TestSuite;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.ui.NodesClipboard;
import org.netbeans.testspeceditor.ui.actions.ActionAnnotation;
import org.netbeans.testspeceditor.ui.actions.ActionsManager;
import org.netbeans.testspeceditor.ui.actions.CopyAction;
import org.netbeans.testspeceditor.ui.actions.CopyAsTextAction;
import org.netbeans.testspeceditor.ui.actions.CutAction;
import org.netbeans.testspeceditor.ui.actions.DeleteAction;
import org.netbeans.testspeceditor.ui.actions.MoveDownAction;
import org.netbeans.testspeceditor.ui.actions.MoveUpAction;
import org.netbeans.testspeceditor.ui.actions.NewAction;
import org.netbeans.testspeceditor.ui.actions.NewTestActionAction;
import org.netbeans.testspeceditor.ui.actions.NewThreeActionsAction;
import org.netbeans.testspeceditor.ui.actions.PasteAction;
import org.netbeans.testspeceditor.ui.actions.SetLatestAction;
import org.netbeans.testspeceditor.utils.Utils;

/**
 *
 * @author ehucka
 */
public class TestCaseNode<C extends TestCase> extends DataNode<C> {
    
    public static Icon icon;
    
    static {
        icon = new javax.swing.ImageIcon(TestCaseNode.class.getResource("/org/netbeans/testspeceditor/ui/icons/case.png"));
        ActionsManager.getInstance().registerActions(TestCaseNode.class,
                new NewAction(new NewTestActionAction(), new NewThreeActionsAction()), null,
                new SetLatestAction(), null,
                new MoveUpAction(), new MoveDownAction(), null,
                new CopyAction(), new CopyAsTextAction(), new CutAction(), new PasteAction(), new DeleteAction());
    }
    
    public TestCaseNode(Node root, C data) {
        super(data, root, true);
        init();
        getTestCase().getBeanSupport().addBeanChangeListener(this);
    }
    
    private void init() {
        TestAction[] actions=getData().getActions();
        for (TestAction ta:actions) {
            if (!(ta instanceof TestResult)) {
                TestActionNode<TestAction> node=new TestActionNode(this, ta);
                addChild(node);
            }
        }
        TestResultNode<TestResult> node=new TestResultNode(this, getTestCase().getResult());
        addChild(node);
    }
    
    public TestCase getTestCase() {
        return getData();
    }
    
    public String toString() {
        return getTestCase().getTitle();
    }
    
    @ActionAnnotation(actionClass=CopyAsTextAction.class)
    public void copyAsText() {
        StringBuilder sb = new StringBuilder();
        TestCase tc = getTestCase();
        sb.append("Setup: ");
        sb.append(Utils.clearTags(tc.getSetup()));
        sb.append("\n\n");
        TestAction[] acts = tc.getActions();
        for (int i=0;i < acts.length-1;i++) {
            sb.append(i+1);
            sb.append(". ");
            sb.append(Utils.clearTags(acts[i].getContent()));
            sb.append("\n");
        }
        sb.append("\nExpected Result: ");
        sb.append(Utils.clearTags(acts[acts.length-1].getContent()));
        sb.append("\n");
        StringSelection ss = new StringSelection(sb.toString());
        Toolkit.getDefaultToolkit().getSystemClipboard().setContents(ss, null);
    }
    
    @ActionAnnotation(actionClass=DeleteAction.class)
    public void delete() {
        if (getParent() instanceof CasesNode) {
            ((TestSpecNode)getParent().getParent()).getTestSpec().removeTestCase(getTestCase());
        } else if (getParent() instanceof TestSuiteNode) {
            ((TestSuiteNode)getParent()).getTestSuite().removeCase(getTestCase());
        }
    }
    
    @ActionAnnotation(actionClass=PasteAction.class)
    public boolean canPaste() {
        Node[] nds=NodesClipboard.getInstance().content();
        for (Node n:nds) {
            if (n.getClass() != TestActionNode.class) {
                return false;
            }
        }
        return true;
    }
    
    @ActionAnnotation(actionClass=CopyAction.class)
    public void copyTo(Node target) {
        if (target instanceof CasesNode) {
            TestSpecNode ts=(TestSpecNode)target.getParent();
            ts.getTestSpec().addTestCase((TestCase)(getTestCase().copy()));
        } else if (target instanceof TestSuiteNode) {
            TestSuite suit=((TestSuiteNode)target).getTestSuite();
            TestCase cop=(TestCase)(getTestCase().copy());
            Application.getInstance().getDocument().getTestSpec().addTestCase(cop);
            ((TestSuiteNode)target).getTestSuite().addCase(cop);
        }
    }
    
    @ActionAnnotation(actionClass=CutAction.class)
    public void cutTo(Node target) {
        if (target instanceof CasesNode) {
            TestSpecNode tsn=(TestSpecNode)target.getParent();
            TestCase tc=getTestCase();
            tsn.getTestSpec().removeTestCase(tc);
            tsn.getTestSpec().addTestCase(tc);
        } else if (target instanceof TestSuiteNode) {
            if (getParent() instanceof TestSuiteNode) {
                ((TestSuiteNode)getParent()).getTestSuite().removeCase(getTestCase());
            }
            ((TestSuiteNode)target).getTestSuite().addCase(getTestCase());
        }
    }
    
    @ActionAnnotation(actionClass=MoveUpAction.class)
    public void moveUp() {
        if (getParent() instanceof TestSuiteNode) {
            TestSuiteNode par=((TestSuiteNode)getParent());
            par.getTestSuite().moveUp(getTestCase());
        } else { //test spec
            ((TestSpecNode)getRoot()).getTestSpec().moveUp(getTestCase());
        }
    }
    
    @ActionAnnotation(actionClass=MoveDownAction.class)
    public void moveDown() {
        if (getParent() instanceof TestSuiteNode) {
            TestSuiteNode par=((TestSuiteNode)getParent());
            par.getTestSuite().moveDown(getTestCase());
        } else { //test spec
            ((TestSpecNode)getRoot()).getTestSpec().moveDown(getTestCase());
        }
    }
    
    public void childrenChanged(Bean source, Bean child, int oldIndex, int newIndex) {
        if (source == getTestCase() && child instanceof TestAction) {
            if (oldIndex == -1) { //new
                //new Exception().printStackTrace();
                TestActionNode<TestAction> node=new TestActionNode(this, (TestAction)child);
                addChild(node, newIndex);
                treeNodesInserted(new TreeModelEvent(getTreeModel(), getTreePath(), new int[]{newIndex}, new Object[] {node}));
            } else if (newIndex == -1) { //old
                TestActionNode<TestAction> node=(TestActionNode)(getChildAt(oldIndex));
                if (node != null && node.getTestAction() == child) {
                    int inx=remove(node);
                    node.release();
                    treeNodesRemoved(new TreeModelEvent(getTreeModel(), getTreePath(), new int[]{inx}, new Object[] {node}));
                } else {
                    System.err.println("undetected node");
                }
            } else {  //moved
                Node n=(Node)(children.remove(oldIndex));
                children.add(newIndex, n);
                treeStructureChanged(new TreeModelEvent(getTreeModel(), getTreePath(), new int[]{newIndex}, new Object[] {n}));
            }
        }
    }
    
    public void propertyChanged(Bean source, String propertyName, Object oldValue, Object newValue) {
        if (propertyName.equals("title")) {
            //getTreeModel().nodeChanged(this);
            treeNodesChanged(getNodeEvent());
        }
    }
    
    public Icon getIcon() {
        return icon;
    }
    
    public TreePath findNode(Bean bean) {
        for (int i = 0; i < getChildCount(); i++) {
            if (((DataNode)getChildAt(i)).getData() == bean) {
                return ((DataNode)getChildAt(i)).getTreePath();
            }
        }
        return null;
    }
    
    public void release() {
        getTestCase().getBeanSupport().removeBeanChangeListener(this);
    }
}
