/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlparser;

import java.util.ArrayList;
import java.util.HashMap;

/**
 *
 * @author  ehucka
 */
public class Token {
    
    char[] buf;
    int index;
    String[] tagNames = null;
    
    
    public Token() {
        buf=new char[64];
        index=0;
    }
    
    public Token(String s) {
        buf=s.toCharArray();
        index=buf.length;
    }
    
    public void add(char c) {
        buf[index++]=c;
        if (index == buf.length) {
            char[] ar=buf;
            buf=new char[buf.length*2];
            System.arraycopy(ar, 0, buf, 0, ar.length);
        }
    }
    
    public void complete() {
        char[] ar=buf;
        buf=new char[index];
        System.arraycopy(ar, 0, buf, 0, buf.length);
    }
    
    public boolean isTag() {
        if (index < 2) return false;
        return (buf[0] == '<' && buf[buf.length-1] == '>');
    }
    
    public boolean isComment() {
        if (index < 4) return false;
        return (buf[0] == '<' && buf[1] == '!' && buf[2] == '-' && buf[3] == '-');
    }
    
    public boolean endComment() {
        if (index < 7) return false;
        return (buf[index-1] == '>' && buf[index-2] == '-' && buf[index-3] == '-');
    }
    
    public boolean isFinishTag() {
        if (buf.length < 2) return false;
        return (buf[0] == '<' && buf[1] == '/');
    }
    
    public boolean isSingleTag() {
        if (buf.length < 4) return false;
        return (buf[buf.length-1] == '>' && buf[buf.length-2] == '/');
    }
    
    public boolean isSpecial() {
        if (buf.length < 2) return false;
        return !isFinishTag() && !isComment() && !isCDATA() && (buf[0] == '<' && !Character.isLetter(buf[1]));
    }
    
    public boolean isCDATA() {
        if (index < 9) return false;
        return (buf[0] == '<' && buf[1] == '!' && buf[2] == '[' && buf[3] == 'C' &&
                buf[4] == 'D' && buf[5] == 'A' && buf[6] == 'T' && buf[7] == 'A' && buf[8] == '[');
    }
    
    public boolean endCDATA() {
        if (index < 12) return false;
        return (buf[index-3] == ']' && buf[index-2] == ']' && buf[index-1] == '>');
    }
    
    public String decodeCDATA() {
        int s,e;
        s=CDATAValue.START_TAG.length();
        e=buf.length-CDATAValue.END_TAG.length()-1;
        
        /*for (int i=CDATAValue.START_TAG.length();i < buf.length;i++) {
            s=i;
            if (!isWhiteSpace(buf[i])) {
                if (buf[i] == '\n') {
                    s=i+1;
                }
                break;
            }
        }
        for (int i=buf.length-CDATAValue.END_TAG.length()-1;i >= s;i--) {
            e=i;
            if (!isWhiteSpace(buf[i])) {
                if (buf[i] == '\n') {
                    e=i-1;
                }
                break;
            }
        }*/
        if ((s > 0 || e < buf.length) && s < e) {
            //char[] old=buf;
            char[] nbuf=new char[e-s+1];
            System.arraycopy(buf, s, nbuf, 0, e-s+1);
            return new String(nbuf);
        }
        return "";
    }
    
    public String decodeComment() {
        int s,e;
        s=4;
        e=buf.length-4;
        /*for (int i=4;i < buf.length;i++) {
            s=i;
            if (!isWhiteSpace(buf[i])) {
                break;
            }
        }
        for (int i=buf.length-4;i >= s;i--) {
            e=i;
            if (!isWhiteSpace(buf[i])) {
                break;
            }
        }*/
        if ((s > 0 || e < buf.length) && s < e) {
            char[] nbuf=new char[e-s+1];
            System.arraycopy(buf, s, nbuf, 0, e-s+1);
            return new String(nbuf);
        }
        return "";
    }
    
    public void trim() {
        int s,e;
        s=0;
        e=buf.length;
        for (int i=0;i < buf.length;i++) {
            s=i;
            if (!isWhiteSpace(buf[i])) {
                break;
            }
        }
        for (int i=buf.length-1;i >= s;i--) {
            e=i;
            if (!isWhiteSpace(buf[i])) {
                break;
            }
        }
        if ((s > 0 || e < buf.length) && s <= e) {
            if (e > s) {
                char[] old=buf;
                buf=new char[e-s+1];
                System.arraycopy(old, s, buf, 0, buf.length);
            } else {
                buf=new char[0];
            }
            index=buf.length;
        }
    }
    
    public String toString() {
        return new String(buf, 0, index);
    }
    
    public int length() {
        return buf.length;
    }
    
    public char[] copy() {
        char[] ret=new char[buf.length];
        System.arraycopy(buf, 0, ret, 0, ret.length);
        return ret;
    }
    
    public String[] getTagName() {
        if (tagNames != null) return tagNames;
        int s,e, s2,e1;
        s=0;
        s2=0;
        e=0;
        e1=0;
        for (int i=0;i < buf.length;i++) {
            if (s == 0 && buf[i] != '<' && buf[i] != '/') {
                if (!Character.isJavaIdentifierStart(buf[i])) return null;
                s=i;
            } else if (s > 0) {
                if ((isWhiteSpace(buf[i]) || buf[i] == '/' || buf[i] == '>')) {
                    e=i;
                    break;
                } else if (buf[i]==':') { //namespace
                    if (e1 > 0) return null;
                    e1=i;
                    s2=i+1;
                }
            }
        }
        String[] retr=null;
        if (e-s > 0) {
            if (s2 > 0) {
                char[] ret=new char[e1-s];
                System.arraycopy(buf, s, ret, 0, e1-s);
                String ns=new String(ret);
                ret=new char[e-s2];
                System.arraycopy(buf, s2, ret, 0, e-s2);
                String name=new String(ret);
                retr=new String[] {name, ns};
            } else {
                char[] ret=new char[e-s];
                System.arraycopy(buf, s, ret, 0, e-s);
                retr=new String[] {new String(ret)};
            }
        }
        tagNames=retr;
        return retr;
    }
    
    public void parseAttributes(Tag tag) throws ParserException {
        HashMap<String, String> map=new HashMap(8);
        ArrayList<String> names = new ArrayList(8);
        int start=0, max =0;
        for (int i=0;i < buf.length;i++) {
            if (isWhiteSpace(buf[i])) {//Character.isWhitespace(buf[i])) {
                start=i;
                break;
            }
            max=i;
        }
        if (max >= buf.length-1) return; //without attributes
        char boundsChar='"';
        int sn,en,sv,ev;
        sn=0;
        en=0;
        sv=0;
        ev=0;
        for (int i=start;i < buf.length;i++) {
            if (sn == 0 && (Character.isJavaIdentifierStart(buf[i]) || Character.isDigit(buf[i]))) {  //start of attr name
                sn=i;
            } else if (en == 0 && sn > 0 && (isWhiteSpace(buf[i]) || buf[i] == '=')) {//end of attr name
                en=i;
            } else if (en > 0 && sv == 0 && (buf[i] == '"' || buf[i] == '\'')) {
                sv=i+1;
                boundsChar=buf[i];
            } else if (sv > 0 && ev == 0 && buf[i] == boundsChar) {
                ev=i;
                String name=new String(buf, sn, en-sn);
                String value=new String(buf, sv, ev-sv);
                map.put(name, value);
                names.add(name);
                sn=0;
                en=0;
                sv=0;
                ev=0;
            }
        }
        if (sn > 0 && en == 0 || sv > 0 && ev == 0 || en > 0 && sv == 0) 
            throw new ParserException("wrong format of attributes: "+new String(buf)+" "+sn+" "+en+" "+sv+" "+ev);
        tag.setAttributes(map, names);
    }
    
    public static boolean isWhiteSpace(char c) {
        return (c == ' ' || c == '\t' || c == '\n' || c == '\r' || c < 0x20);
    }
}
