/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted.actions;

import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.netbeans.modules.regsup.auted.*;
import java.awt.Point;
import org.netbeans.modules.regsup.logic.RexEdgeElement;

/** Action that can always be invoked and work procedurally.
 *
 * @author  Petr Slegr
 */
// !!! pouzit compound undo
public class AddConnectedNodeBelowAction extends ElementAction implements AddAction {
    
    public void performAction (VisualAutomatonElement[] selected) {
        // version where current container (open) is treated as selected element
        /*
        VisualAutomatonContainer container = (VisualAutomatonContainer) selected [0];
        AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
        container.createNode (editor.getCurrentPosition ());
         */
        VisualAutomatonNode source = (VisualAutomatonNode) selected[0];
        AutomatonEditorPanel editorPanel = MainManager.getDefault ().getCurrentPanel ();
        if (editorPanel != null) {
            VisualAutomatonContainer container = editorPanel.getCurrentView ().getContainer ();
            AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
            PointPlus distance = new PointPlus (0, 70);
            VisualAutomatonNode target = container.createNode (distance.add (source.getPosition ()));
            //
            Point current = editor.getCurrentPosition ();
            Point dest = new PointPlus (40, 40).add (current);
            RexEdgeElement value = UserSupport.letUserEditEdgeValue (null, dest);
            if (value != null) {
                container.createEdge (source, target, value);
            } else {
                // rollback
                container.removeElement (target);
            }
        }
    }
    
    public String getName () {
        //return NbBundle.getMessage (AddNodeAction.class, "CTL_ADD_NODE_ACTION");
        return "Add Node Below (Connected)";
    }
    
    protected String iconResource () {
        return "ActionIcon.gif";
    }
    
    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (AddNodeActionAction.class);
    }
    
    /** Test whether the action is currently enabled.
     * @return <code>true</code> if so
     */
    public boolean isEnabled (VisualAutomatonElement[] selected) {
        if (selected.length != 1) {
            return false;
        }
        if (! (selected[0] instanceof VisualAutomatonNode)) {
            return false;
        }
        // !!! testovat umistitelnost
        return true;
    }
    
    /** Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
     * protected void initialize () {
     * super.initialize ();
     * putProperty (Action.SHORT_DESCRIPTION, NbBundle.getMessage (AddNodeActionAction.class, "HINT_Action"));
     * }
     */
    
}
