/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.ide;

import org.openide.util.NbBundle;
import org.openide.windows.*;
import org.netbeans.modules.regsup.auted.*;
import java.awt.BorderLayout;
import org.openide.TopManager;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.Color;
import java.io.*;

// As needed:
/*
import java.awt.Toolkit;
import java.io.*;
import java.net.*;
import org.openide.TopManager;
import org.openide.actions.*;
import org.openide.awt.UndoRedo;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.actions.*;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.io.*;
 */

/** An openable window available to the IDE's window manager.
 *
 * @author  Petr Slegr
 * @version 01.05.02
 */
public class RegsupWindow extends TopComponent /* or CloneableTopComponent */
implements AutomatonEditorPanelOwner {
    
    // REMEMBER: You should have a public default constructor!
    // This is for externalization. If you have a nondefault
    // constructor for normal creation of the component, leave
    // in a default constructor that will put the component into
    // a consistent but unconfigured state, and make sure readExternal
    // initializes it properly. Or, be creative with writeReplace().
    
    private AutomatonEditorPanel panel;

    public RegsupWindow () {
        initComponents ();
        setCloseOperation (CLOSE_LAST); // or CLOSE_EACH
        // Display name of this window (not needed if you use the DataObject constructor):
        setName (NbBundle.getMessage (RegsupWindow.class, "LBL_component_name"));
        // You may set the icon, but often it is better to set the icon for an associated mode instead:
        // setIcon (Toolkit.getDefaultToolkit ().getImage (new URL ("nbresloc:/org/netbeans/modules/regsup/ide/RegsupWindowIcon.gif")));
        // Use the Component Inspector to set tool-tip text. This will be saved
        // automatically. Other JComponent properties you may need to save yuorself.
        // At any time you can affect the node selection:
        // setActivatedNodes (new Node[] { ... });
    }
    
    public RegsupWindow (RegsupDataObject dataObject, AutomatonEditor rootEditor) {
        this ();
        //EditorWindow win = new EditorWindow (rootEditor);
        panel = new AutomatonEditorPanel (rootEditor);
        add (panel, BorderLayout.CENTER);
        //setName (panel.getTitle ());
        setName (dataObject.getPrimaryFile ().getName ());
        MainManager.getDefault ().registerAutomatonEditorPanelOwner (this);
    }
    
    /*
    public HelpCtx getHelpCtx () {
        return new HelpCtx (RegsupWindow.class);
    }
     */
    
    /*
    // If you are using CloneableTopComponent, probably you should override:
    protected CloneableTopComponent createClonedObject () {
        return new RegsupWindow ();
    }
    protected boolean closeLast () {
        // You might want to prompt the user first and maybe return false:
        return true;
    }
     */
    
    // APPEARANCE
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents

        setLayout(new java.awt.BorderLayout());

    }//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
    // MODES AND WORKSPACES
    
    // If you want it to open in a specific mode:
    public static final String RegsupWindow_MODE = "editor";
    
    /** Utility field used by event firing mechanism.  */
    private javax.swing.event.EventListenerList listenerList = null;
    
    public void open (Workspace ws) {
        super.open (ws);
        if (ws == null) ws = TopManager.getDefault ().getWindowManager ().getCurrentWorkspace ();
        Mode m = ws.findMode (RegsupWindow_MODE);
        if (m == null) {
            try {
                m = ws.createMode (RegsupWindow_MODE, // code name
                                   NbBundle.getMessage (RegsupWindow.class, "LBL_mode_name"), // display name
                                   new URL ("nbresloc:/org/netbeans/modules/regsup/ide/RegsupWindowIcon.gif"));
            } catch (MalformedURLException mfue) {
                TopManager.getDefault ().notifyException (mfue);
                return;
            }
            // If you want it in a specific position:
            // m.setBounds (...ws.getBounds ()...);
        }
        m.dockInto (this);
    }
    
    protected void componentActivated () {
        super.componentActivated ();
        fireAutomatonEditorPanelActivated ();
        //System.out.println("!!!!ACTIVATED");
    }
    
    protected void componentDeactivated () {
        super.componentDeactivated ();
        fireAutomatonEditorPanelDeactivated ();
        //System.out.println("!!!!DEACTIVATED");
    }
    
    /** Registers AutomatonEditorPanelListener to receive events.
     * @param listener The listener to register.
     */
    public synchronized void addAutomatonEditorPanelListener (AutomatonEditorPanelListener listener) {
        if (listenerList == null ) {
            listenerList = new javax.swing.event.EventListenerList ();
        }
        listenerList.add (AutomatonEditorPanelListener.class, listener);
    }
    
    /** Removes AutomatonEditorPanelListener from the list of listeners.
     * @param listener The listener to remove.
     */
    public synchronized void removeAutomatonEditorPanelListener (AutomatonEditorPanelListener listener) {
        listenerList.remove (AutomatonEditorPanelListener.class, listener);
    }
    
    /** Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireAutomatonEditorPanelDeactivated () {
        if (listenerList == null) return;
        Object[] listeners = listenerList.getListenerList ();
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i]==AutomatonEditorPanelListener.class) {
                ((AutomatonEditorPanelListener)listeners[i+1]).deactivated (panel);
            }
        }
    }
    
    /** Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireAutomatonEditorPanelActivated () {
        if (listenerList == null) return;
        Object[] listeners = listenerList.getListenerList ();
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i]==AutomatonEditorPanelListener.class) {
                ((AutomatonEditorPanelListener)listeners[i+1]).activated (panel);
            }
        }
    }
    
    /*
    // If you are not specifying a mode you may wish to use:
    public Dimension getPreferredSize () {
        return ...TopManager.getDefault ().getWindowManager ().getCurrentWorkspace ().getBounds ()...;
    }
     */
    
    /*
    // If you want it to open on a specific workspace:
    public static final String RegsupWindow_WORKSPACE = NbBundle.getMessage (RegsupWindow.class, "LBL_workspace_name");
    public void open () {
        WindowManager wm = TopManager.getDefault ().getWindowManager ();
        Workspace ws = wm.findWorkspace (RegsupWindow_WORKSPACE);
        if (ws == null)
            ws = wm.createWorkspace (RegsupWindow_WORKSPACE);
        open (ws);
        ws.activate ();
    }
     */
    
    // PERSISTENCE
    
    // private static final long serialVersionUID = ...;
    /*
    // If you wish to keep any state between IDE restarts, put it here:
    public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal (in);
        //setSomeState ((SomeType) in.readObject ());
    }
    public void writeExternal (ObjectOutput out) throws IOException {
        super.writeExternal (out);
        //out.writeObject (getSomeState ());
    }
    */
    /*
    // The above assumes that the SomeType is safely serializable, e.g. String or Date.
    // If it is some class of your own that might change incompatibly, use e.g.:
    public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal (in);
        NbMarshalledObject read = (NbMarshalledObject) in.readObject ();
        if (read != null) {
            try {
                setSomeState ((SomeType) read.get ());
            } catch (Exception e) {
                TopManager.getDefault ().getErrorManager ().notify (e);
                // If the problem would make this component inconsistent, use:
                // throw new SafeException (e);
            }
        }
    }
    public void writeExternal (ObjectOutput out) throws IOException {
        super.writeExternal (out);
        Object toWrite;
        try {
            toWrite = new NbMarshalledObject (getSomeState ());
        } catch (Exception e) {
            TopManager.getDefault ().getErrorManager ().notify (e);
            toWrite = null;
            // Again you may prefer to use:
            // throw new SafeException (e);
        }
        out.writeObject (toWrite);
    }
     */
    
     /*
    // Use this to discard the component after restarts (make it nonpersistent):
    private Object readResolve () throws ObjectStreamException {
        return null;
        // If you wish to conditionally discard it, make readExternal set
        // or clear some flag acc. to the condition, then use:
        // return discardFlag ? null : this;
        // Singleton component using private static RegsupWindow theInstance:
        // if (theInstance == null) theInstance = this;
        // return theInstance;
    }
      */
    
    // ACTIONS
    
    /*
    // If you wish to have extra actions appear in the window's
    // popup menu, they can go here:
    public SystemAction[] getSystemActions () {
        SystemAction[] supe = super.getSystemActions ();
        SystemAction[] mine = new SystemAction[supe.length + 1];
        System.arraycopy (supe, 0, mine, 0, supe.length);
        mine[supe.length] = SystemAction.get (SomeActionOfMine.class);
        return mine;
    }
     */
    
    /*
    // Associate implementations with copying, searching, etc.:
    protected void componentActivated () {
        ((CallbackSystemAction) SystemAction.get (FindAction.class)).setActionPerformer (new ActionPerformer () {
                public void performAction (SystemAction action) {
                    // search this component somehow
                }
            });
        // similar for CopyAction, CutAction, DeleteAction, GotoAction, ReplaceAction, etc.
        // for PasteAction, use:
        // ((PasteAction) SystemAction.get (PasteAction.class)).setPasteTypes (new PasteType[] { ... });
    }
    protected void componentDeactivated () {
        // FindAction will be turned off by itself
        // ((PasteAction) SystemAction.get (PasteAction.class)).setPasteTypes (null);
    }
     */
    
    /*
    // If you want UndoAction and RedoAction to be enabled on this component:
    public UndoRedo getUndoRedo () {
        return new MyUndoRedo (this);
    }
     */
    
    // Printing, saving, compiling, etc.: use cookies on some appropriate node and
    // use this node as the node selection.
    
}
