/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.logic;

import java.util.*;

/** Singleton class.
 *
 * @author  Petr Slegr
 * @version 07.05.02/2
 */
public class RexEmptyElement extends RexEdgeElement {
    
    private Set intervals = new HashSet ();
    
    private static RexEmptyElement instance;
    
    /** Returns instance
     */
    public static RexEmptyElement create () {
        if (instance == null) {
            instance = new RexEmptyElement ();
        }
        return instance;
    }
    
    private RexEmptyElement () {
        intervals.add (ComplexRexEdgeElement.Interval.FULL);
    }
    
    public String toString () {
        return new String ("''");
    }
    
    public Object clone () {
        // nothing to clone
        return this;
    }
    
    public int size () {
        return 0;
    }
    
    /** With empty string, transition is whenever possible
     */
    public boolean includes (char ch) {
        return true;
    }
    
    /** Automaton with empty string edge is always NFA
     */
    public boolean intersects (RexEdgeElement edgeElem) {
        return true;
    }
    
    /** Returns Set of Intervals describing this element
     */
    Set getIntervals () {
        return new HashSet (intervals);
    }
    
    
}
