/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline;

import java.io.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.openide.*;
import org.openide.util.*;

import org.netbeans.modules.vcscore.*;
import org.netbeans.modules.vcscore.util.*;
import org.netbeans.modules.vcs.cmdline.passwd.*;


/** The CVS filesystem customizer. Descends from WizardDescriptor and implements the Customizer interface.
 * It stores all the data collected and finally applies the changes.
 * After completion, mounts the filesystem.
 * @author Milos Kleint
 * @version 1.0
 */
public class NewCvsCustomizer extends WizardDescriptor implements Customizer, PropertyChangeListener {
    private Debug E=new Debug("CvsCustomizer", true); // NOI18N
    private Debug D=new Debug("CvsCustomizer", true); // NOI18N
    private javax.swing.ButtonGroup bg;
    
    private final static String CVS_LOCAL = "local";
    private final static String CVS_SERVER = "server";
    private final static String CVS_PSERVER = "pserver";

    private CvsFileSystem fileSystem=null;
    private PropertyChangeSupport changeSupport=null;
    private volatile boolean isRootNotSetDlg = true;
    private volatile JDialog rootNotSetDlg = null;
    private String lastRootValue = "";
    private String lastServerType;
    private String lastUserName;
    private String lastServerName;
    private String lastRepository;
    private String lastRootDir;
    private String lastEnteredRootDir;
    private String lastModule;
    private String lastRelMount;
    private String lastCvsExe;
    private boolean lastUseUnixShell;
    private String lastCygwinPath;
//    private String lastRefresh;

    private String newRootValue = "";
    private String newServerType;
    private String newUserName;
    private String newServerName;
    private String newRepository;
    private String newRootDir;
    private String newEnteredRootDir;
    private String newModule;
    private String newRelMount;
    private String newCvsExe;
    private boolean newUseUnixShell;
    private String newCygwinPath;
//    private String newRefresh;
    
    public  String rootValue = "";
    public  String serverType;
    public  String userName;
    public  String serverName;
    public  String repository;
    public  String rootDir;
    public  String relMount;
    public  String cvsExe;
    public  boolean useUnixShell;
    public  String cygwinPath;
//    public  String refresh;
    public  String[] modulesSelected;
    private boolean waitForCheckout;
    private boolean isReadyForMount = false;
    
    static final long serialVersionUID =-4163355905662053542L;


    private WizardDescriptor.Iterator iterPanel;   
    
    private EventListenerList listeners;  
    
/** Creates now instance of customizer, creates the iterator and all panels.
 */
    public NewCvsCustomizer() {
        this(new CustomizerWizard());
    }
    
    
    /** Creates new CvsCustomizer */
    private NewCvsCustomizer(CustomizerWizard www) {
        super(www);
        iterPanel = www;
        this.setTitleFormat(new MessageFormat("{1} - {0}"));
        addPropertyChangeListener (this);
    }

     
/** returns the filesystem passed to the customizer via setObject() method
 * @return the currently edited CVS filesystem.
 */
    public CvsFileSystem getFileSystem() {
      return fileSystem;  
    }    

/** Fills the filesystem (data) to the customizer.
 * @param bean the current cvsfilesystem
 */
    public void setObject(final java.lang.Object bean) {
        fileSystem=(CvsFileSystem)bean;
        String st = fileSystem.getCvsServerType ();
        lastServerType = st;
        serverType = lastServerType;

        lastUserName = fileSystem.getCvsUserName ();
        userName = lastUserName;

        lastServerName = fileSystem.getCvsServer ();
        serverName = lastServerName;
        
        lastRepository = fileSystem.getCvsRoot();
        repository = lastRepository;

        lastRelMount = fileSystem.getRelativeMountPoint();
        relMount = lastRelMount;
        lastRootDir = VcsFileSystem.substractRootDir(fileSystem.getRootDirectory().toString(), lastRelMount);
        rootDir = lastRootDir;
        

        lastCvsExe = fileSystem.getCvsExePath ();
        cvsExe =lastCvsExe;

        lastUseUnixShell = fileSystem.isUseUnixShell();
        useUnixShell = lastUseUnixShell;
        lastCygwinPath = fileSystem.getCygwinPath ();
        cygwinPath = lastCygwinPath;

//        lastRefresh = ""+getRefreshInSecs (); // NOI18N
//        refresh = lastRefresh;

        iterPanel.current().readSettings(this); // HACK MK .. 
        this.updateState();
    }

/** Applies all changes done to the Cvs settings during the customizer.
 */
    public void applyChanges() {
        String value;
        value = serverType;
        if (!lastServerType.equals(value)) {
            fileSystem.setCvsServerType(value);
            lastServerType = serverType;
        }
        
        value = userName;
        fileSystem.setCvsUserName (value);
        lastUserName = value;
        value = serverName;
        fileSystem.setCvsServer(value);
        lastServerName = value;
        value = repository;
        fileSystem.setCvsRoot(value);
        lastRepository = value;
        
        value = rootDir;
        if (!lastRootDir.equals(value)) {
            //fileSystem.setRootDirectory(new File(value));
            rootDirChanged(value, true);
            fileSystem.setCvsModule(""); // we need to do that because the working dir changed
            lastRootDir = value;
        }
        
        value = relMount;
        if (!lastRelMount.equals(value)) {
            fileSystem.setCvsModule (value);
            lastRelMount = value;
            rootDirChanged(rootDir, false);
        }

        value = cvsExe;
        if (!lastCvsExe.equals(value)) {
            fileSystem.setCvsExePath (value);
            lastCvsExe = value;
        }
        boolean bvalue = useUnixShell;
        if (bvalue != lastUseUnixShell) {
            fileSystem.setUseUnixShell(bvalue);
            lastUseUnixShell = bvalue;
        }
        value = cygwinPath;
        if (!lastCygwinPath.equals(value)) {
            fileSystem.setCygwinPath(value);
            lastCygwinPath = value;
        }
/*        value = refresh;
        if (!lastRefresh.equals(value)) {
            refreshChanged(value);
            lastRefresh = refresh;
        }
 */
    }

/** Temporarily sets the values to the filesystem to allow the checkout panel to
 * display the current modules.
 */
    public void setChanges() {
        fileSystem.setCvsServerType(serverType);  
        newServerType = serverType;
        fileSystem.setCvsUserName (userName);
        newUserName = userName;
        fileSystem.setCvsServer(serverName);
        newServerName = serverName;
        fileSystem.setCvsRoot(repository);
        newRepository = repository;
//MK        fileSystem.setCvsModule (relMountTextField.getText());
        rootDirChanged(rootDir, true);
        newRootDir = rootDir;
//        fileSystem.setCvsModuleName(customModule.getModuleTextField()); //MK
        fileSystem.setCvsExePath (cvsExe);
        newCvsExe = cvsExe;
        fileSystem.setUseUnixShell(useUnixShell);
        newUseUnixShell = useUnixShell;
        fileSystem.setCygwinPath(cygwinPath);
        newCygwinPath = cygwinPath;
 //       refreshChanged(refresh);
 //       newRefresh = refresh;
    }

/** unsets changes made to the filesystem with the setChanges()
 */
    public void unSetChanges() {
        fileSystem.setCvsServerType (lastServerType);
        serverType = newServerType;
        fileSystem.setCvsUserName (lastUserName);
        userName = newUserName;
        fileSystem.setCvsServer(lastServerName);
        serverName = newServerName;
        fileSystem.setCvsRoot(lastRepository);
        repository = newRepository;
//        fileSystem.setCvsModuleName(lastModule);
//MK        fileSystem.setCvsModule (lastRelMount);
        rootDirChanged(lastRootDir, false);
        rootDir = newRootDir;
        fileSystem.setCvsExePath(lastCvsExe);
        cvsExe = newCvsExe;
        fileSystem.setUseUnixShell(lastUseUnixShell);
        useUnixShell = newUseUnixShell;
        fileSystem.setCygwinPath(lastCygwinPath);
        cygwinPath = newCygwinPath;
   //     refreshChanged(lastRefresh);
   //     refresh = newRefresh;
    }
    /**
     * Change the root directory of the filesystem. Get the directory from the text field
     * and perform the change in the filesystem. If the root directory does not exists and
     * <code>info</code> is <code>true</code>, nothing is changed.
     * @param info whether to inform the user of nonexistent root directory.
     */
    private void rootDirChanged (String selected, boolean info) {
        // root dir set by hand
        //String selected = rootDirTextField.getText ();
        /*
        String module = moduleTextField.getText ();
        String moduleDir = module.equals ("") ? selected : selected + java.io.File.separator + module; // NOI18N
        File dir=new File(moduleDir);
        */
        File root = new File(selected);
        if( info && !root.isDirectory() ){
            //E.err("not directory "+root); // NOI18N
            final String badDir = root.toString();
            javax.swing.SwingUtilities.invokeLater(new Runnable () {
                public void run () {
                    javax.swing.SwingUtilities.invokeLater(new Runnable () {
                        public void run () {
                            if (true /*browseButton.hasFocus()*/) {
                                try {
                                    Thread.sleep(200);
                                } catch(InterruptedException e) {
                                    // Ignoring interruption
                                }
                            }
                            javax.swing.SwingUtilities.invokeLater(new Runnable () {
                                public void run () {
                                    if (isRootNotSetDlg) {
                                        isRootNotSetDlg = false;
//                                        rootNotSetDlg = notifyMessage(MessageFormat.format (org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.notDirectory"), new Object[] { badDir } ));
//                                        rootNotSetDlg.setVisible(true);
//                                        TopManager.getDefault ().notify (new NotifyDescriptor.Message(MessageFormat.format (org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.notDirectory"), new Object[] { badDir } )));
                                        isRootNotSetDlg = true;
                                    }
                                }
                            });
                        }
                    });
                }
            });
            return ;
        }
        try{
            fileSystem.setRootDirectory(root);
            //rootDirTextField.setText(selected);
        }
        catch (PropertyVetoException veto){
            fileSystem.debug(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.canNotChangeWD"));
            //E.err(veto,"setRootDirectory() failed"); // NOI18N
        }
        catch (IOException e){
            E.err(e,"setRootDirectory() failed"); // NOI18N
            final String badDir = root.toString();
            javax.swing.SwingUtilities.invokeLater(new Runnable () {
                                                       public void run () {
                                                           if (isRootNotSetDlg) {
                                                               isRootNotSetDlg = false;
                                                               TopManager.getDefault ().notify (new NotifyDescriptor.Message(MessageFormat.format (org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.cannotSetDirectory"), new Object[] { badDir } )));
                                                               isRootNotSetDlg = true;
                                                           }
                                                       }
                                                   });
        }
    }
/*
    private void refreshChanged (String refresh) {
        try {
            int time = Integer.parseInt(refresh);
            if (time < 0) throw new NumberFormatException(""+time); // NOI18N
            setRefreshInSecs (time);
        } catch (NumberFormatException e) {
            final String msg = e.getMessage();
            E.deb(msg);
            refresh = "" +getRefreshInSecs (); // NOI18N
            javax.swing.SwingUtilities.invokeLater(new Runnable () {
                                                       public void run () {
                                                           TopManager.getDefault ().notify (new NotifyDescriptor.Message("'"+msg+"': "+org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.nonNegative")));
                                                       }
                                                   });
        }
    }
    
    private int getRefreshInSecs() {
      return (fileSystem.getCustomRefreshTime() / 1000);
    }    
    
    private void setRefreshInSecs(int secs) {
      fileSystem.setCustomRefreshTime(secs * 1000);

    }    
 */   

   private void fillTextFields() {
     BufferedReader buff = null;    
     String dirName = rootDir;
     dirName = dirName + (relMount.equals("") ? "" : (File.separator + relMount));
//     if (dirName.equals(this.lastEnteredRootDir)) return; // user left textfield but didn't do any changes
//     lastEnteredRootDir = dirName;
     dirName = dirName + File.separator + "CVS" + File.separator + "Root";
     File dirFile = new File(dirName);
     if (dirFile.exists()) {
        try { 
          buff = new BufferedReader(new InputStreamReader(new FileInputStream(dirFile.getAbsolutePath())));
          String line = buff.readLine();
          if (line.startsWith(":")) {
            line = line.substring(1);
            StringTokenizer token = new StringTokenizer(line, ":",false);
            if (token.hasMoreTokens()) {
              this.serverType = token.nextToken();
              if (serverType.equals(NewCvsCustomizer.CVS_LOCAL))  {
                 this.repository =  line.substring(serverType.length() + 1);
              } else {   //some kind of server..
                if (token.hasMoreTokens()) {
                   String userServer =  token.nextToken();   
                   this.userName = userServer.substring(0,userServer.indexOf('@'));
                   this.serverName = userServer.substring(userServer.indexOf('@') + 1);
                }
                if (token.hasMoreTokens())  this.repository = token.nextToken();
              }  
            }      
          }    
        } catch (IOException exc) { //doesn't matter - nothing will be filled in
        }              
        finally {
          if (buff != null) {
             try { 
               buff.close(); 
             } catch (IOException eIO) {}  
          }    
        }    
     }    
   }
   
    String getCommLine(String s) {
        D.deb("getting "+s);
        return NbBundle.getBundle
               ("org.netbeans.modules.vcs.cmdline.CommandLines").getString (s);
    }
   
   
    String g(String s) {
        D.deb("getting "+s);
        return NbBundle.getBundle
               ("org.netbeans.modules.vcs.cmdline.Bundle").getString (s);
    }
    String  g(String s, Object obj) {
        return MessageFormat.format (g(s), new Object[] { obj });
    }
    String g(String s, Object obj1, Object obj2) {
        return MessageFormat.format (g(s), new Object[] { obj1, obj2 });
    }
    String g(String s, Object obj1, Object obj2, Object obj3) {
        return MessageFormat.format (g(s), new Object[] { obj1, obj2, obj3 });
    }

    private void D(String debug) {
 //       System.out.println("CvsCustomizer(): "+debug);
    }
    
 
    
/** Checks if the finish panel was pressed. If so, does the final part of filesystem setup.
 * Fills in server name, repository root and user name regarding to the info in the CVS/Root file in the currently mounted directory.
 * It does so, even if new modules were checked out and the server panel was filled.
 * @param event the current property chneg event
 */
    public void propertyChange(final PropertyChangeEvent event) {
        if (event.getPropertyName().equals(DialogDescriptor.PROP_VALUE)){
            Object option = event.getNewValue();
            if (option == WizardDescriptor.FINISH_OPTION) {
                iterPanel.current().storeSettings(this);
          //      fillTextFields();
                if (iterPanel.current().getClass() == Cust2ServerPanel.class) {
                  boolean [] loginCancelled = new boolean[1];
                  if (!((Cust2ServerPanel)iterPanel.current()).isLoggedIn(loginCancelled)) {
                      if (!loginCancelled[0]) {
                          fileSystem.setOffLine(true);
                      } else {
                          return;
                      }
                  }
                }
                applyChanges();
                fileSystem.getCacheProvider().refreshDirFromDiskCache(fileSystem.getFile(""));
                org.openide.TopManager.getDefault ().getRepository ().addFileSystem (fileSystem);
            }
               if (iterPanel.current() instanceof Cust3CheckoutPanel) {
                 if (option == WizardDescriptor.PREVIOUS_OPTION ||
                     option == WizardDescriptor.CANCEL_OPTION) {
                    Cust3CheckoutPanel check = (Cust3CheckoutPanel)iterPanel.current();
                    check.killAllRunning();
                 }
                   
               }
        }
    }
    
/** Checkout panel sets this semafor to let the next panel know that actual checkout hasn't finished yet.
 * The next panel can't go on while wait is on true.
 * @param wait sets tha value of the 'wait' semafor.
 */
    public synchronized void setWait(boolean wait) {
        waitForCheckout = wait;   
    }
    
/** method for the relmount panel to check if the checkout of modules has finished or not.
 * if not, it keeps waiting.
 * @return status of the checkout module command
 */
    public synchronized boolean isWaiting() {
        return waitForCheckout; 
    }
    


}
