/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline.passwd;

import java.util.ArrayList;
import javax.swing.SwingUtilities;

import org.openide.util.RequestProcessor;

import org.netbeans.modules.vcscore.util.VcsUtilities;

import org.netbeans.modules.vcs.cmdline.CvsFileSystem;

/**
 *
 * @author  Martin Entlicher
 * @version
 */
public class LoginDialog extends javax.swing.JDialog {

    public final Object STATUS_CONNECTING = new Integer(0);
    public final Object STATUS_FAILED = new Integer(1);

    private static ArrayList lastSuccessfullLoggings = new ArrayList();

    private String connectStr = "";
    private PasswdEntry entry = null;
    private boolean loggedIn = false;
    private boolean offline = false;
    private String password = null;
    private Thread loginThread = null;
    private CvsFileSystem fileSystem;
    
    /** Creates new form LoginDialog */
    public LoginDialog(java.awt.Frame parent, boolean modal, CvsFileSystem fs) {
        super (parent, modal);
        initComponents ();
        setTitle(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/passwd/Bundle").getString("LoginDialog.title"));
        pack ();
        fileSystem = fs;
        VcsUtilities.removeEnterFromKeymap(passwordField);
        getRootPane().setDefaultButton(loginButton);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        loginPanel = new javax.swing.JPanel();
        loginLabel = new javax.swing.JLabel();
        passwordLabel = new javax.swing.JLabel();
        passwordField = new javax.swing.JPasswordField();
        buttonPanel = new javax.swing.JPanel();
        loginButton = new javax.swing.JButton();
        offlineButton = new javax.swing.JButton();
        statusPanel = new javax.swing.JPanel();
        statusLabel = new javax.swing.JLabel();
        getContentPane().setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        }
        );
        
        loginPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        loginLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/passwd/Bundle").getString("LoginDialog.loginLabel.text"));
          gridBagConstraints2 = new java.awt.GridBagConstraints();
          gridBagConstraints2.gridwidth = 2;
          gridBagConstraints2.insets = new java.awt.Insets(0, 0, 8, 0);
          gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
          loginPanel.add(loginLabel, gridBagConstraints2);
          
          
        passwordLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/passwd/Bundle").getString("LoginDialog.passwordLabel.text"));
          gridBagConstraints2 = new java.awt.GridBagConstraints();
          gridBagConstraints2.gridy = 1;
          gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 8);
          gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
          loginPanel.add(passwordLabel, gridBagConstraints2);
          
          
        gridBagConstraints2 = new java.awt.GridBagConstraints();
          gridBagConstraints2.gridy = 1;
          gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
          gridBagConstraints2.weightx = 1.0;
          loginPanel.add(passwordField, gridBagConstraints2);
          
          
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(8, 8, 8, 8);
        gridBagConstraints1.weightx = 1.0;
        getContentPane().add(loginPanel, gridBagConstraints1);
        
        
        buttonPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        loginButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/passwd/Bundle").getString("LoginDialog.loginButton.text"));
          loginButton.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent evt) {
                  loginButtonActionPerformed(evt);
              }
          }
          );
          gridBagConstraints3 = new java.awt.GridBagConstraints();
          gridBagConstraints3.weightx = 1.0;
          buttonPanel.add(loginButton, gridBagConstraints3);
          
          
        offlineButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/passwd/Bundle").getString("LoginDialog.offlineButton.text"));
          offlineButton.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent evt) {
                  offlineButtonActionPerformed(evt);
              }
          }
          );
          gridBagConstraints3 = new java.awt.GridBagConstraints();
          gridBagConstraints3.weightx = 1.0;
          buttonPanel.add(offlineButton, gridBagConstraints3);
          
          
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 8, 8, 8);
        gridBagConstraints1.weightx = 1.0;
        getContentPane().add(buttonPanel, gridBagConstraints1);
        
        
        statusPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints4;
        statusPanel.setBorder(new javax.swing.border.BevelBorder(1));
        
        statusLabel.setText(" ");
          gridBagConstraints4 = new java.awt.GridBagConstraints();
          gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
          gridBagConstraints4.insets = new java.awt.Insets(0, 8, 0, 8);
          gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
          gridBagConstraints4.weightx = 1.0;
          statusPanel.add(statusLabel, gridBagConstraints4);
          
          
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 8, 8, 8);
        gridBagConstraints1.weightx = 1.0;
        getContentPane().add(statusPanel, gridBagConstraints1);
        
    }//GEN-END:initComponents

  private void offlineButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_offlineButtonActionPerformed
// Add your handling code here:
      loggedIn = false;
      if (loginThread != null) {
          if (loginThread.isAlive()) {
              loginThread.interrupt();
          }
      }
      offline = true;
      closeDialog(null);
  }//GEN-LAST:event_offlineButtonActionPerformed

  private void loginButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loginButtonActionPerformed
// Add your handling code here:
      //loginThread = new Thread("CVS_AUTHORIZING_THREAD") {
      RequestProcessor.postRequest(new Runnable() {
          public void run() {
              CVSPasswd pasFile = new CVSPasswd(fileSystem.getCygwinPath());
              pasFile.loadPassFile();
              password = new String(passwordField.getPassword());
              //entry = new PasswdEntry();
              pasFile.remove(connectStr);
              pasFile.add(connectStr, password); //CVSPasswd.scramble(password));
              pasFile.savePassFile();
              //boolean setRight = entry.setEntry(connectStr + " " + CVSPasswd.scramble(password));
              //if (!setRight) {D("wrongly set entry.");}
              checkLogin();
          }
      });
      //loginThread.start();
  }//GEN-LAST:event_loginButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible (false);
        dispose ();
        fileSystem = null;
    }//GEN-LAST:event_closeDialog

    private boolean wasLoggedIn() {
        return lastSuccessfullLoggings.contains(connectStr);
    }
    
    /**
     * Show the login dialog and start the login process immediately.
     */
    public void showLogging() {
        if (wasLoggedIn()) {
            loggedIn = true; // believe, that the user is still correctly logged in
            return;
        }
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        Runnable login = new Runnable() {
                            public void run() {
                                checkLogin(); // this should be called after show();
                            }
                        };
                        new Thread(login, "CVS_AUTHORIZING_THREAD").start();
                    }
                });
            }
        });
        show();
    }
    
    private void checkLogin() {
        lastSuccessfullLoggings.remove(connectStr);
        passwordField.setEnabled(false);
        loginButton.setEnabled(false);
        setStatus(STATUS_CONNECTING, null);
        StringBuffer message = new StringBuffer();
        try {
            loggedIn = CVSPasswd.checkLogin(fileSystem, message);
        } catch (java.net.UnknownHostException exc) {
            setStatus(org.openide.util.NbBundle.getBundle(LoginDialog.class).getString("LoginDialog.unknownHost"));
            return;
        } catch (java.io.IOException exc) {
            setStatus(org.openide.util.NbBundle.getBundle(LoginDialog.class).getString("LoginDialog.connectionIOError"));
            return;
        } finally {
            passwordField.setEnabled(true);
            loginButton.setEnabled(true);
        }
        if (!loggedIn) {
            setStatus(STATUS_FAILED, message.toString());
        } else {
            lastSuccessfullLoggings.add(connectStr);
            closeDialog(null);
        }
    }
    
    public void setConnectString(String connectStr) {
        this.connectStr = connectStr;
    }
    
    public void setPserverName(String pserverName) {
        loginLabel.setText(java.text.MessageFormat.format(org.openide.util.NbBundle.getBundle(LoginDialog.class).getString("LoginDialog.loginLabel.text"), new Object[] { pserverName }));
        pack();
    }
    
    public void setStatus(Object status) {
        setStatus(status, null);
    }
    
    public void setStatus(Object status, String message) {
        if (status.equals(STATUS_FAILED)) {
            statusLabel.setText(org.openide.util.NbBundle.getBundle(LoginDialog.class).getString("LoginDialog.status.failed"));
            if (message != null) statusLabel.setToolTipText(message);
        } else if (status.equals(STATUS_CONNECTING)) {
            statusLabel.setText(org.openide.util.NbBundle.getBundle(LoginDialog.class).getString("LoginDialog.status.connecting"));
        } else if (status instanceof String) {
            statusLabel.setText((String) status);
        }
    }
    
    public boolean isLoggedIn() {
        return loggedIn;
    }
    
    public boolean isOffline() {
        return offline;
    }
    
    public String getPassword() {
        return password;
    }
    
    public static LoginDialog createDialog(CvsFileSystem fs) {
        LoginDialog dialog = new LoginDialog (new javax.swing.JFrame (), true, fs);
        VcsUtilities.centerWindow(dialog);
        return dialog;
    }
    
    /**
     * @param args the command line arguments
     *
     * public static void main (String args[]) {
     * new LoginDialog (new javax.swing.JFrame (), true, null).show ();
     * }
     */
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel loginPanel;
    private javax.swing.JLabel loginLabel;
    private javax.swing.JLabel passwordLabel;
    private javax.swing.JPasswordField passwordField;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton loginButton;
    private javax.swing.JButton offlineButton;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JLabel statusLabel;
    // End of variables declaration//GEN-END:variables

}
